//Last modified 04.Apr.2014 by M. Pirinen

/* generateR v.0.911. A program for generating an R matrix for program MMM
   that analyses linear mixed models.
   Copyright (C) 2011,2012,2014 Matti Pirinen.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see http://www.gnu.org/licenses/
*/
//to compile:
//gcc generateR.c -o generateR -lgsl -lgslcblas -lm -Wall

/* 
   To print the instructions type on the command line:
   ./generateR 
*/

#define _GNU_SOURCE
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>

#define IBD 1 //if this is 0 then IBS is used
#define ID_COL_GEN_FILE 2 //which column (1 or 2) of gen-file is the snp_id for snp exclusions?
#define NSNP_ID 1000
int nind=0;
double const ind_missing_threshold=0.02;//lists individuals with > missing
double const pairwise_rel_threshold=0.90;//lists pairs of individuals with rel > this threshold

struct PARAMETERS{
  double missing;//snp excluded if proportion of missing individuals > snp_missing_threshold
  double maf;//snp excluded if MAF < MAF_threshold
  double missing_z;
  double missing_gen;
  double info;//snp excluded from GEN-file if info < this
  int z_file;
  int n;
  int output_matrix;//1=prints out matrix, else does not
  int output_list;//1=prints out list, else does not
  int operation;//-1 subtract two lists, 0=compute matrix, 1=add two lists, 2=transforms list to matrix
  FILE *in;
  FILE *in2;
  FILE *out_matrix;
  FILE *out_list;
  FILE *exclusion_file;
  FILE *out_exclusions;
};

void alloc_error(char *str);
int cmpstringp(const void *p1, const void *p2);
int list_operation(struct PARAMETERS *par,gsl_matrix *K, gsl_matrix *N);
void print_instructions();
void parse_arguments(char *argv[],int argc,struct PARAMETERS *par);
char **read_E(FILE *in,int *n);
int read_z(FILE *in,gsl_vector *x,gsl_vector *data_at_locus,double *p,double *prop_missing,int *missing,double missing_z,char *snp_id);
int read_gen(FILE *IN,gsl_vector *x,gsl_vector *data_at_locus,double *p,double *prop_missing,int *missing,double *info,double missing_gen,char *snp_id);
int string_search(char *tag,char **list,int nlist);

int main(int argc, char *argv[]){

  int snp,*missing,i,j,*heterozygote,status,nexcluded_snps,nexclusion_list=0;
  double p,prop_missing,info,t;
  char snp_id[NSNP_ID],**exclusion_list=NULL;
  gsl_vector *x,*data_at_locus;
  gsl_matrix *K,*N;
  struct PARAMETERS par;

  par.exclusion_file=NULL;par.in=NULL;par.in2=NULL;par.out_matrix=NULL;par.out_list=NULL;

  if(argc < 2){
    print_instructions();
    exit(1);
  }

  parse_arguments(argv,argc,&par);
  nind=par.n;
  K=gsl_matrix_calloc(nind,nind);
  N=gsl_matrix_calloc(nind,nind);

  if(par.operation!=0){//do addition, subtraction or list->matrix transformation
    status=list_operation(&par,K,N);
  }
  else{//do matrix calculation from z-file or gen-file
    if(par.exclusion_file!=NULL){
      exclusion_list=read_E(par.exclusion_file,&nexclusion_list);
      fprintf(stdout,"read %d ids for exclusions.\n\n",nexclusion_list);
    }

    x=gsl_vector_alloc(nind);
    data_at_locus=gsl_vector_alloc(nind);

    if((missing=(int *) calloc(nind,sizeof(int)))==NULL)
      alloc_error("missing");
    
    if((heterozygote=(int *) calloc(nind,sizeof(int)))==NULL)
      alloc_error("heterozygote");
    
    snp=0;
    nexcluded_snps=0;
    info=1.0; //if z-file is used then this is fixed to 1.0
    while(!feof(par.in)){
      
      if(par.z_file) status=read_z(par.in,x,data_at_locus,&p,&prop_missing,missing,par.missing_z,snp_id);
      else status=read_gen(par.in,x,data_at_locus,&p,&prop_missing,missing,&info,par.missing_gen,snp_id);
      
      if(status==2) break;//end of file 
      if(status==0 || string_search(snp_id,exclusion_list,nexclusion_list) || 
	 prop_missing>par.missing ||  fabs(1.0-p)<par.maf || fabs(p)<par.maf || info<par.info){
	fprintf(par.out_exclusions,"%s\n",snp_id);
	++nexcluded_snps;
      }
      else{
	
#if IBD == 1

	for(i=0;i<nind;++i)
	  if(gsl_vector_get(data_at_locus,i)==1) 
	    gsl_vector_set(x,i,gsl_vector_get(x,i)-2*p);//mean center non-missing genotypes
	//could this be done with BLAS: x=x-2*p*data_at_locus ?

	gsl_blas_dsyr (CblasLower, 1/(2*p*(1-p)), x, K);//K=K+x x^T /(2*p*(1-p)) for diploid data
	//gsl_blas_dsyr (CblasLower, 1/(2*p*(1-2*p)), x, K);//use this if z_file contains haploid data
	gsl_blas_dsyr (CblasLower, 1.0, data_at_locus, N);//N=N+data_at_locus * (data_at_locus)^T 
	
#else
	
	{int nhetero=0;
	  for(i=0;i<nind;++i){
	    if(gsl_vector_get(x,i)==1)
	      heterozygote[nhetero++]=i;
	    if(gsl_vector_get(x,i)<0) //missing genotype
	      gsl_vector_set(x,i,0.0);
	    else
	      gsl_vector_set(x,i,gsl_vector_get(x,i)-1.0);
	  }
	  gsl_blas_dsyr (CblasLower, 1.0, x, K);//K=K+ 1.0* x x^T
	  gsl_blas_dsyr (CblasLower, 1.0, data_at_locus, N);//N=N+data_at_locus (data_at_locus)^T 
	  
	  for(i=0;i<nhetero;++i){//this is to make pairs of heterozygote share as much as pairs of homozygotes
	    for(j=0;j<=i;++j){
	      gsl_matrix_set(K,heterozygote[i],heterozygote[j],gsl_matrix_get(K,heterozygote[i],heterozygote[j])+1.0);
	      if(j<i)
		gsl_matrix_set(K,heterozygote[j],heterozygote[i],gsl_matrix_get(K,heterozygote[j],heterozygote[i])+1.0);
	    }
	  }
	}
#endif
      }
      if((snp++)%1000==999) 
	printf("%d snps read, %d excluded.\n",snp,nexcluded_snps);
      if(isnan(gsl_matrix_get(K,0,0))){fprintf(stderr,"SNP:%d %s gives 'nan'.\n",snp,snp_id);exit(100);}
    }
    fclose(par.in);//is this ok even if this is stdin? 
    
    for(i=0;i<nind;++i){
      for(j=0;j<=i;++j){
#if IBD == 1
	gsl_matrix_set(K,i,j,gsl_matrix_get(K,i,j)/gsl_matrix_get(N,i,j));
#else
	gsl_matrix_set(K,i,j,0.5+0.5*gsl_matrix_get(K,i,j)/gsl_matrix_get(N,i,j));
#endif
      }
    }

    printf("\nPrinting individuals whose missingness is over %g\n",ind_missing_threshold);
    for(i=0;i<nind;++i){
      if(1.0*missing[i]/snp>ind_missing_threshold){
	printf("%d: %lf\n",i+1,1.0*missing[i]/snp);
      }
    }
    gsl_vector_free(x);
    gsl_vector_free(data_at_locus);
    free(missing);
    free(heterozygote);
  }
  
  printf("\nPrinting the pairs that are related over threshold %g:\nIND1 IND2 relatedness\n",pairwise_rel_threshold);
  if(par.output_matrix){
    for(i=0;i<nind;++i){
      for(j=0;j<=i;++j)
	fprintf(par.out_matrix,"%.5g ",gsl_matrix_get(K,i,j));
      for(j=i+1;j<nind;++j){
	t=gsl_matrix_get(K,j,i);
	fprintf(par.out_matrix,"%.5g ",t);
	if(t>pairwise_rel_threshold)
	  printf("%d %d %g\n",i+1,j+1,t);
      }
      fprintf(par.out_matrix,"\n");
    }
    fclose(par.out_matrix);
  }
  if(par.output_list){
    for(i=0;i<nind;++i){
      for(j=0;j<=i;++j){
	fprintf(par.out_list,"%d %d %d %.10g\n",i+1,j+1,(int) gsl_matrix_get(N,i,j),gsl_matrix_get(K,i,j));
	if(j<i && gsl_matrix_get(K,i,j)>pairwise_rel_threshold)
	  printf("%d %d %g\n",i+1,j+1,gsl_matrix_get(K,i,j));
      }
    }
    fclose(par.out_list);
  }
  gsl_matrix_free(K);
  gsl_matrix_free(N);

  return 0;
}


void print_instructions(){
  printf("This is generateR v.0.911,\na program to generate an R matrix for the program MMM that analyses linear mixed models.\n");
  printf("Copyright Matti Pirinen, 2011,2012,2014. matti.pirinen@iki.fi\n\n");
  printf("Run with command:\n./generateR [OPTION VALUE]...\nWhere OPTIONS and their VALUES are:\nOPTION\t\tVALUE\n");
  printf("-n\t\tnumber of individuals (REQUIRED)\n");
  printf("-z_file\t\tpath to z-file (EITHER z-file OR gen_file REQUIRED)\n");
  printf("-gen_file\tpath to gen-file (EITHER z-file OR gen_file REQUIRED)\n");
  printf("-out_file\tpath to output R-file (EITHER output matrix or list REQUIRED)\n");
  printf("-out_matrix\tpath to output matrix file (EITHER output matrix or list REQUIRED)\n");
  printf("-exclusion_file\tpath to file of ids of excluded zs/gens\n");
  printf("-missing_gen\tthreshold for missingness in gen file\n");
  printf("-missing_z\tvalue for missing data in z file\n");
  printf("-maf\t\tminor allele frequency threshold (default 0.0)\n");
  printf("-missing\tmissingness threshold per single z or SNP (default 1.0)\n");
  printf("-info\t\tinfo threshold for gen-files (default 0.0)\n\n");
  printf("It is also possible to do the following operations on one or two R-files.\n");
  printf("In that case no z-file/gen-file can be specified, but at least one output file must be specified.\n");
  printf("-add\t\ttwo paths to R-files\n");
  printf("-subtract\ttwo paths to R-files\n");
  printf("-to_matrix\tpath to R-file\n\n");
  printf("For more details, see the User manual of the program MMM.\n\n");
}


void parse_arguments(char *argv[],int argc,struct PARAMETERS *par){
  
  int arg=1;
  char tag[100],out_exclusions[1000];
  char *in_file,*in2_file,*out_matrix,*out_list,*exclusion_file;
  char 
    found=0,n_found=0,z_file_found=0,gen_file_found=0,out_matrix_found=0,
    out_file_found=0,exclusion_file_found=0,info_found=0,
    missing_gen_found=0,missing_z_found=0,maf_found=0,missing_found=0,
    add_found=0,subtract_found=0,to_matrix_found=0;

  while(arg<argc){

    found=0;
    sprintf(tag,"-n");
    if(strcmp(tag,argv[arg])==0){
      if(missing_gen_found){fprintf(stderr,"ERROR: option -n found twice\n");exit(1);}
      par->n=atoi(argv[arg+1]);
      if(par->n <1 || par->n >50000){
	fprintf(stderr,"ERROR: n must be between 1 and 50000.\n");exit(1);}
      n_found=1;found=1;
    }
    sprintf(tag,"-z_file");
    if(strcmp(tag,argv[arg])==0){
      if(z_file_found){fprintf(stderr,"ERROR: option -z_file found twice\n");exit(1);}
      par->z_file=1;
      in_file=argv[arg+1];
      z_file_found=1;found=1;
    }
    sprintf(tag,"-gen_file");
    if(strcmp(tag,argv[arg])==0){
      if(gen_file_found){fprintf(stderr,"ERROR: option -gen_file found twice\n");exit(1);}
      par->z_file=0;
      in_file=argv[arg+1];
      gen_file_found=1;found=1;
    }
    sprintf(tag,"-out_matrix");
    if(strcmp(tag,argv[arg])==0){
      if(out_matrix_found){fprintf(stderr,"ERROR: option -out_matrix found twice\n");exit(1);}
      out_matrix=argv[arg+1];
      out_matrix_found=1;found=1;
    }
    sprintf(tag,"-out_file");
    if(strcmp(tag,argv[arg])==0){
      if(out_file_found){fprintf(stderr,"ERROR: option -out_file found twice\n");exit(1);}
      out_list=argv[arg+1];
      out_file_found=1;found=1;
    }
    sprintf(tag,"-exclusion_file");
    if(strcmp(tag,argv[arg])==0){
      if(exclusion_file_found){fprintf(stderr,"ERROR: option -exclusion_file found twice\n");exit(1);}
      exclusion_file=argv[arg+1];
      exclusion_file_found=1;found=1;
    }
    sprintf(tag,"-missing_gen");
    if(strcmp(tag,argv[arg])==0){
      if(missing_gen_found){fprintf(stderr,"ERROR: option -missing_gen found twice\n");exit(1);}
      par->missing_gen=atof(argv[arg+1]);
      if(par->missing_gen>1.0 || par->missing_gen<0.0){
	fprintf(stderr,"ERROR: missing_gen must be between 0.0 and 1.0.\n");exit(1);}
      missing_gen_found=1;found=1;
    }
    sprintf(tag,"-missing_z");
    if(strcmp(tag,argv[arg])==0){
      if(missing_z_found){fprintf(stderr,"ERROR: option -missing_z found twice\n");exit(1);}
      par->missing_z=atof(argv[arg+1]);
      missing_z_found=1;found=1;
    }
    sprintf(tag,"-maf");
    if(strcmp(tag,argv[arg])==0){
      if(maf_found){fprintf(stderr,"ERROR: option -maf found twice\n");exit(1);}
      par->maf=atof(argv[arg+1]);
      if(par->maf>1.0 || par->maf<0.0){
	fprintf(stderr,"ERROR: maf must be between 0.0 and 1.0.\n");exit(1);}
      maf_found=1;found=1;
    }
    sprintf(tag,"-info");
    if(strcmp(tag,argv[arg])==0){
      if(info_found){fprintf(stderr,"ERROR: option -info found twice\n");exit(1);}
      par->info=atof(argv[arg+1]);
      if(par->info>1.0){
	fprintf(stderr,"ERROR: info must be at most 1.0.\n");exit(1);}
      info_found=1;found=1;
    }
    sprintf(tag,"-missing");
    if(strcmp(tag,argv[arg])==0){
      if(missing_found){fprintf(stderr,"ERROR: option -missing found twice\n");exit(1);}
      par->missing=atof(argv[arg+1]);
      if(par->missing>1.0 || par->missing<0.0){
	fprintf(stderr,"ERROR: missing must be between 0.0 and 1.0.\n");exit(1);}
      missing_found=1;found=1;
    }
    sprintf(tag,"-add");
    if(strcmp(tag,argv[arg])==0){
      if(add_found){fprintf(stderr,"ERROR: option -add found twice\n");exit(1);}
      in_file=argv[++arg];in2_file=argv[arg+1];  
      add_found=1;found=1;
    }
    sprintf(tag,"-subtract");
    if(strcmp(tag,argv[arg])==0){
      if(subtract_found){fprintf(stderr,"ERROR: option -subtract found twice\n");exit(1);}
      in_file=argv[++arg];in2_file=argv[arg+1];  
      subtract_found=1;found=1;
    }
    sprintf(tag,"-to_matrix");
    if(strcmp(tag,argv[arg])==0){
      if(to_matrix_found){fprintf(stderr,"ERROR: option -to_matrix found twice\n");exit(1);}
      in_file=argv[arg+1];
      to_matrix_found=1;found=1;
    }
    if(!found){
      fprintf(stderr,"ERROR option %s not valid.\n",argv[arg]);
      exit(1);
    }
    arg+=2;
  }
  
  if(!n_found){
    fprintf(stderr,"ERROR Number of individuals must be specified. Use option -n 'number of individuals'.\n");exit(1);}
  if(add_found+subtract_found+to_matrix_found>1){
    fprintf(stderr,"ERROR only one of -add or -subtract or -to_matrix can be specified.\n"); exit(1);}
  if(add_found) par->operation=1;
  if(subtract_found) par->operation=-1;
  if(to_matrix_found) par->operation=2;
  if(!add_found && !subtract_found && !to_matrix_found) par->operation=0;

  if(par->operation!=0 && (z_file_found || gen_file_found)){
    fprintf(stderr,"ERROR Neither z_file nor gen_file can be specified when -add or -subtract are in use.\n");exit(1);}
  if(par->operation==0 && z_file_found && gen_file_found){
    fprintf(stderr,"ERROR Either z_file or gen_file must be specified. Not both.\n");exit(1);}
  if(par->operation==0 && !z_file_found && !gen_file_found){
    fprintf(stderr,"ERROR Either z_file or gen_file must be specified.\nUse option -z_file 'filename' or -gen_file 'filename'.\n");exit(1);}

  if(!out_matrix_found && !out_file_found){
    fprintf(stderr,"ERROR at least one output file must be specified. Use option -out_matrix 'filename' or -out_file 'filename'.\n");exit(1);}
  par->output_matrix=out_matrix_found;
  par->output_list=out_file_found;

  if(!missing_found) par->missing=1.0;
  if(!maf_found) par->maf=0.0;
  if(!info_found) par->info=0.0;
  if(gen_file_found && !missing_gen_found) par->missing_gen=0.1;
  if(z_file_found && !missing_z_found) par->missing_z=log(-1.0);//NAN
  
  if(in_file[0]=='-' && in_file[1]=='\0') par->in=stdin;
  else{
    if((par->in=fopen(in_file,"r"))==NULL){
      fprintf(stderr,"Can't open input file %s\n",in_file);
      exit(1);
    }
  }
  if(par->operation!=0 && par->operation !=2){
    if(in2_file[0]=='-' && in2_file[1]=='\0') {
      if(in_file[0]=='-' && in_file[1]=='\0'){
	fprintf(stderr,"ERROR both files cannot be read from stdin.\n");exit(2);}
      par->in2=stdin;
    }else{
      if((par->in2=fopen(in2_file,"r"))==NULL){
	fprintf(stderr,"Can't open second input file %s\n",in2_file);
	exit(1);
      }
    }
  }
  if(out_matrix_found && (par->out_matrix=fopen(out_matrix,"w"))==NULL){
    fprintf(stderr,"Can't open output matrix file %s for writing.\n",out_matrix);
    exit(1);
  }
  if(out_file_found && (par->out_list=fopen(out_list,"w"))==NULL){
    fprintf(stderr,"Can't open output list file %s for writing.\n",out_list);
    exit(1);
  }
  if(exclusion_file_found && (par->exclusion_file=fopen(exclusion_file,"r"))==NULL){
    fprintf(stderr,"Can't open exclusion file %s for reading.\n",exclusion_file);
    exit(1);
  }
  sprintf(out_exclusions,"%s_exclusions.out",in_file);
  if((par->out_exclusions=fopen(out_exclusions,"w"))==NULL){
    fprintf(stderr,"Can't open exclusion output file %s for writing.\n",out_exclusions);
    exit(1);
  }

  printf("\nThis is generateR,\na program to generate an R matrix for program MMM that analyses linear mixed models.\n");
  printf("Copyright Matti Pirinen, 2011, 2012. matti.pirinen@iki.fi\n\n");
  if(par->operation==0){
    printf("Reads data for %d individuals from ",par->n);

    if(par->z_file) printf("z-file ");
    else printf("gen-file ");
    //if(in_file[0]=='-' && in_file[1]=='\0') printf("stdin\n"); else 
    printf("'%s'\n",in_file);

    if(par->output_matrix) printf("Writes matrix to '%s'\n",out_matrix);
    if(par->output_list) printf("Writes R-file to '%s'\n",out_list);
    printf("Parameters:\n");
    if(par->exclusion_file!=NULL) printf("exclusion file=%s\n",exclusion_file);
    if(par->z_file) printf("missing_z=%g\n",par->missing_z);
    else printf("missing_gen=%g\n",par->missing_gen);
    printf("maf=%g\n",par->maf);
    if(!par->z_file) printf("info=%g\n",par->info);
    printf("missing=%g\n\n",par->missing);
    
    if(z_file_found && fabs(par->missing_z)<1e-9){
      printf("WARNING missing_z was set to 0.0.\nWas this on purpose or was it because no valid numerical value was read in?\n");
      printf("Check your command line parameter following -missing_z !\n");
    }
  }else{
    if(par->operation!=2) printf("Reads data for %d individuals from list files '%s' and '%s'\n",par->n,in_file,in2_file);
    else printf("Reads data for %d individuals from list file '%s'\n",par->n,in_file);
    if(par->operation==1) printf("Adds the lists.\n");
    if(par->operation==-1) printf("Subtracts the lists.\n");
    if(par->operation==2) printf("Transforms the list to matrix.\n");
    if(par->output_matrix) printf("Writes output matrix to '%s'\n",out_matrix);
    if(par->output_list) printf("Writes R-file to '%s'\n",out_list);
  }
 
}

int list_operation(struct PARAMETERS *par,gsl_matrix *K, gsl_matrix *N){

  int i,j,nline,ii[2],jj[2],ind,m[2],nfiles;
  int n=par->n;
  FILE *in[2];
  double r[2];

  if(par->operation==2) nfiles=1; //operation 2, transform list to matrix
  else nfiles=2; //operation 1 or -1, resp, add or subtract

  in[0]=par->in;
  in[1]=par->in2;
  nline=1;
  for(i=0;i<n;++i){
    for(j=0;j<=i;++j){
      for(ind=0;ind<nfiles;++ind){
	if((fscanf(in[ind],"%d %d %d %lf\n",ii+ind,jj+ind,m+ind,r+ind)!=4)){
	  fprintf(stderr,"ERROR reading file %d at line %d.\nExits!\n",ind+1,nline);
	  exit(1);
	}
	if(ii[ind]!=i+1 || jj[ind]!=j+1){
	  fprintf(stderr,"ERROR reading file %d at line %d.\n",ind+1,nline);
	  fprintf(stderr,"Indexes %d %d do not match expected: %d %d.\n",ii[ind],jj[ind],i+1,j+1);
	  exit(1);
	}
	if(m[ind]<1){
	  fprintf(stderr,"ERROR reading file %d at line %d.\nWeight (3rd col) is %d<1.\nExits!",ind+1,nline,m[ind]);
	  exit(1);
	}
      }
      if(nfiles==1) {
	gsl_matrix_set(K,i,j,r[0]);
	gsl_matrix_set(N,i,j,m[0]);
      }
      else {
	gsl_matrix_set(K,i,j,(m[0]*r[0]+(par->operation)*m[1]*r[1])/(m[0]+(par->operation)*m[1]));
	gsl_matrix_set(N,i,j,m[0]+(par->operation)*m[1]);
      }
      if(gsl_matrix_get(N,i,j)<1){
	fprintf(stderr,"ERROR reading line %d.\nWeight after subtraction is %d-%d=%lf<1.\nExits!",nline,m[0],m[1],gsl_matrix_get(N,i,j));
	exit(1);
      }
      ++nline;
    }
  }
  return 1;
}


int read_z(FILE *in,gsl_vector *x,gsl_vector *data_at_locus,double *p,double *prop_missing,int *missing,double missing_z,char *snp_id){

  int nmissing,i,nchars;
  size_t size;
  char *line;
  double z_sum,g;
  FILE *line_str;

  size=nind*3;
  if((line=(char*) malloc(size*sizeof(char)))==NULL)
    alloc_error("line in read_z");

  nchars=getline(&line,&size,in);
  line_str=fmemopen(line,nchars*sizeof(char),"r");

  if(nchars<1 || fscanf(line_str,"%s",snp_id)!=1){
    free(line);
    if(feof(in))
      return 2;
    else{
      fclose(line_str);
      return 0;
    }
  }
  
  nmissing=0;
  z_sum=0;

  for(i=0;i<nind;++i){
    if(fscanf(line_str,"%lf",&g)!=1){
      fprintf(stderr,"ERROR at z_id %s with individual %d. Excluding z.\n",snp_id,i+1);
      free(line);
      fclose(line_str);
      return 0; //exclude the snp
    }
    if(fabs(g-missing_z)<1e-9){//missing data 
      ++missing[i];
      gsl_vector_set(x,i,0.0);//set to zero, then doesn't affect further calculations
      gsl_vector_set(data_at_locus,i,0);
      nmissing++;
    }
    else{
      gsl_vector_set(x,i,g);
      gsl_vector_set(data_at_locus,i,1);
      z_sum+=g;
    }
  }

  *p=0.5*z_sum/(nind-nmissing);
  *prop_missing=1.0*nmissing/nind;
  fclose(line_str);free(line);

  return 1;

}


int read_gen(FILE *in,gsl_vector *x,gsl_vector *data_at_locus,double *p,double *prop_missing,int *missing,double *info,double missing_gen,char *snp_id){


  /*GEN file format:
    one line per snp:
    id1 id2 position alleleA alleleB p1_AA p1_AB p1_BB p2_AA p2_AB p2_BB ... pn_AA pn_AB pn_BB
    where pi_AA is the probability that individual i has genotype AA

    returns 
    1 if successful
    2 if file ends
    0 if error reading values 
  */

  int i,a,nchars,nmissing;
  size_t size;
  double geno[3],geno_sum,counts[4],var_info,frq,ind_geno_sum,coeff_frq,coeff_frq2,expect;
  double coeff_frq_all,coeff_frq2_all,sum_missing_all,var_info_all;
  char *line,*gen_info;
  FILE *line_str;

  size=nind*8;
  if((line=(char*) malloc(size*sizeof(char)))==NULL)
    alloc_error("line in read_gen");

  nchars=getline(&line,&size,in);
  line_str=fmemopen(line,nchars*sizeof(char),"r");

  if(nchars<1 || fscanf(line_str,"%as",&gen_info)!=1){
    free(line);
    if(feof(in))
      return 2;
    else{
      fclose(line_str);
      return 0;
    }
  }

  if(ID_COL_GEN_FILE==1){
    strncpy(snp_id, gen_info,NSNP_ID);snp_id[NSNP_ID-1]='\0';} //do not exceed max size of snp_id
  //sprintf(snp_id,"%s",gen_info);//1st column is snp_id
  free(gen_info);
  

  if(fscanf(line_str,"%as",&gen_info)!=1){
    fclose(line_str);free(line);
    return 0;}
  if(ID_COL_GEN_FILE!=1) {
    strncpy(snp_id, gen_info,NSNP_ID);snp_id[NSNP_ID-1]='\0';} //do not exceed max size of snp_id
    //sprintf(snp_id,"%s",gen_info);//2nd column is snp_id
  free(gen_info);

  for(i=2;i<5;++i){
    if(fscanf(line_str,"%as",&gen_info)!=1){//pos allele_A allele_B
      free(line);
      fclose(line_str);
      return 0;
    }
    free(gen_info);
  }

  geno_sum=0.0; 
  nmissing=0;
  sum_missing_all=0.0;
  var_info=var_info_all=0.0;
  coeff_frq=coeff_frq2=0.0;
  coeff_frq_all=coeff_frq2_all=0.0;
  for(i=0;i<4;++i) counts[i]=0.0;
  
  for(i=0;i<nind;++i){
    for(a=0;a<3;++a)
      if(fscanf(line_str,"%lf",geno+a)!=1 || geno[a]<0.0 || geno[a]>1.0){
	fprintf(stderr,"reading gen:at id_%d='%s'. ERROR with probability of genotype %d of individual %d (i.e. element %d of that line of the gen file)\n",ID_COL_GEN_FILE,snp_id,a,i+1,5+3*i+a+1);
	free(line);
	return 0;
      }
  
    ind_geno_sum=geno[0]+geno[1]+geno[2];
    if(ind_geno_sum>1.01){
      fprintf(stderr,"reading gen:at id_%d='%s'. ERROR sum of genotype probabilities of individual %d is %g+%g+%g=%g > 1.0\n",
	      ID_COL_GEN_FILE,snp_id,i+1,geno[0],geno[1],geno[2],ind_geno_sum);
      free(line);
      return 0;
    }

    if(ind_geno_sum<missing_gen){
      gsl_vector_set(x,i,0.0);//set to zero, then doesn't affect further relatedness calculations
      gsl_vector_set(data_at_locus,i,0);
      ++missing[i];
      nmissing++;
      if(ind_geno_sum>0.0) expect=(geno[1]+2.0*geno[2])/ind_geno_sum;
      else expect=0.0; //value does not matter
      var_info_all+=(geno[1]+4*geno[2])-ind_geno_sum*expect*expect
	+ind_geno_sum*(1-ind_geno_sum)*expect*expect;//part of info that does not involve pop frequency
      coeff_frq_all-=4*expect*ind_geno_sum*(1-ind_geno_sum);//coeff of pop allele freq
      coeff_frq2_all+=4*ind_geno_sum*(1-ind_geno_sum);//coeff of pop allele freq^2
      sum_missing_all+=1-ind_geno_sum;
    }else{
      gsl_vector_set(x,i,(geno[1]+2.0*geno[2])/ind_geno_sum);
      gsl_vector_set(data_at_locus,i,1);
      if(ind_geno_sum>0.0) expect=(geno[1]+2.0*geno[2])/ind_geno_sum;
      else expect=0.0; //value does not matter
      geno_sum+=expect;
      var_info+=(geno[1]+4*geno[2])-ind_geno_sum*expect*expect
	+ind_geno_sum*(1-ind_geno_sum)*expect*expect;//part of info that does not involve pop frequency
      coeff_frq-=4*expect*ind_geno_sum*(1-ind_geno_sum);//coeff of pop allele freq
      coeff_frq2+=4*ind_geno_sum*(1-ind_geno_sum);//coeff of pop allele freq^2
      for(a=0;a<3;++a) counts[a]+=geno[a];
      counts[3]+=1-ind_geno_sum;//coeff of HW variance 
    }
  }
  
  geno_sum/=nind;
  
  sum_missing_all+=counts[3];
  var_info_all+=var_info;
  coeff_frq_all+=coeff_frq;
  coeff_frq2_all+=coeff_frq2;
  
  frq=(0.5*counts[1]+counts[2])/(nind-nmissing-counts[3]);
  //currently var_info can be used as QC criteria
  var_info+=counts[3]*2*frq*(1-frq)+coeff_frq*frq+coeff_frq2*frq*frq;
  var_info=1-var_info/((nind-nmissing)*2.0*frq*(1-frq));
  //var_info_all is not used
  var_info_all+=sum_missing_all*2*frq*(1-frq)+coeff_frq_all*frq+coeff_frq2_all*frq*frq;
  var_info_all=1-var_info_all/(nind*2.0*frq*(1-frq));
  
  *p=frq;
  *prop_missing=1.0*nmissing/nind;
  *info=var_info;

  fclose(line_str);free(line);
  return 1; 
}




void alloc_error(char *str)
{
  fprintf(stderr,"Can't allocate memory: %s\nExits!\n",str);
  exit(2);
}

int string_search(char *tag,char **list,int nlist){
  //binary search from ordered list of strings
  int i,left,right,cmp;

  if(nlist<1) return 0;
  left=0;right=nlist-1;
  if(strcmp(tag,list[left])==0 || strcmp(tag,list[right])==0)
    return 1;
  while(right-left>1){
    i=left+(right-left)/2;
    cmp=strcmp(tag,list[i]);
    if(cmp==0) return 1; //found
    if(cmp<0)
      right=i;
    else 
      left=i;
  }
  return 0; //not found
}

char **read_E(FILE *in,int *n){

  char **list;
  int i,list_size=100,id_size=1000;

  *n=0;
  if((list=(char**) malloc((list_size)*sizeof(char*)))==NULL)
    alloc_error("list in read_E");
  for(i=0;i<list_size;++i)
    if((list[i]=(char*) malloc(id_size*sizeof(char)))==NULL)
      alloc_error("list in read_E");
  
  while(!feof(in)){
    if(*n==list_size){
      if((list=(char**) realloc(list,2*(list_size)*sizeof(char*)))==NULL)
	alloc_error("list in read_E");
      for(i=list_size;i<2*list_size;++i)
	if((list[i]=(char*) malloc(id_size*sizeof(char)))==NULL)
	  alloc_error("list in read_E");
      list_size *= 2;
    }
    if(fscanf(in,"%s",list[*n])!=1)
      break;
    else
      (*n)++;
  }
  for(i=(*n);i<list_size;++i)
    free(list[i]);
  if(*n>0){
    if((list=(char**) realloc(list,(*n)*sizeof(char*)))==NULL)
      alloc_error("list in read_E");
    qsort(list, *n, sizeof(char*),cmpstringp);
  }
  return list;

}

int cmpstringp(const void *p1, const void *p2){
  //taken from 'man qsort'
  /* The actual arguments to this function are "pointers to
     pointers to char", but strcmp(3) arguments are "pointers
     to char", hence the following cast plus dereference */
  return strcmp(* (char * const *) p1, * (char * const *) p2);
}

