//Last modified 19.September.2012 by M. Pirinen

/* MMM v.1.01 A program for analysing a linear mixed model.
    Copyright (C) 2011,2012 Matti Pirinen.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//to compile with gsl and LAPACK and SCORE_TEST_ETA==1:
//gcc qfc.c MMM_1.01.c -o MMM -lm -lgfortran -lblas -llapack -lgsl -lgslcblas

//to compile with gsl and LAPACK (but without score test)
//gcc MMM_1.01.c -o MMM -lm -lgfortran -lblas -llapack -lgsl -lgslcblas

//if LAPACK doesn't work then set below USE_LAPACK 0 and
//gcc qfc.c MMM_1.01.c -o MMM -lm -lgsl -lgslcblas
//This is much slower for large matrices than the LAPACK version.


//to compile on (my) MAC:
//gcc qfc.c MMM_1.01.c -o MMM  -lm -Wall -lgsl -lgslcblas -llapack -I/opt/local/var/macports/software/gsl/1.13_1+darwin_i386/opt/local/include/ -L/opt/local/var/macports/software/gsl/1.13_1+darwin_i386/opt/local/lib/

#define _GNU_SOURCE
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_eigen.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_sf_gamma.h>
#include <gsl/gsl_sf_result.h>
#include <gsl/gsl_integration.h>


#define LOG2PI (1.83787706641)
#define ECHO_INPUT 0
#define EPSILON (1e-16)  //to avoid division by numbers smaller than this
#define MIN_D_TOLERANCE 0.01 //if min eigenvalues less than this, constrains h<=0.99.
#define TAG_SIZE 100 //max length of tags in parameter file or command line
#define TAG_S "%99s" //to read tag, keep one less than TAG_SIZE
#define SCORE_TEST_ETA 1 //if ==1 does score test for eta==0 when no gen-file/z-file is given
#define READ_MANY_PHENOTYPES 0 //not documented, so keep at 0
#define USE_LAPACK 1 //if this is 0 then GSL is used for matrix decompositions, which is VERY slow compared to LAPACK
#define VERSION 1.01


struct PARAMETERS_STRUCT{//these are parameters read from parameters file
  int n_groups,n_covariates,max_iterations,max_eta_iterations,max_eta_intervals,
    max_eta_divisions,n_integr_intervals,n_good_ind,n_total_ind;
  double tol,fixed_eta,missing_z,missing_gen,min_d;
  int logOR,bayesian,check_geno,mean_center,fix_eta,read_R,read_E,read_z,read_gen,save_U_D,model_missing,impute_missing;
  char *A_file,*E_file,*z_file,*id,*outcome,
    *R_file,*U_file,*D_file,*prior_file,*results_file,*inds_out_file;
  char **covariates;
  gsl_vector *prior_exp;
  gsl_matrix *prior_var;
  double prior_a,prior_d,prior_beta_0,prior_beta_1;
};
typedef struct PARAMETERS_STRUCT PAR;

struct F_PARAMETERS_STRUCT{//these are parameters for likelihood function calculations
  gsl_vector *d;
  gsl_vector *Uy_UXbeta2;
  double *var0;
  int nind;
};
typedef struct F_PARAMETERS_STRUCT FPAR;

struct F_MLK_AT_H_PARAMS{
  double base_const;
  int log_scale;
  int use_beta_prior;
  gsl_vector *Uy;
  gsl_matrix *UX;
  gsl_vector *d;
  int ncovariates;
  int nind;
  PAR *par;
};

//these functions are defined at the bottom of this file
void alloc_error(char *str);
void compute_UX_and_Uy_and_d(gsl_matrix **X_full,gsl_vector **y_full,gsl_matrix **U_full,gsl_vector **d_full,gsl_matrix **UX_full,gsl_vector **Uy_full,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,int *groupsizes,int *relatedness_in_groups);
int cmpstringp(const void *p1, const void *p2);
void compare_data_and_prior_var(gsl_vector **y_full,int *groupsizes,int ngroups,double prior_a,double prior_d);
int compute_mlk(double *aa_const,double *logdet_theta,double *logbeta,gsl_vector **UX_theta_Uy,gsl_matrix **UX_theta_UX,gsl_vector **theta_Uy,gsl_matrix **theta_UX,gsl_matrix *inv_prior_var,gsl_matrix *UX,gsl_vector *d,double logmlk_h0[2],double logmlk[2],int ncovariates,int nind_full,double min_d,PAR *par);
double compute_log10BF_eta_qags(gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,int ncovariates,int nind_full,PAR *par);
double compute_mlk_at_h(double h,void *p);
int compute_var_estimates(gsl_vector *beta,double var0,double h,int include_snp,double **var_estimates,gsl_vector *Uy_UXbeta2,gsl_matrix *UX,gsl_vector *Uy,gsl_vector *d,int nind,FILE *out_var,char *snp_id,int estimate_OR);
int compute_var_estimates_without_h(gsl_vector *beta,double var0,int include_snp,double **var_estimates,gsl_vector *Uy_UXbeta2,gsl_matrix *UX,FILE *out_var,char *snp_id,int estimate_OR);
double davies_cdf(double x,gsl_vector *d);
void decompose_S(gsl_matrix **S,int *groupsizes,int *relatedness_in_groups,gsl_matrix **U,gsl_vector **d,PAR *par);
void decompose_S_lapack(double **S,int *groupsizes,int *relatedness_in_groups,gsl_matrix **U,gsl_vector **d,PAR *par);
double dLk_dh(double h, void * par);
double d2Lk_dh2(double h, void * par);
void dLk_d2Lk_dh(double h, void * par,double *d1,double *d2);
void eigen(double *A, int dim, int nvecs, int values_only, double *evals, double *evecs);
int initialize_integration_variables(double *aa_const,double *logdet_theta,double *logbeta,gsl_vector **UX_theta_Uy,gsl_matrix **UX_theta_UX,gsl_vector **theta_Uy,gsl_matrix **theta_UX,gsl_matrix *inv_prior_var,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,double logmlk[2],double logmlk_h0[2],double *logdet_prior_ratio_1to0,int ncovariates,int nind_full,double current_min_d,PAR *par);
int initialize_params(int include_last_beta,gsl_vector *beta,double *var0,double *h,gsl_matrix *UX, gsl_vector *Uy,gsl_vector *Uy_UXbeta2,int nind);
double loglkhood(gsl_vector *Uy,gsl_matrix *UX,gsl_vector *beta,double var0,double h,gsl_vector *d,int nind);
void  make_data_structures(gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,gsl_matrix **X_full,gsl_vector **y_full,gsl_matrix **U_full,gsl_matrix **UX_full,gsl_vector **Uy_full,gsl_vector **d_full,gsl_matrix **S_full,int nind,int *nexcluded_from_groups,int **excluded_from_groups,int *groupsizes,int *relatedness_in_groups);
void  make_data_structures_LAPACK(gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,gsl_matrix **X_full,gsl_vector **y_full,gsl_matrix **U_full,gsl_matrix **UX_full,gsl_vector **Uy_full,gsl_vector **d_full,double **S_lapack,int nind,int *nexcluded_from_groups,int **excluded_from_groups,int *groupsizes,int *relatedness_in_groups);
int maximise_lkhood(int include_last_beta,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *Uy_UXbeta2, gsl_vector *d,double min_d,gsl_vector *beta,double *var0,double *h,double *lk,int nind,void *fparams,PAR *par,FILE *out);
double multiplicative_correction_logOR(double gamma,double mu, double f);
FILE* open_file_or_stdin(char *name,int *reading_stdin);
void print_available_parameters();
void print_available_switches();
int *read_attributes(FILE *in,gsl_vector **y,gsl_matrix **X_full,PAR *par,int *groupsizes,char **exclusionlist,int nexclusionlist);
void read_D(FILE *D_in,gsl_vector **d,int *groupsizes,int *relatedness_in_groups,double min_d);
void read_data(FILE *in,gsl_vector **y,int *groupsizes);
char **read_E(FILE *in,int *n);
int read_gen(FILE *in,gsl_matrix **X_full,int nind_full,int *nind,int *groupsizes,int *nexcluded_from_group,int **excluded_from_group,char **snp_id,PAR *par,int *good_indexes,double *freq,FILE *out);
int read_new_y(FILE *in,gsl_vector *Uy,gsl_matrix **U_full,int *groupsizes);
void read_parameters(PAR *par,int argc, char *argv[]);
void read_priors(FILE *in,gsl_matrix *inv_prior_var,double *logdet_prior_ratio_1to0,int ncovariates,PAR *par);
void read_S(FILE *in,gsl_matrix **S,int *groupsizes,int *relatedness_in_groups,int *good_indexes,int ngood,int read_list);
void read_S_lapack(FILE *in,double **S,int *groupsizes,int *relatedness_in_groups,int *good_indexes,int ngood,int read_list);
void read_U(FILE *U_in,gsl_matrix **U,int *groupsizes,int *relatedness_in_groups);
int read_z(FILE *in,gsl_matrix **X_full,int nind_full,int *nind,int *groupsizes,int *nexcluded_from_group,int **excluded_from_group,char **snp_id,PAR *par,int *good_indexes,double *freq,FILE *out);
int string_search(char *tag,char **list,int nlist);
double square(double x);
int update_beta(int include_last_beta,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,gsl_vector *beta,double var0,double h,int nind);
void update_h_without_GSL(double *h,void *fparams,PAR *par,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,double min_d,gsl_vector *beta,double var0,int nind);
void update_var0(double *var0,double h,gsl_vector *Uy_UXbeta2,gsl_vector *d,int nind);

#if USE_LAPACK == 1
static double dlamch(char CMACH){//this is for LAPACK
    extern double dlamch_(char *CMACHp);
    return dlamch_(&CMACH);
}
#endif

//these two ints will be read in from parameters file
static int ngroups=1; 
static int ncovariates=1; //this does not include 'z' that will be tested, but always includes population mean
static const int print_snp_interval=10000;

int main(int argc, char* argv[])
{ 

  int i,individual,*groupsizes,*relatedness_in_groups,nind_full,nind,converged,reading_stdin,
    *nexcluded_from_groups,**excluded_from_groups,bayesian_success,nfields,read_z_gen,
    snp_status,g,include_snp,snp_index,init_succ,var_succ,same_null_model,nEfile,*good_indexes;
  double 
    **S_lapack=NULL,*aa_const,*logdet_theta,*logbeta,mu,var0,h,lk,lk_full[2],lk_h0[2],
    **ml_estimates,**var_estimates,snp_est_h0[2],score_stat,
    logdet_prior_ratio_1to0,logmlk[2],logmlk_h0[2],freq,current_min_d;
  char 
    *snp_id,**exclusionlist;
  gsl_vector 
    **y_full,*beta,**Uy_full,**d_full,*Uy_UXbeta2,*Uy,*d,
    *tmp_vec,**theta_Uy,**UX_theta_Uy;
  gsl_vector_view 
    vec_view;
  gsl_matrix 
    **X_full,**S_full=NULL,**UX_full,**U_full,*UX,
    *inv_prior_var,**UX_theta_UX,**theta_UX;
  FILE 
    *in,*out,*out_var=NULL;

  FPAR fparams;
  PAR par;  
  clock_t starting_time;

  S_full=NULL;S_lapack=NULL;
  starting_time=clock();
  printf("\n*************************************************************************\n");
  printf("This is MMM v.%4.2f\nA program for analysing a linear mixed model.\n",VERSION);
  printf("Copyright (C) 2011,2012 Matti Pirinen (matti.pirinen@iki.fi).\n");
  printf("This software is distributed under GNU General Public License version 3.\n");
  printf("http://www.gnu.org/licenses/\n");
  printf("*************************************************************************\n");

  read_parameters(&par,argc,argv);

  ncovariates=par.n_covariates+1; //pop mean + covariates but not z 
  ngroups=par.n_groups;
  read_z_gen=par.read_z+par.read_gen;

  if(par.bayesian){
    par.prior_exp=gsl_vector_calloc(ncovariates+1);//prior expectation
    par.prior_var=gsl_matrix_calloc(ncovariates+1,ncovariates+1);//prior variance matrix
    inv_prior_var=gsl_matrix_calloc(ncovariates+1,ncovariates+1);//inverse of variance matrix

    in=open_file_or_stdin(par.prior_file,&reading_stdin);
    read_priors(in,inv_prior_var,&logdet_prior_ratio_1to0,ncovariates,&par);
    if(!reading_stdin) fclose(in);
  }else{par.prior_exp=NULL;par.prior_var=NULL;inv_prior_var=NULL;}

  if((groupsizes=(int *) malloc(ngroups*sizeof(int)))==NULL)
    alloc_error("groupsizes");
  if((nexcluded_from_groups=(int *) calloc(ngroups,sizeof(int)))==NULL)
    alloc_error("nexcluded_from_groups");
  if((excluded_from_groups=(int **) malloc(ngroups*sizeof(int*)))==NULL)
    alloc_error("excluded_from_groups");
  if((relatedness_in_groups=(int *) malloc(ngroups*sizeof(int)))==NULL)
    alloc_error("relatedness_in_groups");
  if((y_full=(gsl_vector **) malloc(ngroups*sizeof(gsl_vector*)))==NULL)
    alloc_error("y_full");
  if((X_full=(gsl_matrix **) malloc(ngroups*sizeof(gsl_matrix*)))==NULL)
    alloc_error("X");
  if((ml_estimates=(double **) malloc(2*sizeof(double*)))==NULL)
    alloc_error("ml_estimates"); 
  if((var_estimates=(double **) malloc(2*sizeof(double*)))==NULL)
    alloc_error("var_estimates");
  for(i=0;i<2;++i){
    if((ml_estimates[i]=(double *) calloc((ncovariates+3),sizeof(double)))==NULL)
      alloc_error("ml_estimates");
    if((var_estimates[i]=(double *) calloc((ncovariates+3),sizeof(double)))==NULL)
      alloc_error("var_estimates");
  }
  
  if(par.read_E){
    in=open_file_or_stdin(par.E_file,&reading_stdin);
    if(!reading_stdin) fprintf(stdout,"\nReading exclusion list from file '%s' ... ",par.E_file);
    else fprintf(stdout,"\nReading exclusion list from stdin ... ");
    exclusionlist=read_E(in,&nEfile);
    if(!reading_stdin) fclose(in);
    fprintf(stdout," %d ids read.\n",nEfile);
  }else{
    exclusionlist=NULL;
    nEfile=0;
  }
  
  in=open_file_or_stdin(par.A_file,&reading_stdin);
  if(!reading_stdin) printf("\nReading attributes from file '%s'\n",par.A_file);
  else fprintf(stdout,"\nReading attributes from stdin ... ");
  good_indexes=read_attributes(in,y_full,X_full,&par,groupsizes,exclusionlist,nEfile);
  if(!reading_stdin) fclose(in);

  if(nEfile){
    for(i=0;i<nEfile;++i)
      free(exclusionlist[i]);
    free(exclusionlist);
  }

  nind_full=par.n_good_ind;
   
  if((U_full=(gsl_matrix **) malloc(ngroups*sizeof(gsl_matrix*)))==NULL)
    alloc_error("U_full");
  if((d_full=(gsl_vector **) malloc(ngroups*sizeof(gsl_vector*)))==NULL)
    alloc_error("d_full");
  
  if(par.read_R){

    in=open_file_or_stdin(par.R_file,&reading_stdin);
    if(reading_stdin) printf("\nReading covariance matrices from stdin.\n");
    else printf("\nReading covariance matrices from R-file '%s'\n",par.R_file);

#if USE_LAPACK == 1
      if((S_lapack=(double **) malloc(ngroups*sizeof(double*)))==NULL)
	alloc_error("S_lapack"); 
      read_S_lapack(in,S_lapack,groupsizes,relatedness_in_groups,good_indexes,par.n_good_ind,par.read_R==1);
      decompose_S_lapack(S_lapack,groupsizes,relatedness_in_groups,U_full,d_full,&par);
#else
      if((S_full=(gsl_matrix **) malloc(ngroups*sizeof(gsl_matrix*)))==NULL)
	alloc_error("S_full"); 
      read_S(in,S_full,groupsizes,relatedness_in_groups,good_indexes,par.n_good_ind,par.read_R==1);
      decompose_S(S_full,groupsizes,relatedness_in_groups,U_full,d_full,&par);
#endif
      if(!reading_stdin) fclose(in);
  }
  else{ //reading U and d from files

    in=open_file_or_stdin(par.U_file,&reading_stdin);
    if(!reading_stdin) printf("\nReading eigenvectors from file '%s'\n",par.U_file);
    else printf("\nReading eigenvectors from stdin\n");
    read_U(in,U_full,groupsizes,relatedness_in_groups);
    if(!reading_stdin) fclose(in);
    
    in=open_file_or_stdin(par.D_file,&reading_stdin);
    if(!reading_stdin) printf("\nReading eigenvalues from file '%s'\n\n",par.D_file);
    else printf("\nReading eigenvectors from stdin\n\n");
    read_D(in,d_full,groupsizes,relatedness_in_groups,par.min_d);
    if(!reading_stdin) fclose(in);
  }
   
  if((UX_full=(gsl_matrix **) malloc(ngroups*sizeof(gsl_matrix*)))==NULL)
    alloc_error("UX_full");

  if((Uy_full=(gsl_vector **) malloc(ngroups*sizeof(gsl_vector*)))==NULL)
    alloc_error("Uy_full");
 
  Uy=gsl_vector_calloc(nind_full); 
  d=gsl_vector_calloc(nind_full);
  Uy_UXbeta2=gsl_vector_calloc(nind_full);
  UX=gsl_matrix_calloc(nind_full,ncovariates+1);
  compute_UX_and_Uy_and_d(X_full,y_full,U_full,d_full,UX_full,Uy_full,Uy,UX,d,groupsizes,relatedness_in_groups);
  current_min_d=gsl_vector_min (d);
  

  if(par.bayesian){
    compare_data_and_prior_var(y_full,groupsizes,ngroups,par.prior_a,par.prior_d);
    
    if((aa_const=(double *) malloc((par.n_integr_intervals+1)*sizeof(double)))==NULL)
      alloc_error("aa_const");
    if((logdet_theta=(double *) malloc((par.n_integr_intervals+1)*sizeof(double)))==NULL)
      alloc_error("logdet_theta");
    if((logbeta=(double *) malloc((par.n_integr_intervals+1)*sizeof(double)))==NULL)
      alloc_error("logbeta");
    if((theta_Uy=(gsl_vector **) malloc((par.n_integr_intervals+1)*sizeof(gsl_vector*)))==NULL)
      alloc_error("theta_Uy");
    if((theta_UX=(gsl_matrix **) malloc((par.n_integr_intervals+1)*sizeof(gsl_matrix*)))==NULL)
      alloc_error("theta_UX");
    if((UX_theta_Uy=(gsl_vector **) malloc((par.n_integr_intervals+1)*sizeof(gsl_vector*)))==NULL)
      alloc_error("UX_theta_Uy");
    if((UX_theta_UX=(gsl_matrix **) malloc((par.n_integr_intervals+1)*sizeof(gsl_matrix*)))==NULL)
      alloc_error("UX_theta_UX");

    for(i=0;i<(par.n_integr_intervals+1);++i)
      theta_Uy[i]=NULL;
#if READ_MANY_PHENOTYPES == 1  //not explained in the manual
    i=1;
    in=open_file_or_stdin(par.z_file,&reading_stdin);
    sprintf(par.results_file,"%s.MMM.out",par.z_file);
    out=fopen(par.results_file,"w");

    bayesian_success=1;
    while(bayesian_success){
      bayesian_success=read_new_y(in,Uy,U_full,groupsizes);
      if(!bayesian_success){
	fprintf(stderr,"ERROR intialising integration variable.\nExits!\n");
	exit(1);
      }
      bayesian_success+=initialize_integration_variables(aa_const,logdet_theta,logbeta,UX_theta_Uy,UX_theta_UX,theta_Uy,theta_UX,inv_prior_var,Uy,UX,d,logmlk,logmlk_h0,&logdet_prior_ratio_1to0,ncovariates,nind_full,current_min_d,&par);
      if(!bayesian_success){
	fprintf(stderr,"ERROR intialising integration variable.\nExits!\n");
	exit(1);
      }
      if((i++)%100==0)
	printf("PHENO %d\n",i);
      fprintf(out,"%12.10g %12.10g\n",(logmlk[0]-logmlk_h0[0])/log(10),compute_log10BF_eta_qags(Uy,UX,d,ncovariates,nind_full,&par));
    }
    fclose(out);
    if(!reading_stdin) fclose(in);
    exit(1);
#else
  //printf("Initializing integration parameters.\n");
    bayesian_success=initialize_integration_variables(aa_const,logdet_theta,logbeta,UX_theta_Uy,UX_theta_UX,theta_Uy,theta_UX,inv_prior_var,Uy,UX,d,logmlk,logmlk_h0,&logdet_prior_ratio_1to0,ncovariates,nind_full,current_min_d,&par);
  //printf("Done.\n");
    if(!bayesian_success){
      fprintf(stderr,"ERROR intialising integration variable.\nExits!\n");
      exit(1);
    }
#endif 
  }//if(par.bayesian)

  fparams.var0=&var0;
  beta=gsl_vector_calloc(ncovariates+1);
  if(current_min_d<MIN_D_TOLERANCE){
    printf("\n\nWARNING: minimum eigenvalue < %g.\nConstrains eta to interval [0,0.99].\n",MIN_D_TOLERANCE);
    printf("Consider using min_d parameter to set a lower bound (>%g) on eigenvalues.\n\n",MIN_D_TOLERANCE);
  }
  if(read_z_gen){
    in=open_file_or_stdin(par.z_file,&reading_stdin);
    if(!reading_stdin) printf("\nReading data from file '%s'\n",par.z_file);
    else printf("\nReading data from stdin\n");
  }
  
  printf("Writing results to file '%s'\n\n",par.results_file);
  
  out=fopen(par.results_file,"w");

  if(par.read_gen)
    fprintf(out,"snp_id1 snp_id2 pos allele_0 allele_1 n_included n_missing n_imputed gen_00 gen_01 gen_11 gen_NULL freq_1 var_info_nonmissing var_info_all ");
  if(par.read_z)
    fprintf(out,"z_id n_included n_missing n_imputed mean_z sd_z ");
  fprintf(out,"error pop_mean_est pop_mean_se ");
  for(i=1;i<ncovariates;++i)
    fprintf(out,"covar_%s_est covar_%s_se ",par.covariates[i-1],par.covariates[i-1]);
  fprintf(out,"var_est var_se eta_est eta_se loglkhood ");
  nfields=ncovariates*2+5;
  if(par.read_z){
    fprintf(out,"z_est_eta0 z_se_eta0 chisq_eta0 pval_eta0 z_est z_se chisq pval");
    nfields+=8;
  }
  if(par.read_gen){
    fprintf(out,"z_est_eta0 z_se_eta0 chisq_eta0 pval_eta0 z_est z_se chisq pval");
    nfields+=8;
  }
  if(!read_z_gen){
    fprintf(out,"lr_stat_eta");nfields++;
#if SCORE_TEST_ETA == 1
    fprintf(out," pval_score_eta");nfields++;
#endif
  }
  if(par.bayesian){
    if(read_z_gen){
      fprintf(out," log10BF_eta0 log10BF");
      nfields+=2;
    }
    else{
      fprintf(out," log10BF_eta");
      nfields++;
    }
  }
  fprintf(out,"\n");

  for(i=0;i<ngroups;++i){
    if(groupsizes[i])
      if((excluded_from_groups[i]=(int *) malloc(groupsizes[i]*sizeof(int)))==NULL)
	alloc_error("excluded_from_groups");
  }
  
  //COMPUTATIONS START
  /*
    At high level this works as follows.

    (1) Reads in a predictor (either from z-file or gen-file).

    (2) If there are missing data and it is modelled properly then makes new data structures 
    by excluding the individuals with missing data.

    (3) Maximises the model without predictor (include_snp==0) and/or with predictor (include_snp==1).

    The key variables are:

    snp_est_h0[i] 
    ML estimate of predictor effect (i==0) or its standard error (i==1), when eta==0.0
    i.e. under standard linear model. If logOR==1 then these estimates are given on log-odds scale.

    ml_estimates[snp][i] 
    ML estimates for the full model without the predictor (snp==0) 
    or with the predictor (snp==1) for i=0,...,ncovariates+2 where the order is
    covariates 0,...,ncovariates-1, predictor,var0 and eta.

    var_estimates[snp][i] 
    similarly for asymptotic variances of the ML estimates

    (4) Then does the Bayesian computation, if required.

   */

  snp_status=1;
  snp_index=1;
  nind=nind_full;
  same_null_model=0;
  score_stat=0;
  while(snp_status==1){

    if(par.read_z)
      snp_status=read_z(in,X_full,nind_full,&nind,groupsizes,nexcluded_from_groups,
			excluded_from_groups,&snp_id,&par,good_indexes,&freq,out);
    if(par.read_gen)
      snp_status=read_gen(in,X_full,nind_full,&nind,groupsizes,nexcluded_from_groups,
			  excluded_from_groups,&snp_id,&par,good_indexes,&freq,out);
    if(snp_status==1){
      if(read_z_gen){
	if( (snp_index%print_snp_interval)==0)
	  fprintf(stdout,"Going at line %d. Time used:%g secs.\n",
		  snp_index,(clock()-starting_time)/1.0/CLOCKS_PER_SEC);
	
	if(Uy->size==nind_full && nind==nind_full){
	  //In this case the current Uy is for all individuals
	  //and also data just read in is for the full data set.
	  //Then only the last column of UX needs to be changed
	  same_null_model=1; //null model needs not be recomputed
	  individual=0;
	  for(g=0;g<ngroups;++g){
	    if(relatedness_in_groups[g]){
	      tmp_vec=gsl_vector_alloc(groupsizes[g]);
	      vec_view=gsl_matrix_column (X_full[g], ncovariates);
	      gsl_blas_dgemv (CblasTrans, 1.0, U_full[g], &vec_view.vector, 0.0, tmp_vec);
	      for(i=0;i<groupsizes[g];++i)
		gsl_matrix_set(UX,individual++,ncovariates,gsl_vector_get(tmp_vec,i));
	      gsl_vector_free(tmp_vec);
	    }else{
	      for(i=0;i<groupsizes[g];++i)
		gsl_matrix_set(UX,individual++,ncovariates,gsl_matrix_get(X_full[g],i,ncovariates));
	    }
	  }
	}
	else{//exclusions need to be made
	  if(par.bayesian){
	    fprintf(stderr,"Missing data not currently tolerated in Bayesian analysis.\nExits!\n");
	    exit(1);
	  }	
	  same_null_model=0;//re-estimate the null model as well
	  if(nind!=Uy->size){
	    gsl_vector_free(Uy);
	    gsl_vector_free(d);
	    gsl_vector_free(Uy_UXbeta2);
	    gsl_matrix_free(UX);

	    d=gsl_vector_alloc(nind);
	    Uy=gsl_vector_alloc(nind);
	    Uy_UXbeta2=gsl_vector_alloc(nind);
	    UX=gsl_matrix_alloc(nind,ncovariates+1);
	  }	  
#if USE_LAPACK == 1
	  make_data_structures_LAPACK(Uy,UX,d,X_full,y_full,U_full,UX_full,Uy_full,d_full,S_lapack,nind,nexcluded_from_groups,excluded_from_groups,groupsizes,relatedness_in_groups);
#else
	  make_data_structures(Uy,UX,d,X_full,y_full,U_full,UX_full,Uy_full,d_full,S_full,nind,nexcluded_from_groups,excluded_from_groups,groupsizes,relatedness_in_groups);
#endif
	  current_min_d=gsl_vector_min(d);
	}
      }

      fparams.d=d;
      fparams.Uy_UXbeta2=Uy_UXbeta2;
      fparams.nind=nind;
    
      for(include_snp=0;include_snp<=(read_z_gen);++include_snp){

	if(snp_index>1 && same_null_model) 
	  include_snp=1; //null model is the same as previously, no need to re-estimate
 
	init_succ=initialize_params(include_snp,beta,&var0,&h,UX,Uy,Uy_UXbeta2,nind);
	//sets beta and var0 to ML estimates (on linear scale) when h==0.0
	
	if(init_succ){ //init_succ==0 if cholesky(X^t X) failed, otherwise init_succ==1
	  lk_h0[include_snp]=-0.5*nind*log(var0)-(nind/2.0); //ignores -nind/2*LOG2PI
	  //this is the same as loglkhood(Uy,UX,beta,var0,0.0,d,nind);
	  if(!read_z_gen){//compute score statistic for eta==0
	    gsl_blas_ddot (d, Uy_UXbeta2,&score_stat);//score_stat=d.(U'y-U'X beta)^2
	    score_stat=(score_stat-gsl_blas_dasum (Uy_UXbeta2))/var0;//score_stat=[(d-1).(U'y-U'X beta)^2]/var0
	  }
	  if(include_snp){
	    var_succ=compute_var_estimates_without_h(beta,var0,include_snp,var_estimates,
						     Uy_UXbeta2,UX,out_var,snp_id,par.logOR);
	    snp_est_h0[0]=gsl_vector_get(beta,ncovariates);
	    mu=gsl_vector_get(beta,0);
	    if(par.logOR) snp_est_h0[0]/= mu*(1-mu);
	    snp_est_h0[1]=var_estimates[include_snp][ncovariates];
	    if(par.logOR==2) {
	      if(var_succ)
		snp_est_h0[1]*=square(multiplicative_correction_logOR(snp_est_h0[0],mu,freq));//adjust variance
	      snp_est_h0[0]*=multiplicative_correction_logOR(snp_est_h0[0],mu,freq);//adjust logOR estimate
	    }
	  }
	      
	  if(par.fix_eta)
	    h=par.fixed_eta;
	  else
	    h=ml_estimates[0][ncovariates+2]; 
	  //take initial value from previous snp (or 0 if this is the first)

	  converged=maximise_lkhood(include_snp,Uy,UX,Uy_UXbeta2,d,current_min_d,beta,&var0,&h,&lk,nind,&fparams,&par,NULL);

	  if(!converged){
	    fprintf(stderr,"%s didn't converge within %d iterations\n",snp_id,par.max_iterations);
	    //return 1;
	  }
	  else{
	    
	    lk_full[include_snp]=lk;
	    mu=gsl_vector_get(beta,0);
	    if(par.logOR>0) { //logistic model
	      ml_estimates[include_snp][0]=log(mu/(1-mu));
	      for(i=1;i<ncovariates+1;++i) ml_estimates[include_snp][i]=gsl_vector_get(beta,i)/mu/(1-mu);
	    }else{ //linear model
	      ml_estimates[include_snp][0]=mu;
	      for(i=1;i<ncovariates+1;++i) ml_estimates[include_snp][i]=gsl_vector_get(beta,i);
	    }
	    ml_estimates[include_snp][i++]=var0; //var param always on linear scale
	    ml_estimates[include_snp][i++]=h;
	    var_succ=compute_var_estimates(beta,var0,h,include_snp,var_estimates,Uy_UXbeta2,UX,Uy,d,nind,out_var,snp_id,par.logOR);
	    if(include_snp && par.logOR==2) {
	      if(var_succ) 
		var_estimates[1][ncovariates]*=square(multiplicative_correction_logOR(ml_estimates[1][ncovariates],mu,freq));//adjust variance
	      ml_estimates[1][ncovariates]*=multiplicative_correction_logOR(ml_estimates[1][ncovariates],mu,freq);//adjust logOR estimate
	    }

	  }  	  
	}
	else{//init_succ == 0
	  break;
	}
      }

      if(init_succ && converged){
	fprintf(out," 0");
	for(i=0;i<ncovariates;++i)
	  if(var_estimates[read_z_gen][i]>=0)
	    fprintf(out," %10.6g %10.6g",ml_estimates[read_z_gen][i],sqrt(var_estimates[read_z_gen][i]));
	  else
	    fprintf(out," %10.6g NA",ml_estimates[read_z_gen][i]);
	for(i=ncovariates+1;i<ncovariates+1+2;++i)
	  if(var_estimates[read_z_gen][i]>=0)
	    fprintf(out," %10.6g %10.6g",ml_estimates[read_z_gen][i],sqrt(var_estimates[read_z_gen][i]));
	  else
	    fprintf(out," %10.6g NA",ml_estimates[read_z_gen][i]);
	fprintf(out," %10.6g",lk_full[read_z_gen]-0.5*nind*LOG2PI);
	if(read_z_gen){
	  if(snp_est_h0[1]>=0)
	    fprintf(out," %10.6g %10.6g",snp_est_h0[0],sqrt(snp_est_h0[1]));
	  else
	    fprintf(out," %10.6g NA",snp_est_h0[0]);
	  fprintf(out," %10.6lf %g",-2*(lk_h0[0]-lk_h0[1]),1-gsl_cdf_chisq_P (-2*(lk_h0[0]-lk_h0[1]), 1));

	  i=ncovariates;
	  if(var_estimates[1][i]>=0)
	    fprintf(out," %10.6g %10.6g",ml_estimates[1][i],sqrt(var_estimates[1][i]));
	  else
	    fprintf(out," %10.6g NA",ml_estimates[1][i]);
	  fprintf(out," %10.6lf %g",-2*(lk_full[0]-lk_full[1]),1-gsl_cdf_chisq_P (-2*(lk_full[0]-lk_full[1]), 1));
	  //,(clock()-starting_time)/1.0/CLOCKS_PER_SEC);
	}else{// print lr_stat_eta and pval_score_eta
	  if(-2*(lk_h0[0]-lk_full[0])<0.0)//h==0.0 
	    fprintf(out," %10.6g",0.0);
	  else
	    fprintf(out," %10.6g",-2*(lk_h0[0]-lk_full[0]));//likelihood ratio statistic between models eta>0 and eta==0
#if SCORE_TEST_ETA == 1	  
	  fprintf(out," %10.6g",davies_cdf(score_stat,d));
#endif
	}
      }else{
	if(!init_succ) fprintf(out," 1"); //initialisaton failed
	else fprintf(out," 2"); //didn't converge
	//fprintf(stderr,"0 %s ERROR with covariate matrix. Check this SNP.\n",snp_id);
	for(i=0;i<nfields-(2*par.bayesian);++i)
	  fprintf(out," NA");
      }
      //ML part ENDS

      if(par.bayesian){
	if(read_z_gen){
	  bayesian_success=compute_mlk(aa_const,logdet_theta,logbeta,UX_theta_Uy,UX_theta_UX,theta_Uy,theta_UX,inv_prior_var,UX,d,logmlk_h0,logmlk,ncovariates,nind_full,current_min_d,&par);
	  //logdet_prior_ratio_1to0: log(det(V))-log(det(V')), where V' is V without last row and column
	  //logmlk_h0[0]: 0.5*log(det(VVsub))-(prior_d+nind)/2*log(aa_sub) for model where h==0.0
	  //logmlk_h0[1]: 0.5*log(det(VV))-(prior_d+nind)/2*log(aa) for model where h==0.0
	  //logmlk[0]: 0.5*log(det(VVsub))-(prior_d+nind)/2*log(aa_sub) for model where h>=0.0
	  //logmlk[1]: 0.5*log(det(VV))-(prior_d+nind)/2*log(aa) for model where h>=0.0
	  //printf("logmlk_h0[0]=%g, logmlk[0]=%g\n",logmlk_h0[0],logmlk[0]);
	  //printf("logmlk_h0[1]=%g, logmlk[1]=%g\n",logmlk_h0[1],logmlk[1]);
	  if(bayesian_success)
	    fprintf(out," %10.6g %10.6g",
		    (logmlk_h0[1]-logmlk_h0[0]-0.5*(logdet_prior_ratio_1to0))/log(10),
		    (logmlk[1]-logmlk[0]-0.5*(logdet_prior_ratio_1to0))/log(10));
	  else
	    fprintf(out," NA NA");
	  if(snp_index==1) 
	    printf("log10BF h>0 vs h==0:%g\n",(logmlk[0]-logmlk_h0[0])/log(10));
	}else{
	  fprintf(out," %10.6g",compute_log10BF_eta_qags(Uy,UX,d,ncovariates,nind_full,&par));
	}
      }
      fprintf(out,"\n");
      snp_index++;
      if(read_z_gen==0) snp_status=0; //this will end the while loop
      else free(snp_id);
    }
    else{      //Not successful with reading snp
      if(snp_status==2){//end of file
	fprintf(stdout,"End of input file.\n");
	break;
      }
      if(snp_status==3){//either error reading entry or all data missing
	fprintf(out," 3");
	free(snp_id);
      }
      if(snp_status>3){//error reading id fields
	for(i=0;i<snp_status;++i)
	  fprintf(out,"NA ");
	fprintf(out,"3");
      }
 
      for(i=0;i<nfields;++i)
	fprintf(out," NA");
      fprintf(out,"\n");
      snp_status=1;
    }
    
  }

  fclose(out);
  if(read_z_gen && !reading_stdin)
    fclose(in);

  for(g=0;g<ngroups;++g){
    if(groupsizes[g]){
      free(excluded_from_groups[g]);
      gsl_vector_free(y_full[g]);
      gsl_matrix_free(U_full[g]);
      gsl_vector_free(d_full[g]);
      gsl_matrix_free(X_full[g]);
      gsl_matrix_free(UX_full[g]);
      gsl_vector_free(Uy_full[g]);
      if(par.read_R && relatedness_in_groups[g]){
#if USE_LAPACK==1
	  free(S_lapack[g]);
#else
	  gsl_matrix_free(S_full[g]);
#endif
      }
    }
  }
  free(relatedness_in_groups);
  free(nexcluded_from_groups);
  free(excluded_from_groups);
  free(U_full);
  free(d_full);
  free(UX_full);
  free(Uy_full);
  free(X_full);
  free(y_full);
  free(groupsizes);
  if(par.read_R) 
#if USE_LAPACK==1
    free(S_lapack);
#else
  free(S_full);
#endif

  for(i=0;i<2;++i){
    free(ml_estimates[i]);
    free(var_estimates[i]);
  }
  free(ml_estimates);
  free(var_estimates);
  free(good_indexes);
  gsl_vector_free(Uy);
  gsl_matrix_free(UX);
  gsl_vector_free(d);
  gsl_vector_free(Uy_UXbeta2);
  gsl_vector_free(beta);
  if(par.bayesian){
    gsl_matrix_free(par.prior_var);
    gsl_matrix_free(inv_prior_var);
    gsl_vector_free(par.prior_exp);
    free(aa_const);
    free(logdet_theta);
    free(logbeta);
    free(par.prior_file);
    for(i=0;i<par.n_integr_intervals+1;++i){
      gsl_vector_free(theta_Uy[i]);
      gsl_matrix_free(theta_UX[i]);
      gsl_vector_free(UX_theta_Uy[i]);
      gsl_matrix_free(UX_theta_UX[i]);
    }
    free(theta_Uy);
    free(theta_UX);
    free(UX_theta_Uy);
    free(UX_theta_UX);
  }
  for(i=0;i<par.n_covariates;++i)
    free(par.covariates[i]);
  if(par.n_covariates)
    free(par.covariates);

  if(read_z_gen) {
    free(par.z_file);
  }
  if(par.A_file!=NULL)free(par.A_file);
  if(par.inds_out_file!=NULL)free(par.inds_out_file);
  if(par.results_file!=NULL)free(par.results_file);
  if(par.U_file!=NULL)free(par.U_file);
  if(par.D_file!=NULL)free(par.D_file);
  if(par.outcome!=NULL)free(par.outcome);
  if(par.R_file!=NULL)free(par.R_file);
  if(par.id!=NULL) free(par.id);
  if(par.read_E) free(par.E_file);

  printf("Time used: %g seconds.\nExits normally.\n",(clock()-starting_time)/1.0/CLOCKS_PER_SEC);
  
  return 0;
  
}


void compare_data_and_prior_var(gsl_vector **y_full,int *groupsizes,int ngroups,double prior_a,double prior_d){

  int i,g,nind=0;
  double pheno,data_var,sum=0.0,sqsum=0.0;
  gsl_sf_result result;

  for(g=0;g<ngroups;++g){
    nind+=groupsizes[g];
    for(i=0;i<groupsizes[g];++i){
      pheno=gsl_vector_get(y_full[g],i);
      sum+=pheno;
      sqsum+=pheno*pheno;
    }
  }
  sum/=nind;
  sqsum/=nind;
  data_var=sqsum-sum*sum;

  printf("\n\nPrior for variance: mode=%g mean=",prior_a/(prior_d+2));
  if(prior_d>2) printf("%g sd=",prior_a/(prior_d-2));
  else printf("NA sd=");
  if(prior_d>4) printf("%g\n",sqrt(2*prior_a*prior_a/(prior_d-2)/(prior_d-2)/(prior_d-4)));
  else printf("NA\n");
  printf("Variance in phenotype data:%g\n",data_var);
  if(gsl_sf_gamma_inc_Q_e (prior_d/2, prior_a/2/data_var,&result)==GSL_SUCCESS)
    printf("priorP(var<%g)=%g\n",data_var,result.val);

}

void  make_data_structures(gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,gsl_matrix **X_full,gsl_vector **y_full,gsl_matrix **U_full,gsl_matrix **UX_full,gsl_vector **Uy_full,gsl_vector **d_full,gsl_matrix **S_full,int nind,int *nexcluded_from_groups,int **excluded_from_groups,int *groupsizes,int *relatedness_in_groups){

  /*Will fill Uy=U^t y and UX=U^t X and d=eigenvalues of decomposition S=U d U^t
    All are calculated by taking into account possible exclusions.
   */
  int i,j,g,i_index_excluded,j_index_excluded;
  int individual,new_groupsize,row,column;
  double v;
  gsl_matrix *UXg,*Sg,*Ug,*Xg;
  gsl_vector *Uyg,*dg,*yg;
  gsl_eigen_symmv_workspace *eigen_space;

  individual=0; //position in vector Uy of current individual

  for(g=0;g<ngroups;++g){

    new_groupsize=groupsizes[g]-nexcluded_from_groups[g];

    if(new_groupsize==0)
      continue; //jump over the group

    if(nexcluded_from_groups[g]>0){
      
      if(relatedness_in_groups[g] && new_groupsize>1){

	Xg=gsl_matrix_alloc(new_groupsize,ncovariates+1);
	yg=gsl_vector_alloc(new_groupsize);

	i_index_excluded=0;
	row=0;
	for(i=0;i<groupsizes[g];++i){
	  while(i_index_excluded < nexcluded_from_groups[g] && excluded_from_groups[g][i_index_excluded]==i){
	    ++i;
	    ++i_index_excluded;
	  } 
	  if(i>=groupsizes[g])
	    break;
	  for(j=0;j<ncovariates+1;++j)
	    gsl_matrix_set(Xg,row,j,gsl_matrix_get(X_full[g],i,j));
	  gsl_vector_set(yg,row,gsl_vector_get(y_full[g],i));
	  ++row;
	}

	Sg=gsl_matrix_alloc(new_groupsize,new_groupsize);

	row=0;
	i_index_excluded=0;
	for(i=0;i<groupsizes[g];++i){
	  while(i_index_excluded < nexcluded_from_groups[g] && excluded_from_groups[g][i_index_excluded]==i){
	    ++i;
	    ++i_index_excluded;
	  }
	  if(i>=groupsizes[g])
	    break;
	  j_index_excluded=0;
	  column=0;
	  for(j=0;j<groupsizes[g];++j){
	    while(j_index_excluded < nexcluded_from_groups[g] && excluded_from_groups[g][j_index_excluded]==j){
	      ++j;
	      ++j_index_excluded;
	    }
	    if(j>=groupsizes[g])
	      break;
	    gsl_matrix_set(Sg,row,column,gsl_matrix_get(S_full[g],i,j));
	    ++column;
	  }
	  ++row;
	}
	//dg,Ug,etc are variables for this group
	dg=gsl_vector_alloc(new_groupsize);
	Ug=gsl_matrix_alloc(new_groupsize,new_groupsize);
	eigen_space=gsl_eigen_symmv_alloc (new_groupsize);
	gsl_eigen_symmv (Sg, dg, Ug, eigen_space);

	UXg=gsl_matrix_alloc(new_groupsize,ncovariates+1);
	Uyg=gsl_vector_alloc(new_groupsize);
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, Ug, Xg, 0.0, UXg);//UXg==Ug^T Xg
	gsl_blas_dgemv (CblasTrans, 1.0, Ug, yg, 0.0, Uyg); //Uyg=Ug^T yg

	for(i=0;i<new_groupsize;++i){
	  gsl_vector_set(Uy,individual,gsl_vector_get(Uyg,i));
	  gsl_vector_set(d,individual,gsl_vector_get(dg,i));
	  for(j=0;j<ncovariates+1;++j)
	    gsl_matrix_set(UX,individual,j,gsl_matrix_get(UXg,i,j));
	  ++individual;
	}
	  
	gsl_eigen_symmv_free(eigen_space);
	gsl_matrix_free(Sg);
	gsl_matrix_free(Ug);
	gsl_matrix_free(UXg);
	gsl_vector_free(dg);
	gsl_vector_free(Uyg); 
	gsl_matrix_free(Xg);
	gsl_vector_free(yg);
	
      }
      else{ // there are exclusions but no relatedness in this group, or the group has only one individual left after exclusions
	
	i_index_excluded=0;
	for(i=0;i<groupsizes[g];++i){
	  while(i_index_excluded < nexcluded_from_groups[g] && excluded_from_groups[g][i_index_excluded]==i){
	    ++i;
	    ++i_index_excluded;
	  }
	  if(i>=groupsizes[g])
	    break;
	  for(j=0;j<ncovariates+1;++j)
	    gsl_matrix_set(UX,individual,j,gsl_matrix_get(X_full[g],i,j));
	  gsl_vector_set(Uy,individual,gsl_vector_get(y_full[g],i));
	  gsl_vector_set(d,individual,1.0);
	  ++individual;
	}
      }
    }
    else{// no exclusions in this group

      if(relatedness_in_groups[g]){
      	for(i=0;i<groupsizes[g];++i){
	  for(j=0;j<ncovariates;++j)
	    gsl_matrix_set(UX,individual,j,gsl_matrix_get(UX_full[g],i,j));
	  v=0.0;
	  for(j=0;j<groupsizes[g];++j)
	    v+=gsl_matrix_get(U_full[g],j,i)*gsl_matrix_get(X_full[g],j,ncovariates);
	  gsl_matrix_set(UX,individual,ncovariates,v);
	  gsl_vector_set(Uy,individual,gsl_vector_get(Uy_full[g],i));
	  gsl_vector_set(d,individual,gsl_vector_get(d_full[g],i));
	  
	  ++individual;
	}
      }
      else{
	for(i=0;i<groupsizes[g];++i){
	  for(j=0;j<ncovariates+1;++j)
	    gsl_matrix_set(UX,individual,j,gsl_matrix_get(X_full[g],i,j));
	  gsl_vector_set(Uy,individual,gsl_vector_get(y_full[g],i));
	  gsl_vector_set(d,individual,1.0);
	  ++individual;
	}
      }
    }
  }
  if(individual != nind){
    fprintf(stderr,"ERROR in make_data_structures\n");
    exit(1);
  }
}

#if USE_LAPACK == 1
void  make_data_structures_LAPACK(gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,gsl_matrix **X_full,gsl_vector **y_full,gsl_matrix **U_full,gsl_matrix **UX_full,gsl_vector **Uy_full,gsl_vector **d_full,double **S_full,int nind,int *nexcluded_from_groups,int **excluded_from_groups,int *groupsizes,int *relatedness_in_groups){

  /*Will fill Uy=U^t y and UX=U^t X and d=eigenvalues of decomposition S=U d U^t
    All are calculated by taking into account possible exclusions.
   */
  int i,j,g,i_index_excluded,j_index_excluded;
  int individual,new_groupsize,row,column;
  double v;
  double *dg,*Sg,*Ug;
  gsl_vector *Uyg,*yg;
  gsl_matrix *UXg,*Xg,*Ug_gsl;

  individual=0; //position in vector Uy of current individual

  for(g=0;g<ngroups;++g){

    new_groupsize=groupsizes[g]-nexcluded_from_groups[g];

    if(new_groupsize==0)
      continue; //jump over the group

    if(nexcluded_from_groups[g]>0){
      
      if(relatedness_in_groups[g] && new_groupsize>1){

	Xg=gsl_matrix_alloc(new_groupsize,ncovariates+1);
	yg=gsl_vector_alloc(new_groupsize);

	i_index_excluded=0;
	row=0;
	for(i=0;i<groupsizes[g];++i){
	  while(i_index_excluded < nexcluded_from_groups[g] && excluded_from_groups[g][i_index_excluded]==i){
	    ++i;
	    ++i_index_excluded;
	  } 
	  if(i>=groupsizes[g])
	    break;
	  for(j=0;j<ncovariates+1;++j)
	    gsl_matrix_set(Xg,row,j,gsl_matrix_get(X_full[g],i,j));
	  gsl_vector_set(yg,row,gsl_vector_get(y_full[g],i));
	  ++row;
	}

	if((Sg = (double *) malloc(new_groupsize*new_groupsize*sizeof(double)))==NULL)
	  alloc_error("Sg");

	row=0;
	i_index_excluded=0;
	for(i=0;i<groupsizes[g];++i){
	  while(i_index_excluded < nexcluded_from_groups[g] && excluded_from_groups[g][i_index_excluded]==i){
	    ++i;
	    ++i_index_excluded;
	  }
	  if(i>=groupsizes[g])
	    break;
	  j_index_excluded=0;
	  column=0;
	  for(j=0;j<groupsizes[g];++j){
	    while(j_index_excluded < nexcluded_from_groups[g] && excluded_from_groups[g][j_index_excluded]==j){
	      ++j;
	      ++j_index_excluded;
	    }
	    if(j>=groupsizes[g])
	      break;
	    //gsl_matrix_set(Sg,row,column,gsl_matrix_get(S_full[g],i,j));
	    Sg[column*new_groupsize+row]=S_full[g][i+groupsizes[g]*j];
	    ++column;
	  }
	  ++row;
	}

	//dg,Ug,etc are variables for this group
	if((Ug = (double *) malloc(new_groupsize*new_groupsize*sizeof(double)))==NULL)
	  alloc_error("Ug in make_data_structures_LAPACK");
	if((dg = (double *) malloc(new_groupsize*sizeof(double)))==NULL)
	  alloc_error("dg in make_data_structures_LAPACK");
	eigen(Sg,new_groupsize,new_groupsize,0,dg,Ug);

	//this copying could be avoided by using cblas directly instead of gsl_cblas in matrix products below!
	Ug_gsl=gsl_matrix_alloc(new_groupsize,new_groupsize); 
	for(i=0;i<new_groupsize;++i)
	  for(j=0;j<new_groupsize;++j)
	    gsl_matrix_set(Ug_gsl,j,new_groupsize-i-1,Ug[i*new_groupsize+j]);

	UXg=gsl_matrix_alloc(new_groupsize,ncovariates+1);
	Uyg=gsl_vector_alloc(new_groupsize);
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, Ug_gsl, Xg, 0.0, UXg);//UXg==Ug^T Xg
	gsl_blas_dgemv (CblasTrans, 1.0, Ug_gsl, yg, 0.0, Uyg); //Uyg=Ug^T yg

	for(i=0;i<new_groupsize;++i){
	  gsl_vector_set(Uy,individual,gsl_vector_get(Uyg,i));
	  gsl_vector_set(d,individual,dg[new_groupsize-1-i]);
	  for(j=0;j<ncovariates+1;++j)
	    gsl_matrix_set(UX,individual,j,gsl_matrix_get(UXg,i,j));
	  ++individual;
	}
	  
	free(Sg);
	free(Ug);
	free(dg);
	gsl_matrix_free(Ug_gsl);
	gsl_matrix_free(UXg);
	gsl_vector_free(Uyg); 
	gsl_matrix_free(Xg);
	gsl_vector_free(yg);
	
      }
      else{ // there are exclusions but no relatedness in this group, or the group has only one individual left after exclusions
	
	i_index_excluded=0;
	for(i=0;i<groupsizes[g];++i){
	  while(i_index_excluded < nexcluded_from_groups[g] && excluded_from_groups[g][i_index_excluded]==i){
	    ++i;
	    ++i_index_excluded;
	  }
	  if(i>=groupsizes[g])
	    break;
	  for(j=0;j<ncovariates+1;++j)
	    gsl_matrix_set(UX,individual,j,gsl_matrix_get(X_full[g],i,j));
	  gsl_vector_set(Uy,individual,gsl_vector_get(y_full[g],i));
	  gsl_vector_set(d,individual,1.0);
	  ++individual;
	}
      }
    }
    else{// no exclusions in this group

      if(relatedness_in_groups[g]){
      	for(i=0;i<groupsizes[g];++i){
	  for(j=0;j<ncovariates;++j)
	    gsl_matrix_set(UX,individual,j,gsl_matrix_get(UX_full[g],i,j));
	  v=0.0;
	  for(j=0;j<groupsizes[g];++j)
	    v+=gsl_matrix_get(U_full[g],j,i)*gsl_matrix_get(X_full[g],j,ncovariates);
	  gsl_matrix_set(UX,individual,ncovariates,v);
	  gsl_vector_set(Uy,individual,gsl_vector_get(Uy_full[g],i));
	  gsl_vector_set(d,individual,gsl_vector_get(d_full[g],i));
	  
	  ++individual;
	}
      }
      else{
	for(i=0;i<groupsizes[g];++i){
	  for(j=0;j<ncovariates+1;++j)
	    gsl_matrix_set(UX,individual,j,gsl_matrix_get(X_full[g],i,j));
	  gsl_vector_set(Uy,individual,gsl_vector_get(y_full[g],i));
	  gsl_vector_set(d,individual,1.0);
	  ++individual;
	}
      }
    }
  }
  if(individual != nind){
    fprintf(stderr,"ERROR in make_data_structures\n");
    exit(1);
  }
}
#endif


int read_z(FILE *in,gsl_matrix **X_full,int nind_full,int *nind,int *groupsizes,int *nexcluded_from_group,int **excluded_from_group,char **snp_id,PAR *par,int *good_indexes,double *freq,FILE *out){

  /*returns 
    1 if successful
    2 if file ends
    3 if error reading some z value or if all data are missing or if too many entries on the line
    6 if error reading z_id from the first column
   */

  int i,g,nchars,total_index,good_ind,nimputed;
  size_t size;
  double geno,geno_sum,geno_sd;
  char *line;
  FILE *line_str;

  size=nind_full*8;
  if((line=(char*) malloc(size*sizeof(char)))==NULL)
    alloc_error("line in read_z");

  nchars=getline(&line,&size,in);
  line_str=fmemopen(line,nchars*sizeof(char),"r");

  if(nchars<1 || fscanf(line_str,"%as",snp_id)!=1){
    free(line);
    if(feof(in))
      return 2;
    else
      return 6;
  }
  fprintf(out,"%s",*snp_id);

  *nind=0;
  geno_sum=0.0;geno_sd=0.0;
  total_index=0;
  good_ind=0;
  for(g=0;g<ngroups;++g){
    nexcluded_from_group[g]=0;
    for(i=0;i<groupsizes[g];++i){
      while(total_index <= good_indexes[good_ind]){
	if(fscanf(line_str,"%lf",&geno)!=1){
	  fprintf(out," 0 %d 0 NA NA",nind_full);
	  fprintf(stderr,"reading z:'%s'. ERROR with subject %d of group %d (i.e. element %d from left)\n",*snp_id,i+1,g+1,total_index+2);
	  free(line);
	  return 3;
	}
	++total_index;
      }
      if(par->check_geno){
	if((geno>2.0 || geno<0.0) && (!par->model_missing || geno != par->missing_z)){ 
	  //allowed values in [0,...,2] or missing_z
	  fprintf(out," 0 %d 0 NA NA",nind_full);
	  fprintf(stderr,"reading z:'%s'. ERROR genotype of individual %d of group %d is %f\n",*snp_id,i+1,g+1,geno);
	  free(line);
	  return 3;
	}
      }
      gsl_matrix_set(X_full[g],i,ncovariates,geno);
      if(par->model_missing && geno==par->missing_z)
	excluded_from_group[g][nexcluded_from_group[g]++]=i;
      else{
	++(*nind);
	geno_sum+=geno;
	geno_sd+=geno*geno;
      }
      ++good_ind;
    }
  }//has read data on all non-excluded individuals (total_index is the count)  

  //Check that the line contains exactly as many z-entries 
  //as there were individuals in the attributes file (without any exclusions)
  while(total_index< par->n_total_ind){
    if(fscanf(line_str,"%lf",&geno)!=1){
      fprintf(out," 0 %d 0 NA NA",nind_full);
      fprintf(stderr,"reading z:'%s'. ERROR with entry %d from left.\n",*snp_id,total_index+2);
      free(line);
      return 3;
    }
    ++total_index;
  }
  if(fscanf(line_str,"%lf",&geno)==1){
    fprintf(out," 0 %d 0 NA NA",nind_full);
    fprintf(stderr,"reading z:'%s'. ERROR more individuals than in Attributes-file (%d).\n",*snp_id,total_index);
    free(line);
    return 3;
  }



  if(*nind>0){  //else geno_sum==0.0;
    geno_sum/=(*nind);
    geno_sd=sqrt(geno_sd/(*nind)-geno_sum*geno_sum);
  }else{
    fprintf(out," 0 %d 0 NA NA",nind_full);
    fprintf(stderr,"reading z:'%s'. ERROR all data missing\n",*snp_id);
    free(line);
    return 3;
  }

  nimputed=0;
  if(par->model_missing && par->impute_missing){
    for(g=0;g<ngroups;++g){
      for(i=0;i<nexcluded_from_group[g];++i)
	gsl_matrix_set(X_full[g],excluded_from_group[g][i],ncovariates,geno_sum);
      nimputed+=nexcluded_from_group[g];
      nexcluded_from_group[g]=0;
    }
    *nind=nind_full;
  }

  if(par->mean_center){
    for(g=0;g<ngroups;++g){
      for(i=0;i<groupsizes[g];++i){
	geno=gsl_matrix_get(X_full[g],i,ncovariates);
	gsl_matrix_set(X_full[g],i,ncovariates,geno-geno_sum);//center z
      }
    }
  }
  *freq=0.5*geno_sum;//freq of allele 1
  fprintf(out," %d %d %d %10.6g %10.6g",*nind,good_ind-*nind,nimputed,geno_sum,geno_sd);
  //printf("Read snp %s, excluding %d individuals\nAvg genotype %g\n",*snp_id,good_ind-*nind,geno_sum);
  fclose(line_str);free(line);
  return 1; 
}


int read_gen(FILE *in,gsl_matrix **X_full,int nind_full,int *nind,int *groupsizes,int *nexcluded_from_group,int **excluded_from_group,char **snp_id,PAR *par,int *good_indexes,double *freq,FILE *out){

  /*GEN file format:
    one line per snp:
    id1 id2 position alleleA alleleB p1_AA p1_AB p1_BB p2_AA p2_AB p2_BB ... pn_AA pn_AB pn_BB
    where pi_AA is the probability that individual i has genotype AA

    returns 
    1 if successful
    2 if file ends
    3 if error reading some z value or if all data are missing
    >3 if error reading some of the first 5 columns, then this many 'NA' should be printed outside of this function
  */

  int i,g,a,nchars,total_index,good_ind,missing,nimputed;
  size_t size;
  double geno[3],geno_sum,counts[4],var_info,frq,ind_geno_sum,t,coeff_frq,coeff_frq2,expect;
  double coeff_frq_all,coeff_frq2_all,sum_missing_all,var_info_all;
  char *line,*info;
  FILE *line_str;

  size=nind_full*8;
  if((line=(char*) malloc(size*sizeof(char)))==NULL)
    alloc_error("line in read_z");

  nchars=getline(&line,&size,in);
  line_str=fmemopen(line,nchars*sizeof(char),"r");

  if(nchars<1 || fscanf(line_str,"%as",&info)!=1){
    free(line);
    if(feof(in))
      return 2;
    else
      return 15;
  }
  fprintf(out,"%s ",info);free(info);
  if(fscanf(line_str,"%as",snp_id)!=1){//2nd column is snp_id
    fclose(line_str);free(line);
    return 14;
  }
  fprintf(out,"%s ",*snp_id);
  for(i=2;i<5;++i){
    if(fscanf(line_str,"%as",&info)!=1){//pos allele_A allele_B
      fclose(line_str);free(line);
      return (15-i);
    }
    fprintf(out,"%s ",info);free(info);
  }

  *nind=0;
  geno_sum=0.0; 
  total_index=0;
  good_ind=0;
  missing=0;
  sum_missing_all=0.0;
  var_info=var_info_all=0.0;
  coeff_frq=coeff_frq2=0.0;
  coeff_frq_all=coeff_frq2_all=0.0;
  for(i=0;i<4;++i) counts[i]=0.0;

  for(g=0;g<ngroups;++g){
    nexcluded_from_group[g]=0;
    for(i=0;i<groupsizes[g];++i){
      while(total_index <= good_indexes[good_ind]){
	for(a=0;a<3;++a)
	  if(fscanf(line_str,"%lf",geno+a)!=1 || geno[a]<0.0 || geno[a]>1.0){
	    fprintf(out,"0 %d 0 NA NA NA NA NA NA NA",nind_full);
	    fprintf(stderr,"reading gen:at id_2='%s'. ERROR with probability of genotype %d of individual %d (i.e. element %d of that line of the gen file)\n",
		    *snp_id,a,total_index+1,5+3*total_index+a+1);
	    fclose(line_str);free(line);
	    return 3;
	  }
	++total_index;
      }
      ind_geno_sum=geno[0]+geno[1]+geno[2];
      if(ind_geno_sum>1.01){
	fprintf(out,"0 %d 0 NA NA NA NA NA NA NA",nind_full);
	fprintf(stderr,"reading gen:at id_2='%s'. ERROR sum of genotype probabilities of individual %d is %g+%g+%g=%g > 1.01\n",
		*snp_id,total_index,geno[0],geno[1],geno[2],ind_geno_sum);
	fclose(line_str);free(line);
	return 3;
      }

      if(ind_geno_sum<par->missing_gen){
	gsl_matrix_set(X_full[g],i,ncovariates,-9);
	excluded_from_group[g][nexcluded_from_group[g]++]=i;
	++missing;
	if(ind_geno_sum>0.0) expect=(geno[1]+2.0*geno[2])/ind_geno_sum;
	else expect=0.0; //value does not matter
	var_info_all+=(geno[1]+4*geno[2])-ind_geno_sum*expect*expect
	  +ind_geno_sum*(1-ind_geno_sum)*expect*expect;//part of info that does not involve pop frequency
	coeff_frq_all-=4*expect*ind_geno_sum*(1-ind_geno_sum);//coeff of pop allele freq
	coeff_frq2_all+=4*ind_geno_sum*(1-ind_geno_sum);//coeff of pop allele freq^2
	sum_missing_all+=1-ind_geno_sum;
      }else{
	gsl_matrix_set(X_full[g],i,ncovariates,(geno[1]+2.0*geno[2])/ind_geno_sum);
	++(*nind);
	if(ind_geno_sum>0.0) expect=(geno[1]+2.0*geno[2])/ind_geno_sum;
	else expect=0.0; //value does not matter
	geno_sum+=expect;
	var_info+=(geno[1]+4*geno[2])-ind_geno_sum*expect*expect
	  +ind_geno_sum*(1-ind_geno_sum)*expect*expect;//part of info that does not involve pop frequency
	coeff_frq-=4*expect*ind_geno_sum*(1-ind_geno_sum);//coeff of pop allele freq
	coeff_frq2+=4*ind_geno_sum*(1-ind_geno_sum);//coeff of pop allele freq^2
	for(a=0;a<3;++a) counts[a]+=geno[a];
	counts[3]+=1-ind_geno_sum;//coeff of HW variance 
      }
      ++good_ind;
    }
  }

  //Check that the line contains exactly as many individuals
  //as there were individuals in the attributes file (without any exclusions)
  while(total_index < par->n_total_ind){
    for(a=0;a<3;++a)
      if(fscanf(line_str,"%lf",geno+a)!=1 || geno[a]<0.0 || geno[a]>1.0){
	fprintf(out,"0 %d 0 NA NA NA NA NA NA NA",nind_full);
	fprintf(stderr,"reading gen:at id_2='%s'. ERROR with probability of genotype %d of individual %d (i.e. element %d of that line of the gen file)\n",
		*snp_id,a,total_index+1,5+3*total_index+a+1);
	fclose(line_str);free(line);
	return 3;
      }
    ++total_index;
  }
  if(fscanf(line_str,"%lf",geno)==1){
    fprintf(out,"0 %d 0 NA NA NA NA NA NA NA",nind_full);
    fprintf(stderr,"reading gen:at id_2='%s'. ERROR too many elements on this line. Expected %d individuals and %d elements.\n",
	    *snp_id,total_index,5+3*total_index);
    fclose(line_str);free(line);
    return 3;
  }

  if(*nind==0 || counts[3]==*nind){
    fprintf(out,"0 %d 0 NA NA NA NA NA NA NA",nind_full);
    fprintf(stderr,"reading gen:'%s'. ERROR no data above non NULL treshold %g\n",*snp_id,par->missing_gen);
    fclose(line_str);free(line);
    return 3;
  }

  nimputed=0;
  geno_sum/=*nind;
  if(*nind<nind_full && par->impute_missing){ //set missing ones to avg genotype
    for(g=0;g<ngroups;++g){
      for(i=0;i<nexcluded_from_group[g];++i){
	gsl_matrix_set(X_full[g],excluded_from_group[g][i],ncovariates,geno_sum);
	++nimputed;
      }
      nexcluded_from_group[g]=0;
    }
    *nind=nind_full;
  }

  if(par->mean_center){
    for(g=0;g<ngroups;++g){
      for(i=0;i<groupsizes[g];++i){
	t=gsl_matrix_get(X_full[g],i,ncovariates);
	if(t!=-9)//geno==-9 possible only if par->impute_missing ==0
	  gsl_matrix_set(X_full[g],i,ncovariates,t-geno_sum);//center genotypes
      }
    }
  }
  sum_missing_all+=counts[3];
  var_info_all+=var_info;
  coeff_frq_all+=coeff_frq;
  coeff_frq2_all+=coeff_frq2;

  frq=(0.5*counts[1]+counts[2])/(*nind-nimputed-counts[3]);

  var_info+=counts[3]*2*frq*(1-frq)+coeff_frq*frq+coeff_frq2*frq*frq;
  var_info=1-var_info/((*nind)*2.0*frq*(1-frq));

  var_info_all+=sum_missing_all*2*frq*(1-frq)+coeff_frq_all*frq+coeff_frq2_all*frq*frq;
  var_info_all=1-var_info_all/((*nind+missing)*2.0*frq*(1-frq));
 
  fprintf(out," %d %d %d %g %g %g %g",*nind,good_ind-(*nind),nimputed,counts[0],counts[1],counts[2],counts[3]);
  fprintf(out," %g %g %g",frq,var_info,var_info_all);
  *freq=frq;//freq of allele 1
  //printf("Read snp %s, excluding %d individuals\nAvg genotype %g\n",*snp_id,good_ind-*nind,geno_sum);
  fclose(line_str);free(line);
  return 1; 
}


double multiplicative_correction_logOR(double gamma,double mu, double f){
  return(1/(1+0.5*(1-2*mu)*(1-2*f)*gamma-(0.084+0.9*mu*(1-2*mu)*f*(1-f))*gamma*gamma));

}

int compute_var_estimates(gsl_vector *beta,double var0,double h,int include_snp,double **var_estimates,gsl_vector *Uy_UXbeta2,gsl_matrix *UX,gsl_vector *Uy,gsl_vector *d,int nind,FILE *out_var,char *snp_id,int estimate_OR){

  int dim,i,j,k;
  double mu0,v,s,s2,dk;
  gsl_matrix *A,*J;
  gsl_matrix_view Asub;
  gsl_vector *Uy_UXbeta;

  for(i=0;i<ncovariates+3;++i)
    var_estimates[include_snp][i]=-1.0;//if information matrix not invertible neg values returned 
  
  if(h<1e-9){//h=0 so take sd errors from the standard linear model
    return compute_var_estimates_without_h(beta,var0,include_snp,var_estimates,Uy_UXbeta2,UX,out_var,snp_id,estimate_OR);
  }					     

  dim=ncovariates+2+include_snp;
  A=gsl_matrix_calloc(dim,dim);//collects minus 2nd derivatives and then inverts

  //(dL)^2/(dbeta)^2  = - UX^T Sigma^-1 UX
  for(i=0;i<ncovariates+include_snp;++i)
    for(j=0;j<=i;++j){
      v=0.0;
      for(k=0;k<nind;++k)
	v+=gsl_matrix_get(UX,k,i)*gsl_matrix_get(UX,k,j)/(h*gsl_vector_get(d,k)+1-h);
      v=v/var0;
      gsl_matrix_set(A,i,j,v);gsl_matrix_set(A,j,i,v);
    }

  //(dL)^2/dbeta dsigma=0 at MLE (beta and sigma are asympt orthogonal)
  //(dL)^2/dsigma
  v=0.0;
  for(k=0;k<nind;++k)
    v+=gsl_vector_get(Uy_UXbeta2,k)/(h*gsl_vector_get(d,k)+1-h);
  gsl_matrix_set(A,dim-2,dim-2,-nind/2.0/var0/var0+v/var0/var0/var0);//changed 19-Sep-2012

  Uy_UXbeta=gsl_vector_alloc(nind);
  gsl_blas_dcopy (Uy,Uy_UXbeta);
  gsl_blas_dgemv (CblasNoTrans, -1.0, UX, beta, 1.0, Uy_UXbeta);
 
  //(dL)^2/dbeta dh & (dL)^2/dsigma dh & (dL)^2/dH dh 
  for(k=0;k<nind;++k){
    dk=gsl_vector_get(d,k);
    v=h*dk+(1-h);
    s2=gsl_vector_get(Uy_UXbeta2,k);
    s=gsl_vector_get(Uy_UXbeta,k);
    for(i=0;i<ncovariates+include_snp;++i)
      gsl_matrix_set(A,i,dim-1,gsl_matrix_get(A,i,dim-1)+(dk-1)*s*gsl_matrix_get(UX,k,i)/v/v);
    gsl_matrix_set(A,dim-2,dim-1,gsl_matrix_get(A,dim-2,dim-1)+s2*(dk-1)/v/v);
    gsl_matrix_set(A,dim-1,dim-1,gsl_matrix_get(A,dim-1,dim-1)+(dk-1)*(dk-1)/v/v*(2*s2/var0/v-1));
  }
  for(i=0;i<ncovariates+include_snp;++i){
    gsl_matrix_set(A,i,dim-1,gsl_matrix_get(A,i,dim-1)/var0);
    gsl_matrix_set(A,dim-1,i,gsl_matrix_get(A,i,dim-1));
  }
  gsl_matrix_set(A,dim-2,dim-1,gsl_matrix_get(A,dim-2,dim-1)/2.0/var0/var0);
  gsl_matrix_set(A,dim-1,dim-2,gsl_matrix_get(A,dim-2,dim-1));
  gsl_matrix_set(A,dim-1,dim-1,gsl_matrix_get(A,dim-1,dim-1)/2.0);
  gsl_vector_free(Uy_UXbeta);
 
  /*
  printf("Information A:\n");
  for(i=0;i<dim;++i){
    for(j=0;j<dim;++j)
      printf("%g ",gsl_matrix_get(A,i,j));
    printf("\n");
  }
  */
  gsl_set_error_handler_off ();//Check return value of cholesky manually
  if(gsl_linalg_cholesky_decomp (A)==GSL_EDOM){//A not positive semi-definite
    gsl_matrix_free(A);
    //printf("not pos def\n");
    gsl_set_error_handler (NULL);
    return 0;
  }
  gsl_set_error_handler (NULL);//Return the default error handler
  gsl_linalg_cholesky_invert (A);

  if(estimate_OR){
    mu0=gsl_vector_get(beta,0);//1st covariate must be the MEAN
    J=gsl_matrix_calloc(ncovariates+include_snp,ncovariates+include_snp);//Jacobian (d(logistic param)/d(linear param))
    gsl_matrix_set(J,0,0,1.0);
    for(i=1;i<ncovariates+include_snp;++i){
      gsl_matrix_set(J,i,i,1.0);
      gsl_matrix_set(J,i,0,-gsl_vector_get(beta,i)*(1-2*mu0)/mu0/(1-mu0));
    }//if include_snp==0 then last row and column oj J are full of zeros
    gsl_matrix_scale (J,1/mu0/(1-mu0));
    Asub=gsl_matrix_submatrix (A, 0, 0, ncovariates+include_snp, ncovariates+include_snp);
    gsl_blas_dtrmm (CblasLeft, CblasLower, CblasNoTrans, CblasNonUnit, 1.0, J, &(Asub.matrix));//A = J A 
    gsl_blas_dtrmm (CblasRight, CblasLower, CblasTrans, CblasNonUnit, 1.0, J, &(Asub.matrix));//A = A J^T 
    gsl_matrix_free(J);
  }
  for(i=0;i<ncovariates+include_snp;++i)
    var_estimates[include_snp][i]=gsl_matrix_get(A,i,i);
  for(i=ncovariates+include_snp;i<dim;++i)
    var_estimates[include_snp][i+(1-include_snp)]=gsl_matrix_get(A,i,i);

  gsl_matrix_free(A);
  return 1;

}


int compute_var_estimates_without_h(gsl_vector *beta,double var0,int include_snp,double **var_estimates,gsl_vector *Uy_UXbeta2,gsl_matrix *UX,FILE *out_var,char *snp_id,int estimate_OR){

  int dim,i,n;
  double mu0;
  gsl_matrix *A,*J=NULL;

  dim=ncovariates+1;//ncovariates and the snp but no variance parameter(s)
  A=gsl_matrix_calloc(dim,dim);//-2nd derivatives -> inverse
  for(i=0;i<ncovariates+3;++i)
    var_estimates[include_snp][i]=-1.0;//if information matrix not invertible neg values returned 

  //(dL)^2/(dbeta)^2  = 1/var0*X^t X=1/var0 * UX^t UX
  gsl_blas_dgemm (CblasTrans, CblasNoTrans, 1/var0, UX, UX, 0.0, A); //A=1/var0*UX^t UX
  if(!include_snp){//set last col and row to 0, except diagonal to 1
    for(i=0;i<ncovariates;++i){
      gsl_matrix_set(A,ncovariates,i,0.0);
      gsl_matrix_set(A,i,ncovariates,0.0);
    }
    gsl_matrix_set(A,ncovariates,ncovariates,1.0);
  }
 
  //(dL)^2/dbeta dsigma=0 at MLE (beta and sigma are asympt orthogonal)
  //(dL)^2/dsigma
  n=(int) Uy_UXbeta2->size;
  var_estimates[include_snp][ncovariates+1]=1/(-n/2.0/var0/var0+gsl_blas_dasum (Uy_UXbeta2)/var0/var0/var0);// Changed 19-Sep-2012
  //gsl_matrix_set(A,dim-1,dim-1,-nind/var0+3/var0/var0*v);

  /*
  printf("Information A:\n");
  for(i=0;i<dim;++i){
    for(j=0;j<dim;++j)
      printf("%g ",gsl_matrix_get(A,i,j));
    printf("\n");
  }
  */
  gsl_set_error_handler_off ();//Check return value of cholesky manually
  if(gsl_linalg_cholesky_decomp (A)==GSL_EDOM){//A not positive semi-definite
    gsl_matrix_free(A);
    //printf("not pos def");
    return 0;
  }
  gsl_set_error_handler (NULL);//Return the default error handler
  gsl_linalg_cholesky_invert (A);

  if(estimate_OR){
    mu0=gsl_vector_get(beta,0);//1st covariate must be the MEAN
    J=gsl_matrix_calloc(dim,dim);//Jacobian (d(logistic param)/d(linear param))
    gsl_matrix_set(J,0,0,1.0);
    for(i=1;i<ncovariates+include_snp;++i){
      gsl_matrix_set(J,i,i,1.0);
      gsl_matrix_set(J,i,0,-gsl_vector_get(beta,i)*(1-2*mu0)/mu0/(1-mu0));
    }//if include_snp==0 then last row and column oj J are full of zeros
    gsl_matrix_scale (J,1/mu0/(1-mu0));    
    gsl_blas_dtrmm (CblasLeft, CblasLower, CblasNoTrans, CblasNonUnit, 1.0, J, A);//A = J A 
    gsl_blas_dtrmm (CblasRight, CblasLower, CblasTrans, CblasNonUnit, 1.0, J, A);//A = A J^T 
    gsl_matrix_free(J);
  }
  for(i=0;i<ncovariates+include_snp;++i)
    var_estimates[include_snp][i]=gsl_matrix_get(A,i,i);

  gsl_matrix_free(A);
  return 1;

}
	 
int maximise_lkhood(int include_last_beta,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *Uy_UXbeta2, gsl_vector *d,double min_d,gsl_vector *beta,double *var0,double *h,double *lk,int nind,void *fparams,PAR *par,FILE *out){

  int i,converged=0,iter=0;
  double lk_prev;

  if(par->fix_eta){
    if(!update_beta(include_last_beta,Uy,UX,d,beta,*var0,*h,nind)) 
      return converged;
    gsl_blas_dcopy (Uy,Uy_UXbeta2);
    gsl_blas_dgemv (CblasNoTrans, -1.0, UX, beta, 1.0, Uy_UXbeta2);
    update_var0(var0,*h,Uy_UXbeta2,d,nind); //also squares each element of Uy_Uxbeta
    *lk=loglkhood(Uy,UX,beta,*var0,*h,d,nind);
    converged=1;
  }
  else{

    lk_prev=loglkhood(Uy,UX,beta,*var0,*h,d,nind);

    do{
    
      if(!update_beta(include_last_beta,Uy,UX,d,beta,*var0,*h,nind)) 
	return converged;
      
      gsl_blas_dcopy (Uy,Uy_UXbeta2);
      gsl_blas_dgemv (CblasNoTrans, -1.0, UX, beta, 1.0, Uy_UXbeta2);
      
      update_var0(var0,*h,Uy_UXbeta2,d,nind); //also squares each element of Uy_Uxbeta
      
      //update_h(h,FDF,fdf_solver,fparams,n_maxim_intervals,Uy,UX,d,beta,*var0,nind);
      update_h_without_GSL(h,fparams,par,Uy,UX,d,min_d,beta,*var0,nind);
      
      
      *lk=loglkhood(Uy,UX,beta,*var0,*h,d,nind);
      if(out != NULL){
	for(i=0;i<ncovariates+1;++i)
	  fprintf(out,"%g ",gsl_vector_get(beta,i));
	fprintf(out,"%g %g %15.6f\n",*var0,*h,*lk);
      }
      converged=(*lk-lk_prev<par->tol);
      lk_prev=*lk;
      ++iter;
    }while(!converged && iter<par->max_iterations);
  }

  return converged;

}

void update_h_without_GSL(double *h,void *fparams,PAR *par,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,double min_d,gsl_vector *beta,double var0,int nind){

  int i,status,iter,nroots,division,found_within_interval;
  double f,max_f,max_h,x,der_prev,der,der_middle,left_der,right_der,step,left_end,right_end;
  double d1,d2,pos=0.0;

  der_prev=dLk_dh(pos,fparams);
  max_f=loglkhood(Uy,UX,beta,var0,pos,d,nind);
  max_h=pos;
  nroots=0;

  if(min_d>MIN_D_TOLERANCE) step=1.0/par->max_eta_intervals;//no constrains on h
  else step=0.99/par->max_eta_intervals; //smallest d is ~ 0, constrain h<0.99

  for(i=0;i<par->max_eta_intervals;++i){
    pos+=step;
    der=dLk_dh(pos,fparams);
    if(der<0.0 && der_prev>0.0){ //look for maximum only
      ++nroots;
      //search for a root within the given interval
      found_within_interval=0;
      division=1;
      left_end=pos-step;
      right_end=pos;
      left_der=der_prev;
      right_der=der;
      while(!found_within_interval && division<=par->max_eta_divisions){
	x=(right_end+left_end)*0.5;
	iter=0;
	status=0;
	while(fabs(d1=dLk_dh(x,fparams))>par->tol && !status)
	  {
	    iter++;
	    d2=d2Lk_dh2(x,fparams);
	    
	    if(fabs(d2)>EPSILON)
	      x=x-d1/d2;
	    else
	      status=1;
	    if(iter==par->max_eta_iterations)
	      status=2;
	    if(x<0 || x>1)
	      status=3;
	  }

	if(status!=3){
	  f=loglkhood(Uy,UX,beta,var0,x,d,nind);
	  if(f>max_f){max_f=f;max_h=x;}
	}
	if(x>=left_end && x<=right_end && !status){
	  //printf("%g (%g)< %g < %g (%g) %g\n",left_end,left_der,x,right_end,right_der,f);
	  found_within_interval=1;
	}
	
	if(!found_within_interval){
	  //printf("interval=(%g,%g),division=%d\n",left_end,right_end,division);
	  der_middle=dLk_dh((left_end+right_end)*0.5,fparams);
	  if(left_der*der_middle<0){
	    right_end=0.5*(left_end+right_end);
	    right_der=der_middle;
	  }
	  else{
	    left_end=0.5*(left_end+right_end);
	    left_der=der_middle;
	  }
	  division++;
	}
      }
      if(!found_within_interval){
	fprintf(stderr,"Failed to converge within interval with given parameters!\n");
	f=loglkhood(Uy,UX,beta,var0,left_end,d,nind);
	if(f>max_f){max_f=f;max_h=left_end;}
	f=loglkhood(Uy,UX,beta,var0,right_end,d,nind);
	if(f>max_f){max_f=f;max_h=right_end;}
      }
    }
    der_prev=der;
  }
  f=loglkhood(Uy,UX,beta,var0,pos,d,nind);
  if(f>max_f){max_f=f;max_h=pos;}
  *h=max_h;

  //if(nroots>1) printf("%d roots\n",nroots);
    
}



double dLk_dh(double h, void * par)
{
  double d,a,sum=0.0;
  int i;
  FPAR *fparams=(FPAR*) par;

  for(i=0;i<fparams->nind;++i){
    d=gsl_vector_get(fparams->d,i)-1;
    a=1+h*d;
    sum+=(d/a)*(1-gsl_vector_get(fparams->Uy_UXbeta2,i)/(*(fparams->var0))/a);
  }
  return -0.5*sum;
}

double d2Lk_dh2(double h, void * par)
{
  double d,a,sum=0.0;
  int i;
  FPAR *fparams=(FPAR*) par;
  for(i=0;i<fparams->nind;++i){
    d=gsl_vector_get(fparams->d,i)-1;
    a=1+h*d;
    sum+=(d*d/a/a)*(1-2*gsl_vector_get(fparams->Uy_UXbeta2,i)/(*(fparams->var0))/a);
  }
  return 0.5*sum;
}

void dLk_d2Lk_dh(double h, void * par,double *d1,double *d2)
{
  double d,a,v;
  int i;
  FPAR *fparams=(FPAR*) par;
  *d1=0.0;
  *d2=0.0;

  for(i=0;i<fparams->nind;++i){
    d=gsl_vector_get(fparams->d,i)-1;
    a=1+h*d;
    v=gsl_vector_get(fparams->Uy_UXbeta2,i)/(*(fparams->var0))/a;
    *d2+=(d*d/a/a)*(1-2*v);
    *d1+=(d/a)*(1-v);
  }
  *d1=-0.5*(*d1);
  *d2=0.5*(*d2);
}

int initialize_integration_variables(double *aa_const,double *logdet_theta,double *logbeta,gsl_vector **UX_theta_Uy,gsl_matrix **UX_theta_UX,gsl_vector **theta_Uy,gsl_matrix **theta_UX,gsl_matrix *inv_prior_var,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,double logmlk[2],double logmlk_h0[2],double *logdet_prior_ratio_1to0,int ncovariates,int nind_full,double min_d,PAR *par){

  int i,j,k,ind;
  double a_mVm,a_m_Vsub_m,y_theta_y,integral,integrand_max,h,logdet_Vss,mss_Vss_mss;
  double *integrands,logbeta_const,max_h;
  gsl_vector *tmp_vec,*tmp_vec2,*msub,*mss;
  gsl_matrix *Vsub,*Vss,*Vss2;
  int success;

  logbeta_const=-gsl_sf_lnbeta (par->prior_beta_1, par->prior_beta_0);//Gamma(b_0+b_1)/Gamma(b_0)/Gamma(b_1)

  if((integrands=(double *) calloc(par->n_integr_intervals+1,sizeof(double)))==NULL)
    alloc_error("integrands");

  success=1;
  tmp_vec=gsl_vector_alloc(ncovariates+1);
  gsl_blas_dgemv (CblasNoTrans, 1.0, inv_prior_var, par->prior_exp, 0.0, tmp_vec); //tmp_vec=V^-1 m
  gsl_blas_ddot (par->prior_exp, tmp_vec,&a_mVm);//a_mVm=m' V^-1 m
  gsl_vector_free(tmp_vec);
  a_mVm+=par->prior_a;

  //a_mVm = prior_a + m' V^-1 m
  //printf("a_mVm=%g \n",a_mVm);

  Vsub=gsl_matrix_calloc(ncovariates,ncovariates);
  msub=gsl_vector_calloc(ncovariates);
  for(i=0;i<ncovariates;++i){
    gsl_vector_set(msub,i,gsl_vector_get(par->prior_exp,i));
    for(j=0;j<ncovariates;++j)
      gsl_matrix_set(Vsub,i,j,gsl_matrix_get(par->prior_var,i,j));
  }

  //Vsub and msub are V and m w/o last column
  gsl_set_error_handler_off ();//Check return value of cholesky manually
  if(gsl_linalg_cholesky_decomp(Vsub)==GSL_EDOM){
    success=0;
    fprintf(stderr,"Sub-matrix of prior variance is not positive definite!\nExits!\n");
    gsl_matrix_free(Vsub);gsl_vector_free(msub);
  }
  gsl_set_error_handler (NULL);//Return the default error handler

  for(i=0;i<ncovariates;++i)
    (*logdet_prior_ratio_1to0)-=2.0*log(gsl_matrix_get(Vsub,i,i)); //previously logdet_prior_ratio_1to0=log(det(prior_var))

  //logdet_prior_ratio_1to0=log(det(V))-log(det(Vsub)), where V=prior_var 
  //printf("%g\n",*logdet_prior_ratio_1to0);

  gsl_linalg_cholesky_invert (Vsub);

  tmp_vec=gsl_vector_alloc(ncovariates);
  gsl_blas_dgemv (CblasNoTrans, 1.0, Vsub, msub, 0.0, tmp_vec); //tmp_vec=V_sub^-1 m_sub
  gsl_blas_ddot (msub, tmp_vec,&a_m_Vsub_m);//a_m_Vsub_m=msub' Vsub^-1 msub
  a_m_Vsub_m+=par->prior_a;
  //do not free tmp_vec here

  //a_m_Vsub_m= prior_a + msub' inv(Vsub) msub

  tmp_vec2=gsl_vector_alloc(ncovariates);
  mss=gsl_vector_alloc(ncovariates);
  Vss=gsl_matrix_calloc(ncovariates,ncovariates);
  Vss2=gsl_matrix_calloc(ncovariates,ncovariates);
  integrand_max=-1.0/0.0;

  if(min_d<MIN_D_TOLERANCE) max_h=0.99;//constrain h to [0,0.99]
  else max_h=1.0;//h in [0,1.0]
  
  for(i=0;i<par->n_integr_intervals+1;++i){

    h=max_h*i/par->n_integr_intervals; 

    logbeta[i]=logbeta_const;
    if(par->prior_beta_1>1) logbeta[i]+=(par->prior_beta_1-1)*log(h);
    if(par->prior_beta_0>1) logbeta[i]+=(par->prior_beta_0-1)*log(1-h);
  
    if(theta_Uy[i]==NULL){
      theta_Uy[i]=gsl_vector_calloc(nind_full);
      UX_theta_Uy[i]=gsl_vector_calloc(ncovariates+1);
      theta_UX[i]=gsl_matrix_calloc(nind_full,ncovariates+1);
      UX_theta_UX[i]=gsl_matrix_calloc(ncovariates+1,ncovariates+1);
    }
    logdet_theta[i]=0.0;
    for(ind=0;ind<nind_full;++ind){
      gsl_vector_set(theta_Uy[i],ind,gsl_vector_get(Uy,ind)/(h*(gsl_vector_get(d,ind)-1)+1));
      for(j=0;j<ncovariates+1;++j)
	gsl_matrix_set(theta_UX[i],ind,j,gsl_matrix_get(UX,ind,j)/(h*(gsl_vector_get(d,ind)-1)+1));
      logdet_theta[i]+=log(h*(gsl_vector_get(d,ind)-1)+1);
    }

    gsl_blas_dgemm (CblasTrans, CblasNoTrans, 1.0, UX, theta_UX[i], 0.0, UX_theta_UX[i]); //UX_theta_UX[i]=(UX)^T theta UX
    gsl_blas_dgemv (CblasTrans, 1.0, UX, theta_Uy[i], 0.0, UX_theta_Uy[i]);//UX_theta_Uy[i]=(UX)^T theta Uy

    gsl_blas_ddot (Uy, theta_Uy[i], &y_theta_y);
    aa_const[i]=a_mVm+y_theta_y;

    //Compute integral for the model with no snp
    //tmp_vec=V_sub^-1 m_sub
    for(j=0;j<ncovariates;++j){
      gsl_vector_set(tmp_vec2,j,gsl_vector_get(tmp_vec,j)+gsl_vector_get(UX_theta_Uy[i],j));//tmp_vec2=Vsub^-1 msub+UX_theta_Uy
      for(k=0;k<ncovariates;++k)
	gsl_matrix_set(Vss,j,k,gsl_matrix_get(UX_theta_UX[i],j,k));
    }
    gsl_matrix_add (Vss, Vsub); //Vss=Vsub+UX_theta_UX
    gsl_matrix_memcpy (Vss2, Vss);

    gsl_set_error_handler_off ();
    if(gsl_linalg_cholesky_decomp(Vss2)!=GSL_EDOM){
      gsl_set_error_handler (NULL);//Return the default error handler
      logdet_Vss=0.0;
      for(j=0;j<ncovariates;++j)
	logdet_Vss-=2.0*log(gsl_matrix_get(Vss2,j,j));
      gsl_linalg_cholesky_solve (Vss2, tmp_vec2, mss);//mss=inv(Vss)*tmp_vec2
      gsl_blas_dgemv (CblasNoTrans, 1.0, Vss, mss, 0.0, tmp_vec2); //tmp_vec2=V_ss mss
      gsl_blas_ddot (mss, tmp_vec2, &mss_Vss_mss);

      integrands[i]=0.5*(-(nind_full+par->prior_d)*log(a_m_Vsub_m+y_theta_y-mss_Vss_mss)+logdet_Vss-logdet_theta[i]);
      if(i==0) logmlk_h0[0]=integrands[0];
      integrands[i]+=logbeta[i];

      if(integrands[i]>integrand_max)
	integrand_max=integrands[i];
    }else{
      gsl_set_error_handler (NULL);//Return the default error handler
      fprintf(stderr,"Initialising integration variables: Vss not pos def!\nExits!\n");
      success=0;
      break;
    }
    //printf("%g: %g \n",h,integrands[i]);
  }

  if(success){
    integral=0.5*exp(integrands[0]-integrand_max);
    for(i=1;i<par->n_integr_intervals;++i)
      integral+=exp(integrands[i]-integrand_max);
    integral+=0.5*exp(integrands[par->n_integr_intervals]-integrand_max);    
    logmlk[0]=log(integral)+integrand_max+log(max_h/par->n_integr_intervals);
  }
  
  gsl_vector_free(tmp_vec);
  gsl_vector_free(tmp_vec2);
  gsl_vector_free(msub);
  gsl_vector_free(mss);
  gsl_matrix_free(Vsub);
  gsl_matrix_free(Vss);
  gsl_matrix_free(Vss2);
  free(integrands);

  return success;
}

int compute_mlk(double *aa_const,double *logdet_theta,double *logbeta,gsl_vector **UX_theta_Uy,gsl_matrix **UX_theta_UX,gsl_vector **theta_Uy,gsl_matrix **theta_UX,gsl_matrix *inv_prior_var,gsl_matrix *UX,gsl_vector *d,double logmlk_h0[2],double logmlk[2],int ncovariates,int nind_full,double min_d,PAR *par){


  int i,j,ind,success=1;
  double integral,*integrands,integrand_max,h,logdet_Vs,ms_Vs_ms,v,max_h;
  gsl_vector *tmp_vec,*tmp_vec2,*ms;
  gsl_matrix *Vs,*Vs2;
  //FILE *output=fopen("h_posterior.out","w");

  if((integrands=(double *) calloc(par->n_integr_intervals+1,sizeof(double)))==NULL)
    alloc_error("integrands");

  tmp_vec=gsl_vector_alloc(ncovariates+1);
  gsl_blas_dgemv (CblasNoTrans, 1.0, inv_prior_var, par->prior_exp, 0.0, tmp_vec); //tmp_vec=V^-1 m

  tmp_vec2=gsl_vector_alloc(ncovariates+1);
  ms=gsl_vector_alloc(ncovariates+1);
  Vs=gsl_matrix_calloc(ncovariates+1,ncovariates+1);
  Vs2=gsl_matrix_calloc(ncovariates+1,ncovariates+1);

  if(min_d<MIN_D_TOLERANCE) max_h=0.99;//constrain h to [0,0.99]
  else max_h=1.0;//h in [0,1.0]
 
  integrand_max=-1.0/0.0;
  for(i=0;i<par->n_integr_intervals+1;++i){

    h=max_h*i/par->n_integr_intervals; 

    v=0.0;
    for(ind=0;ind<nind_full;++ind){
      v+=gsl_matrix_get(UX,ind,ncovariates)*gsl_vector_get(theta_Uy[i],ind);
      gsl_matrix_set(theta_UX[i],ind,ncovariates,gsl_matrix_get(UX,ind,ncovariates)/(h*(gsl_vector_get(d,ind)-1)+1));
    }
    gsl_vector_set(UX_theta_Uy[i],ncovariates,v);

    for(j=0;j<ncovariates+1;++j){
      v=0.0;
      for(ind=0;ind<nind_full;++ind)
	v+=gsl_matrix_get(UX,ind,ncovariates)*gsl_matrix_get(theta_UX[i],ind,j);
      gsl_matrix_set(UX_theta_UX[i],j,ncovariates,v);
      gsl_matrix_set(UX_theta_UX[i],ncovariates,j,v);
    }
    gsl_matrix_memcpy (Vs, inv_prior_var);
    gsl_matrix_add (Vs, UX_theta_UX[i]); //Vs=inv(V)+UX_theta_UX
    gsl_matrix_memcpy (Vs2, Vs);
    gsl_vector_memcpy (tmp_vec2, tmp_vec);
    gsl_vector_add(tmp_vec2,UX_theta_Uy[i]);//tmp_vec2=inv(V)*m+UX_theta_Uy

    gsl_set_error_handler_off ();
    if(!(success=(gsl_linalg_cholesky_decomp(Vs2)!=GSL_EDOM))){
      gsl_set_error_handler (NULL);
      break;
    }
    gsl_set_error_handler (NULL);//Return the default error handler
    
    logdet_Vs=0.0;
    for(j=0;j<ncovariates+1;++j)
      logdet_Vs-=2.0*log(gsl_matrix_get(Vs2,j,j));
    //printf("logdet_Vs=%g\n",logdet_Vs);
    gsl_linalg_cholesky_solve (Vs2, tmp_vec2, ms);//ms=inv(Vss)*tmp_vec2
    gsl_blas_dgemv (CblasNoTrans, 1.0, Vs, ms, 0.0, tmp_vec2); //tmp_vec2=Vs ms
    gsl_blas_ddot (ms, tmp_vec2, &ms_Vs_ms);

    integrands[i]=0.5*(-(nind_full+par->prior_d)*log(aa_const[i]-ms_Vs_ms)+logdet_Vs-logdet_theta[i]);
    if(i==0) logmlk_h0[1]=integrands[0];
    integrands[i]+=logbeta[i];

    if(integrands[i]>integrand_max)
      integrand_max=integrands[i];

    //fprintf(output,"%g %20.15g\n",h,integrands[i]);

  }
  //fclose(output);
  if(success){
    integral=0.5*exp(integrands[0]-integrand_max);
    for(i=1;i<par->n_integr_intervals;++i)
      integral+=exp(integrands[i]-integrand_max);
    integral+=0.5*exp(integrands[par->n_integr_intervals]-integrand_max);
    logmlk[1]=log(integral)+integrand_max+log(max_h/par->n_integr_intervals);
  }
  gsl_vector_free(tmp_vec);
  gsl_vector_free(tmp_vec2);
  gsl_vector_free(ms);
  gsl_matrix_free(Vs);
  gsl_matrix_free(Vs2);
  free(integrands);

  return success;
}

double compute_log10BF_eta_qags(gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,int ncovariates,int nind_full,PAR *par){

  gsl_function F_mlk_at_h;
  gsl_integration_workspace *int_work;
  int int_work_size=1000;
  double int_result,int_error,mlk_h0;
  struct F_MLK_AT_H_PARAMS f_params;

  f_params.Uy=Uy;
  f_params.UX=UX;
  f_params.d=d;
  f_params.ncovariates=ncovariates;
  f_params.nind=nind_full;
  f_params.par=par;


  F_mlk_at_h.function = &compute_mlk_at_h;
  F_mlk_at_h.params = &f_params;
  int_work=gsl_integration_workspace_alloc (int_work_size);

  //model with point mass at eta==0.0
  f_params.base_const=0.0;f_params.log_scale=1;f_params.use_beta_prior=0;
  mlk_h0=compute_mlk_at_h(0.0, (void *)(&f_params));

  //model with beta prior on eta 
  f_params.base_const=mlk_h0;f_params.log_scale=0;f_params.use_beta_prior=1;
  gsl_integration_qags (&F_mlk_at_h, 0, 1.0, 0, 1e-7, int_work_size,int_work, &int_result, &int_error); 
  
  //printf("QAGS: log10BF=%g log_mlk_h0=%g error=%g\n",log(int_result)/log(10),mlk_h0/log(10),int_error);

  gsl_integration_workspace_free(int_work);

  return(log(int_result)/log(10));

}

double compute_mlk_at_h(double h,void *p){

  int i,j,ind;
  double a_m_Vsub_m,y_theta_y,logdet_Vss,mss_Vss_mss,v;
  double logbeta,logdet_theta;
  gsl_vector *tmp_vec,*msub,*mss,*UX_theta_Uy,*theta_Uy;
  gsl_matrix *Vsub,*Vss,*Vss2,*UX_theta_UX,*theta_UX;
  gsl_matrix_view UX_sub;
  int success;

  struct F_MLK_AT_H_PARAMS *Fp= (struct F_MLK_AT_H_PARAMS *) p;
  
  double base_const=Fp->base_const;
  int log_scale=Fp->log_scale;
  int use_beta_prior=Fp->use_beta_prior;
  gsl_vector *Uy=Fp->Uy;
  gsl_matrix *UX=Fp->UX;
  gsl_vector *d=Fp->d;
  int ncovariates=Fp->ncovariates;
  int nind_full=Fp->nind;
  PAR *par=Fp->par;

  success=1;

  Vsub=gsl_matrix_calloc(ncovariates,ncovariates);
  msub=gsl_vector_calloc(ncovariates);
  for(i=0;i<ncovariates;++i){
    gsl_vector_set(msub,i,gsl_vector_get(par->prior_exp,i));
    for(j=0;j<ncovariates;++j)
      gsl_matrix_set(Vsub,i,j,gsl_matrix_get(par->prior_var,i,j));
  }

  //Vsub and msub are V and m w/o last column
  gsl_set_error_handler_off ();//Check return value of cholesky manually
  if(gsl_linalg_cholesky_decomp(Vsub)==GSL_EDOM){
    success=0;
    fprintf(stderr,"Sub-matrix of prior variance is not positive definite!\nExits!\n");
    gsl_matrix_free(Vsub);gsl_vector_free(msub);
  }
  gsl_set_error_handler (NULL);//Return the default error handler

  gsl_linalg_cholesky_invert (Vsub);

  tmp_vec=gsl_vector_alloc(ncovariates);
  gsl_blas_dgemv (CblasNoTrans, 1.0, Vsub, msub, 0.0, tmp_vec); //tmp_vec=V_sub^-1 m_sub
  gsl_blas_ddot (msub, tmp_vec,&a_m_Vsub_m);//a_m_Vsub_m=msub' Vsub^-1 msub
  a_m_Vsub_m+=par->prior_a;
  //so far nothing depends on h, could be done outside of this functions
  //do not free tmp_vec here

  //a_m_Vsub_m= prior_a + msub' inv(Vsub) msub

  mss=gsl_vector_alloc(ncovariates);
  Vss=gsl_matrix_calloc(ncovariates,ncovariates);
  Vss2=gsl_matrix_calloc(ncovariates,ncovariates);

  if(use_beta_prior){
    logbeta=-gsl_sf_lnbeta (par->prior_beta_1, par->prior_beta_0);//Gamma(b_0+b_1)/Gamma(b_0)/Gamma(b_1)
    if(par->prior_beta_1>1) logbeta+=(par->prior_beta_1-1)*log(h);
    if(par->prior_beta_0>1) logbeta+=(par->prior_beta_0-1)*log(1-h);
  } else logbeta=0.0;

  theta_Uy=gsl_vector_calloc(nind_full);
  UX_theta_Uy=gsl_vector_calloc(ncovariates);
  theta_UX=gsl_matrix_calloc(nind_full,ncovariates);
  UX_theta_UX=gsl_matrix_calloc(ncovariates,ncovariates);

  logdet_theta=0.0;
  for(ind=0;ind<nind_full;++ind){
    gsl_vector_set(theta_Uy,ind,gsl_vector_get(Uy,ind)/(h*(gsl_vector_get(d,ind)-1)+1));
    for(j=0;j<ncovariates;++j)
      gsl_matrix_set(theta_UX,ind,j,gsl_matrix_get(UX,ind,j)/(h*(gsl_vector_get(d,ind)-1)+1));
    logdet_theta+=log(h*(gsl_vector_get(d,ind)-1)+1);
  }

  UX_sub=gsl_matrix_submatrix (UX, 0, 0,nind_full ,ncovariates);
  gsl_blas_dgemm (CblasTrans, CblasNoTrans, 1.0, &UX_sub.matrix, theta_UX, 0.0, UX_theta_UX); //UX_theta_UX=(UX)^T theta UX
  gsl_blas_dgemv (CblasTrans, 1.0, &UX_sub.matrix, theta_Uy, 0.0, UX_theta_Uy);//UX_theta_Uy=(UX)^T theta Uy
  gsl_blas_ddot (Uy, theta_Uy, &y_theta_y);


  //Compute marg likelihood for the model with no snp at eta=h
  //tmp_vec=V_sub^-1 m_sub
  gsl_vector_add (tmp_vec, UX_theta_Uy);//tmp_vec=V_sub^-1 m_sub + UX_theta_Uy
  gsl_matrix_memcpy (Vss,UX_theta_UX);//Vss=UX_theta_UX

  gsl_matrix_add (Vss, Vsub); //Vss=Vsub+UX_theta_UX
  gsl_matrix_memcpy (Vss2, Vss);

  gsl_set_error_handler_off ();
  if(gsl_linalg_cholesky_decomp(Vss2)!=GSL_EDOM){
    gsl_set_error_handler (NULL);//Return the default error handler
    logdet_Vss=0.0;
    for(j=0;j<ncovariates;++j)
      logdet_Vss-=2.0*log(gsl_matrix_get(Vss2,j,j));
    gsl_linalg_cholesky_solve (Vss2, tmp_vec, mss);//mss=inv(Vss)*tmp_vec
    gsl_blas_dgemv (CblasNoTrans, 1.0, Vss, mss, 0.0, tmp_vec); //tmp_vec=V_ss mss
    gsl_blas_ddot (mss, tmp_vec, &mss_Vss_mss);

    v=logbeta+0.5*(-(nind_full+par->prior_d)*log(a_m_Vsub_m+y_theta_y-mss_Vss_mss)+logdet_Vss-logdet_theta);

  }else{
    gsl_set_error_handler (NULL);//Return the default error handler
    fprintf(stderr,"Initialising integration variables: Vss not pos def!\nExits!\n");
    success=0;
  }
  //printf("%g: %g \n",h,integrands[i]);
  
  gsl_vector_free(tmp_vec);
  gsl_vector_free(msub);
  gsl_vector_free(mss);
  gsl_matrix_free(Vsub);
  gsl_matrix_free(Vss);
  gsl_matrix_free(Vss2);
  gsl_vector_free(theta_Uy);
  gsl_vector_free(UX_theta_Uy);
  gsl_matrix_free(theta_UX);
  gsl_matrix_free(UX_theta_UX);
  if(!success){
    fprintf(stderr,"ERROR in computing marginal likelihood at eta=%g\nExits!\n",h);
    exit(1);
  }
  if(log_scale) {return(v);}
  else{return(exp(v-base_const));}

}



int initialize_params(int include_last_beta,gsl_vector *beta,double *var0,double *h,gsl_matrix *UX, gsl_vector *Uy,gsl_vector *Uy_UXbeta2,int nind){

  int i,j;
  double sqsum=0.0;
  gsl_matrix *C,*Z;
  gsl_vector *c,*z;

  C=gsl_matrix_calloc(ncovariates+1,ncovariates+1);
  gsl_blas_dgemm (CblasTrans,CblasNoTrans , 1.0, UX, UX, 0.0, C); //C=UX^t * UX = (U^tX)^t U^tX=X^t X
  c=gsl_vector_calloc(ncovariates+1);
  gsl_blas_dgemv (CblasTrans, 1.0, UX, Uy, 0.0, c);//c=X'U U'y=X'y
  if(!include_last_beta){
    z=gsl_vector_alloc(ncovariates);
    Z=gsl_matrix_alloc(ncovariates,ncovariates);
    for(i=0;i<ncovariates;++i){
      gsl_vector_set(z,i,gsl_vector_get(c,i));
      for(j=0;j<ncovariates;++j){
	gsl_matrix_set(Z,i,j,gsl_matrix_get(C,i,j));
      }
    }
    gsl_matrix_free(C);
    gsl_vector_free(c);
    C=Z;
    c=gsl_vector_alloc(ncovariates);
  }

  gsl_set_error_handler_off ();//Check return value of cholesky manually
  if(gsl_linalg_cholesky_decomp (C)==GSL_EDOM){
    gsl_set_error_handler (NULL);
    gsl_vector_free(c);
    if(include_last_beta) gsl_matrix_free(C);
    return 0;}//X^t X not positive semi-definite
  gsl_set_error_handler (NULL);//Return the default error handler

  if(include_last_beta)
    gsl_linalg_cholesky_solve (C, c, beta);//beta=inv(C)*c=inv(X'X)*(X'y) ,ML solution when h==0.0
  else{
    gsl_linalg_cholesky_solve (C, z, c);//beta=inv(C)*c=inv(X'X)*(X'y) ,ML solution when h==0.0
    for(i=0;i<ncovariates;++i)
      gsl_vector_set(beta,i,gsl_vector_get(c,i));
    gsl_vector_set(beta,i,0.0);  
    gsl_vector_free(z);
  }
  //gsl_linalg_cholesky_invert (C); //for std err of betas

  gsl_blas_dcopy (Uy,Uy_UXbeta2);
  gsl_blas_dgemv (CblasNoTrans, -1.0, UX, beta, 1.0, Uy_UXbeta2);//Uy_UXbeta2=U'y-U'X beta

  sqsum=square(gsl_blas_dnrm2 (Uy_UXbeta2));  
  for(i=0;i<nind;++i) 
    gsl_vector_set(Uy_UXbeta2,i,square(gsl_vector_get(Uy_UXbeta2,i)));
  *var0=(sqsum/nind); //ML estimate for var0 is (sqsum/nind) when h == 0.0 
  *h=0.0; //important for further calculations is that var0 == (sqsum/nind) !

  gsl_vector_free(c);
  gsl_matrix_free(C);
  return 1;

}


void decompose_S(gsl_matrix **S,int *groupsizes,int *relatedness_in_groups,gsl_matrix **U,gsl_vector **d,PAR *par){
  //if relatedness_in_groups[g]==0 then 
  //U[g] and d[g] will be left NULL

  int i,g,j,row,zeroed;
  double *Sdiag;
  gsl_eigen_symmv_workspace * eigen_space;
  FILE *U_out,*D_out;

  if(par->save_U_D){
    U_out=fopen(par->U_file,"w");
    D_out=fopen(par->D_file,"w");
  }

  printf("Decomposing full covariance matrix using GSL:\n ...\n");    
  row=0;
  for(g=0;g<ngroups;++g){ 
    if(par->save_U_D){	
      fprintf(D_out,"GROUP%d %d %d\n",g+1,groupsizes[g],relatedness_in_groups[g]);
      fprintf(U_out,"GROUP%d %d %d\n",g+1,groupsizes[g],relatedness_in_groups[g]);
    }
    if(relatedness_in_groups[g]){
      if((Sdiag=(double *) malloc(groupsizes[g]*sizeof(double)))==NULL)
	alloc_error("decompose_S");
      for(i=0;i<groupsizes[g];++i) Sdiag[i]=gsl_matrix_get(S[g],i,i);
      d[g]=gsl_vector_alloc(groupsizes[g]);
      U[g]=gsl_matrix_alloc(groupsizes[g],groupsizes[g]);
      eigen_space=gsl_eigen_symmv_alloc (groupsizes[g]);
      gsl_eigen_symmv (S[g], d[g], U[g], eigen_space); // diagonal and lower part of S will be changed!
      gsl_eigen_symmv_sort (d[g], U[g], GSL_EIGEN_SORT_VAL_DESC); //descending ordering
      for(i=0;i<groupsizes[g];++i){ //recover original S[g]
	gsl_matrix_set(S[g],i,i,Sdiag[i]);
	for(j=0;j<i;++j)
	  gsl_matrix_set(S[g],i,j,gsl_matrix_get(S[g],j,i));
      }
      free(Sdiag);
      if(par->save_U_D){
	for(i=0;i<groupsizes[g];++i){
	  fprintf(D_out,"%g\n",gsl_vector_get(d[g],i));
	  for(j=0;j<groupsizes[g];++j)
	    fprintf(U_out,"%g ",gsl_matrix_get(U[g],i,j));
	  fprintf(U_out,"\n");
	}
      }

      gsl_eigen_symmv_free(eigen_space);
    }
    else{
      d[g]=NULL;
      U[g]=NULL;
    }
  }
  printf("done.\n");
  if(par->save_U_D){
    printf("Wrote U to '%s' and D to '%s'.\n",par->U_file,par->D_file);
    fclose(U_out);
    fclose(D_out);
  }

  zeroed=0;
  for(g=0;g<ngroups;++g){
    if(relatedness_in_groups[g]){
      for(i=0;i<groupsizes[g];++i){ 
	if(gsl_vector_get(d[g],i)<par->min_d){
	  ++zeroed;
	  gsl_vector_set(d[g],i,par->min_d);
	}
      }
    }
  }
  if(zeroed>0)
    fprintf(stderr,"\n\nWARNING: %d eigenvalues were set to minimum value of %g.\n\n",zeroed,par->min_d);
}

#if USE_LAPACK == 1
void decompose_S_lapack(double **S,int *groupsizes,int *relatedness_in_groups,gsl_matrix **U,gsl_vector **d,PAR *par){
  //if relatedness_in_groups[g]==0 then 
  //U[g] and d[g] will be left NULL

  int i,g,j,row,zeroed;
  double *evecs,*evals,*Sdiag;
  FILE *U_out,*D_out;

  if(par->save_U_D){
    U_out=fopen(par->U_file,"w");
    D_out=fopen(par->D_file,"w");
  }

  printf("Decomposing full covariance matrix using LAPACK ...\n ");    
  row=0;
  for(g=0;g<ngroups;++g){ 
    if(par->save_U_D){	
      fprintf(D_out,"GROUP%d %d %d\n",g+1,groupsizes[g],relatedness_in_groups[g]);
      fprintf(U_out,"GROUP%d %d %d\n",g+1,groupsizes[g],relatedness_in_groups[g]);
    }
    if(groupsizes[g]>0 && relatedness_in_groups[g]){
      if((evecs = (double *) malloc(groupsizes[g]*groupsizes[g]*sizeof(double)))==NULL)
	alloc_error("evecs");
      if((evals = (double *) malloc(groupsizes[g]*sizeof(double)))==NULL)
	alloc_error("evals");
      if((Sdiag=(double *) malloc(groupsizes[g]*sizeof(double)))==NULL)
	alloc_error("Sdiag");
      for(i=0;i<groupsizes[g];++i) Sdiag[i]=S[g][i*groupsizes[g]+i];
   
      eigen(S[g],groupsizes[g],groupsizes[g],0,evals,evecs);

      d[g]=gsl_vector_alloc(groupsizes[g]);
      U[g]=gsl_matrix_alloc(groupsizes[g],groupsizes[g]);
      for(i=0;i<groupsizes[g];++i){
	gsl_vector_set(d[g],groupsizes[g]-i-1,evals[i]);
	for(j=0;j<groupsizes[g];++j)
	  gsl_matrix_set(U[g],j,groupsizes[g]-i-1,evecs[i*groupsizes[g]+j]);
      }
      for(i=0;i<groupsizes[g];++i){//recover full S
	for(j=0;j<i;++j)
	  S[g][i+j*groupsizes[g]]=S[g][j+i*groupsizes[g]];
	S[g][i*groupsizes[g]+i]=Sdiag[i];
      }
      free(evals);free(evecs);free(Sdiag);

      if(par->save_U_D){
	for(i=0;i<groupsizes[g];++i){
	  fprintf(D_out,"%g\n",gsl_vector_get(d[g],i));
	  for(j=0;j<groupsizes[g];++j)
	    fprintf(U_out,"%g ",gsl_matrix_get(U[g],i,j));
	  fprintf(U_out,"\n");
	}
      }
    }
    else{
      d[g]=NULL;
      U[g]=NULL;
    }
  }
  printf("done.\n");
  if(par->save_U_D){
    printf("Wrote U to '%s' and D to '%s'.\n",par->U_file,par->D_file);
    fclose(U_out);
    fclose(D_out);
  }

  zeroed=0;
  for(g=0;g<ngroups;++g){
    if(relatedness_in_groups[g]){
      for(i=0;i<groupsizes[g];++i){ 
	if(gsl_vector_get(d[g],i)<par->min_d){
	  ++zeroed;
	  gsl_vector_set(d[g],i,par->min_d);
	}
      }
    }
  }
  if(zeroed>0)
    fprintf(stderr,"\nWARNING: %d eigenvalues were set to minimum value of %g.\n",zeroed,par->min_d);

}
#endif



void compute_UX_and_Uy_and_d(gsl_matrix **X_full,gsl_vector **y_full,gsl_matrix **U_full,gsl_vector **d_full,gsl_matrix **UX_full,gsl_vector **Uy_full,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,int *groupsizes,int *relatedness_in_groups){

  int i,j,k;
  int ind=0;

  for(i=0;i<ngroups;++i){
    if(groupsizes[i]){
      UX_full[i]=gsl_matrix_calloc(groupsizes[i],ncovariates+1);
      Uy_full[i]=gsl_vector_calloc(groupsizes[i]);
      if(relatedness_in_groups[i]){
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, U_full[i], X_full[i], 0.0, UX_full[i]);//X_temp=U[i]^T X[i]
	gsl_blas_dgemv (CblasTrans, 1.0, U_full[i], y_full[i], 0.0, Uy_full[i]); //y_temp=U[i]^T y[i]
      }
      else{
	gsl_blas_dcopy (y_full[i], Uy_full[i]);//U=I
	gsl_matrix_memcpy (UX_full[i], X_full[i]);//U=I
	
      }
    }

    for(j=0;j<groupsizes[i];++j){
      gsl_vector_set(Uy,ind,gsl_vector_get(Uy_full[i],j));
      for(k=0;k<ncovariates+1;++k)
	gsl_matrix_set(UX,ind,k,gsl_matrix_get(UX_full[i],j,k));
      if(relatedness_in_groups[i])
	gsl_vector_set(d,ind,gsl_vector_get(d_full[i],j));
      else
	gsl_vector_set(d,ind,1.0);
      ++ind;
    }
  }
}

int read_new_y(FILE *in,gsl_vector *Uy,gsl_matrix **U_full,int *groupsizes){

  int i;
  gsl_vector *y;
  double t;

  if(ngroups!=1){
    fprintf(stderr,"ERROR in read_new_y: ngroups !=1.\n");
    exit(1);
  }

  y=gsl_vector_alloc(groupsizes[0]);
  for(i=0;i<groupsizes[0];++i){
    if(fscanf(in,"%lf",&t)!=1){
      fprintf(stderr,"ERROR reading phenotypes!\n");
      return 0;
    }
    gsl_vector_set(y,i,t);
  }
  gsl_blas_dgemv (CblasTrans, 1.0, U_full[0], y, 0.0, Uy);
  gsl_vector_free(y);
  return 1;
}


double square(double x){
  return x*x;
}


void update_var0(double *var0,double h,gsl_vector *Uy_UXbeta2,gsl_vector *d,int nind){

 int i;
 double sum=0.0,v;

 
 for(i=0;i<nind;++i){
   v=square(gsl_vector_get(Uy_UXbeta2,i));
   gsl_vector_set(Uy_UXbeta2,i,v);
   sum+=v/(1-h+h*gsl_vector_get(d,i));
 }
 *var0=sum/nind;
}


int update_beta(int include_last_beta,gsl_vector *Uy,gsl_matrix *UX,gsl_vector *d,gsl_vector *beta,double var0,double h,int nind){

  int i,j,k,ncov;
  double v;
  gsl_vector *b,*z;
  gsl_matrix *A,*B;

  ncov=ncovariates+1;
  if(!include_last_beta)
    ncov=ncovariates;

  b=gsl_vector_calloc(ncov);
  B=gsl_matrix_calloc(ncov,ncov);
  A=gsl_matrix_calloc(ncov,nind); 
  for(i=0;i<ncov;++i)
    for(j=0;j<nind;++j){
      v=gsl_matrix_get(UX,j,i)/var0/(1-h+h*gsl_vector_get(d,j));
      gsl_matrix_set(A,i,j,v);
      for(k=0;k<ncov;++k)
	gsl_matrix_set(B,i,k,gsl_matrix_get(B,i,k)+v*gsl_matrix_get(UX,j,k));
    }

  //B= UX^t 1/diag(d) UX

  gsl_blas_dgemv (CblasNoTrans, 1.0, A, Uy, 0.0, b); //b=0.0*b+1.0*A*Uy=UX^t 1/diag(d) Uy

  gsl_set_error_handler_off ();//Check return value of cholesky manually
  if(gsl_linalg_cholesky_decomp (B)==GSL_EDOM){//B not positive semi-definite
    gsl_matrix_free(B);  
    gsl_matrix_free(A);
    gsl_vector_free(b);
    gsl_set_error_handler (NULL);
    return 0;
  }
  gsl_set_error_handler (NULL);//Return the default error handler


  if(include_last_beta)
    gsl_linalg_cholesky_solve (B,b,beta); //beta=B^(-1)*b
  else{
    z=gsl_vector_alloc(ncov);
    gsl_linalg_cholesky_solve (B,b,z);
    for(i=0;i<ncov;++i)
      gsl_vector_set(beta,i,gsl_vector_get(z,i));
    gsl_vector_set(beta,ncov,0.0);
    gsl_vector_free(z);
  }
  gsl_matrix_free(A);
  gsl_vector_free(b);
  gsl_matrix_free(B);
  return 1;
}
    



double loglkhood(gsl_vector *Uy,gsl_matrix *UX,gsl_vector *beta,double var0,double h,gsl_vector *d,int nind){
  /* Returns the log of the likelihood (over the groups)
     ignoring the constant (2*pi)^(-nind/2)
  */

  int i;
  double logdet,quadr,x;
  gsl_vector *z;

  z=gsl_vector_calloc(nind);   
  logdet=0.0;
  quadr=0.0;
  if(ncovariates>0){
    gsl_blas_dgemv(CblasNoTrans,-1.0,UX,beta,0.0,z); //z=-1.0*UX*beta
    gsl_blas_daxpy(1.0,Uy,z); //z=1.0*Uy+z = Uy-UX*beta
  }
  else{
    gsl_blas_dcopy(Uy,z);
  }

  for(i=0;i<nind;++i){
    x=var0*(1+h*gsl_vector_get(d,i)-h);
    quadr+=gsl_vector_get(z,i)*gsl_vector_get(z,i)/x;
    logdet+=log(x);
  }
  gsl_vector_free(z);

  return (-0.5*(quadr+logdet));

}


void read_S(FILE *in,gsl_matrix **S,int *groupsizes,int *relatedness_in_groups,int *good_indexes,int ngood,int read_list){

  int nchars,i,g,j,ind1,ind2,ind_good,ind_good2,ngood_in_group,*good_in_group,lower_good_ind=0,total_nind=0;
  char tag[TAG_SIZE],target[TAG_SIZE],*line;
  size_t line_size=100;
  double t,nsnps;
  FILE *line_str;

  g=0; 
  if((line=(char*) malloc(line_size*sizeof(char)))==NULL)
    alloc_error("line in read_S");
  
  if((good_in_group=(int *) malloc(ngood*sizeof(int)))==NULL)
    alloc_error("good_in_group in read_S");
 
  while(!feof(in) && g<ngroups){
  nchars=getline(&line,&line_size,in);
    if(feof(in)) break; 
    line_str=fmemopen(line,nchars*sizeof(char),"r");
    if(nchars<1){
      fprintf(stderr,"ERROR reading R file. Title line of GROUP%d is empty.\n",g+1);exit(1);}

    if(fscanf(line_str,TAG_S,tag)!=1){
      fprintf(stderr,"ERROR reading R file. Title line of GROUP%d.\n",g+1);
      exit(2);
    }

    sprintf(target,"GROUP%d",g+1);
    if(strcmp(tag,target)!=0){
      fprintf(stderr,"ERROR with R file: Expected:%s Got:%s\n",target,tag);
      exit(2);
    }

    if(fscanf(line_str,"%d %d",&i,relatedness_in_groups+g)!=2)
      fprintf(stderr,"ERROR with R file: Title line of %s.\n",tag);
    if(groupsizes[g]!=i){
      fprintf(stderr,"ERROR with R file: size of %s is %d (R-file) != %d (A-file)\n",tag,i,groupsizes[g]);
      exit(2);
    }
    if(relatedness_in_groups[g]!=0 && relatedness_in_groups[g]!=1){
      fprintf(stderr,"ERROR with R file: relatedness status of %s is %d \n",tag,relatedness_in_groups[g]);
      exit(2);
    }
    fclose(line_str);

    ind_good=lower_good_ind;
    ngood_in_group=0;
    good_in_group[0]=-1;//will remain if no good in group
    while(ind_good<ngood && good_indexes[ind_good]<total_nind+groupsizes[g]){
	good_in_group[ngood_in_group++]=good_indexes[ind_good]-total_nind;
	ind_good++;
    }
    total_nind+=groupsizes[g];
    lower_good_ind=ind_good;

    if(relatedness_in_groups[g]==1 && groupsizes[g]>0){
      if(ngood_in_group>0)
	S[g]=gsl_matrix_alloc(ngood_in_group,ngood_in_group);
      else
	relatedness_in_groups[g]=0;

      if(!read_list){//reads in matrix format
	ind_good=0;
	for(i=0;i<groupsizes[g];++i){
	  nchars=getline(&line,&line_size,in);
	  if(good_in_group[ind_good]==i){
	    line_str=fmemopen(line,nchars*sizeof(char),"r");
	    ind_good2=0;
	    for(j=0;j<groupsizes[g];++j){
	      if(fscanf(line_str,"%lf",&t)!=1 || !isfinite(t)){
		fprintf(stderr,"ERROR with covariance file: %s pair %d,%d\n",tag,i+1,j+1);
		exit(2);
	      }
	      if(good_in_group[ind_good2]==j){
		gsl_matrix_set(S[g],ind_good,ind_good2,t);
		if(ind_good2<ind_good && fabs(gsl_matrix_get(S[g],ind_good2,ind_good)-t)>1e-3){
		  fprintf(stderr,"ERROR with R-file: %s pair (%d , %d) is not symmetric\n",tag,i+1,j+1);
		  fprintf(stderr,"values are: %g != %g (tolerance 1e-3)\n",
			  t,gsl_matrix_get(S[g],ind_good2,ind_good));
		  exit(2);
		}
		if(ind_good==ind_good2 && fabs(1.0-t)>0.1){
		  //fprintf(stderr,"WARNING: with covariance file:%s diagonal element %d = %g is not 1.0\n",tag,i+1,t);
		  //exit(2);
		}
		++ind_good2;
	      }
	    }
	    //CHECK positive semi-definiteness!!!
	    fclose(line_str);
	    ind_good++;
	  }
	}
      }else{//reads in list format
	ind_good=0;
	for(i=0;i<groupsizes[g];++i){
	  ind_good2=0;
	  for(j=0;j<=i;++j){
	    nchars=getline(&line,&line_size,in);
	    line_str=fmemopen(line,nchars*sizeof(char),"r");
	    if(fscanf(line_str,"%d %d %lf %lf",&ind1,&ind2,&nsnps,&t)!=4 || !isfinite(t)){
	      fprintf(stderr,"ERROR with R list file: %s pair %d,%d\n",tag,i+1,j+1);
	      exit(2);
	    }
	    fclose(line_str);
	    if(ind1!=i+1 || ind2!=j+1){
	      fprintf(stderr,"ERROR with R list file: %s expected pair %d,%d got pair %d,%d\n",tag,i+1,j+1,ind1,ind2);
	      exit(2);
	    }
	    if(good_in_group[ind_good]==i && good_in_group[ind_good2]==j){
	      gsl_matrix_set(S[g],ind_good,ind_good2,t);
	      gsl_matrix_set(S[g],ind_good2,ind_good,t);
	      if(ind_good==ind_good2 && fabs(1.0-t)>0.1){
		//fprintf(stderr,"WARNING: with covariance file:%s diagonal element %d = %g is not 1.0\n",tag,i+1,t);
		//exit(2);
	      }
	      ++ind_good2;
	    }
	  }
	  if(good_in_group[ind_good]==i) ++ind_good;
	}
      }
    }else{//else covariance matrix is I
      S[g]=NULL;
    }
    groupsizes[g]=ngood_in_group;
    
#if ECHO_INPUT == 1
      printf("GROUP %d:\n",g+1);
      if(relatedness_in_groups[g]==1){
	for(i=0;i<groupsizes[g];++i){
	  for(j=0;j<groupsizes[g];++j){
	    printf("%lf ",gsl_matrix_get(S[g],i,j));
	  }
	  printf("\n");
	}
	printf("\n");
      }
      else
	printf("No relatedness\n");
#endif
      ++g;
  }

  if(g<ngroups){
     fprintf(stderr,"ERROR with R file: too few GROUPS (found %d expected %d) \n",g,ngroups);
    exit(2);
  }
  free(good_in_group);
  free(line);

}

void read_S_lapack(FILE *in,double **S,int *groupsizes,int *relatedness_in_groups,int *good_indexes,int ngood,int read_list){

  int nchars,i,g,j,ind1,ind2,ind_good,ind_good2,ngood_in_group,*good_in_group,lower_good_ind=0,total_nind=0;
  char tag[TAG_SIZE],target[TAG_SIZE],*line;
  size_t line_size=100;
  double t,nsnps;
  FILE *line_str;

  g=0;  
  if((line=(char*) malloc(line_size*sizeof(char)))==NULL)
    alloc_error("line in read_S_lapack");

  if((good_in_group=(int *) malloc(ngood*sizeof(int)))==NULL)
    alloc_error("good_in_group in read_S_lapack");

  while(!feof(in) && g<ngroups){
    nchars=getline(&line,&line_size,in);
    if(feof(in)) break; 
    line_str=fmemopen(line,nchars*sizeof(char),"r");
    if(nchars<1){
      fprintf(stderr,"ERROR reading R file. Title line of GROUP%d is empty.\n",g+1);exit(1);}

    if(fscanf(line_str,TAG_S,tag)!=1){
      fprintf(stderr,"ERROR reading R file. Title line of GROUP%d.\n",g+1);
      exit(2);
    }
      
    sprintf(target,"GROUP%d",g+1);
    if(strcmp(tag,target)!=0){
      fprintf(stderr,"ERROR with R file: Expected:%s Got:%s\n",target,tag);
      exit(2);
    }

    if(fscanf(line_str,"%d %d",&i,relatedness_in_groups+g)!=2)
      fprintf(stderr,"ERROR with R file: Title line of %s.\n",tag);
    if(groupsizes[g]!=i){
      fprintf(stderr,"ERROR with R file: size of %s is %d (R-file) != %d (A-file)\n",tag,i,groupsizes[g]);
      exit(2);
    }
    if(relatedness_in_groups[g]!=0 && relatedness_in_groups[g]!=1){
      fprintf(stderr,"ERROR with R file: relatedness status of %s is %d \n",tag,relatedness_in_groups[g]);
      exit(2);
    }
    fclose(line_str);

    ind_good=lower_good_ind;
    ngood_in_group=0;
    good_in_group[0]=-1;//will remain if no good in group
    while(ind_good<ngood && good_indexes[ind_good]<total_nind+groupsizes[g]){
	good_in_group[ngood_in_group++]=good_indexes[ind_good]-total_nind;
	ind_good++;
    }
    total_nind+=groupsizes[g];
    lower_good_ind=ind_good;

    if(relatedness_in_groups[g]==1 && groupsizes[g]>0){
      
      if(ngood_in_group>0){
	if((S[g] = (double *) malloc(ngood_in_group*ngood_in_group*sizeof(double)))==NULL)
	  alloc_error("S");
      }else
	relatedness_in_groups[g]=0;
      //*S[g]=1;//what is this???

      if(!read_list){//reads in matrix format
	ind_good=0;
	for(i=0;i<groupsizes[g];++i){
	  nchars=getline(&line,&line_size,in);
	  if(good_in_group[ind_good]==i){
	    line_str=fmemopen(line,nchars*sizeof(char),"r");
	    ind_good2=0;
	    for(j=0;j<groupsizes[g];++j){
	      if(fscanf(line_str,"%lf",&t)!=1 || !isfinite(t)){
		fprintf(stderr,"ERROR with covariance file: %s pair %d,%d\n",tag,i+1,j+1);
		exit(2);
	      }
	      if(good_in_group[ind_good2]==j){
		*((S[g])+ind_good+ngood_in_group*ind_good2)=t;
		if(ind_good2<ind_good && fabs(S[g][ind_good+ngood_in_group*ind_good2]-S[g][ind_good2+ngood_in_group*ind_good])>1e-3){
		  fprintf(stderr,"ERROR with R-file: %s pair (%d , %d) is not symmetric\n",tag,i+1,j+1);
		  fprintf(stderr,"values are: %g != %g (tolerance 1e-3)\n",
			  S[g][ind_good+ngood_in_group*ind_good2],S[g][ind_good2+ngood_in_group*ind_good]);
		  exit(2);
		}
		if(ind_good==ind_good2 && fabs(1.0-S[g][ind_good+ngood_in_group*ind_good2])>0.1){
		  //fprintf(stderr,"WARNING: with covariance file:%s diagonal element %d = %g is not 1.0\n",tag,i+1,t);
		  //exit(2);
		}
		++ind_good2;
	      }
	    }
	    //CHECK positive semi-definiteness!!!
	    fclose(line_str);
	    ind_good++;
	  }
	}
      }else{//reads in list format
	ind_good=0;
	for(i=0;i<groupsizes[g];++i){
	  ind_good2=0;
	  for(j=0;j<=i;++j){
	    nchars=getline(&line,&line_size,in);
	    line_str=fmemopen(line,nchars*sizeof(char),"r");
	    if(fscanf(line_str,"%d %d %lf %lf",&ind1,&ind2,&nsnps,&t)!=4 || !isfinite(t)){
	      fprintf(stderr,"ERROR with R list file: %s pair %d,%d\n",tag,i+1,j+1);
	      exit(2);
	    }
	    fclose(line_str);
	    if(ind1!=i+1 || ind2!=j+1){
	      fprintf(stderr,"ERROR with R list file: %s expected pair %d,%d got pair %d,%d\n",tag,i+1,j+1,ind1,ind2);
	      exit(2);
	    }
	    if(good_in_group[ind_good]==i && good_in_group[ind_good2]==j){
	      *((S[g])+ind_good+ngood_in_group*ind_good2)=t;
	      *((S[g])+ind_good2+ngood_in_group*ind_good)=t;
	      ++ind_good2;
	    }
	  }
	  if(good_in_group[ind_good]==i) ++ind_good;
	}
      }
    }else{//else covariance matrix is I
      S[g]=NULL;
    }
    groupsizes[g]=ngood_in_group;

#if ECHO_INPUT == 1
      printf("GROUP %d:\n",g+1);
      if(relatedness_in_groups[g]==1){
	for(i=0;i<groupsizes[g];++i){
	  for(j=0;j<groupsizes[g];++j){
	    printf("%lf ",S[g][i+groupsizes[g]*j]);
	  }
	  printf("\n");
	}
	printf("\n");
      }
      else
	printf("No relatedness\n");
#endif
      ++g;
  }

  if(g<ngroups){
    fprintf(stderr,"ERROR with R file: too few GROUPS (found %d expected %d) \n",g,ngroups);
    exit(2);
  }
  free(good_in_group);
  free(line);

}


#if USE_LAPACK == 1
void eigen(double *A, int dim, int nvecs, int values_only, double *evals, double *evecs) {
  /* 
     MP: This is taken from Dan Davison's Shellfish sources. Only small changes to replace 
     Dan's own alloc and error functions.

     Compute eigendecomposition of square matrix A.
     A has 'dim' rows and columns and is assumed to be stored in column major order, 
     with the increment in memory to jump between columns also equal to dim (if A were a submatrix of
     a larger matrix, that wouldn't be true and LDA would need to be passed in and set correctly).
       
     Sufficient space for the evecs and evals must be allocated outside this function,
     but all other work arrays are alloc()d and free()d here 
     (including the potentially informative ISUPPZ)
       
     The argument nvecs specifies the number of evecs/evals you want to compute.
     if( nvecs >= dim || nvecs <= 0) then all evecs and evals are computed. 
     Otherwise the first nvecs evecs/evals are computed (considered in order of decreasing eval).

     Note that in the output the evals are in *increasing* order, which might not be what you expect.
     The evecs are in a corresponding order.
    */
  
  extern void dsyevr_(char *JOBZp, char *RANGEp, char *UPLOp, int *Np,
			double *A, int *LDAp, double *VLp, double *VUp,
			int *ILp, int *IUp, double *ABSTOLp, int *Mp,
			double *W, double *Z, int *LDZp, int *ISUPPZ,
			double *WORK, int *LWORKp, int *IWORK, int *LIWORKp,
			int *INFOp);
    
  char
    JOBZ = values_only ? 'N' : 'V',    // 'N' doesn't compute evecs (faster)
    RANGE,                             // see below
    UPLO = 'L' ;                       // use upper / lower triangle of A ?
  int 
    LDA = dim,                         /* increment between columns (NB if e.g. A is some sort of submatrix
					  of a larger matrix, this might not be the same as dim */
    IL, IU,                            // see below
    M,                                 // will contain total number of evals found
    LDZ = dim,

    *ISUPPZ,                           // indices (2 for each eval) indicating support of evectors
    // (?meaning) 

    LWORK,                             // size of WORK
    *IWORK,                            // work array of ints
    LIWORK,                            // size of IWORK
    INFO,                              // receives error code from fortran routine
    tmp_int ;                          // used to get size of work array of ints
  double
    VL, VU,                            // see below
    ABSTOL=dlamch('S'),                                      
    *WORK,                             // work array of doubles
    tmp_double ;                       // used to get size of work array of ints
  /*
    RANGE
    'A' computes all evals/evecs and ignores IL, IU, VL, VU
    'I' computes ILth through IUth evals/evecs 
    (considering evals ordered in increasing order of eval size; numbering from 1)
    'V' computes evals/evecs whose evals lie within (VL, VU]
  */
  ISUPPZ=NULL;
  if (nvecs > 0 && nvecs < dim) {
    RANGE = 'I' ;
    IL = dim - nvecs + 1;
    IU = dim ;
  }
  else  {
    RANGE = 'A' ;
    nvecs = dim ;
    IL = IU = -1.0 ;
  }
  VL = VU = -1.0 ; //we don't use option 'V'
  
   
  /* First request optimal sizes of work arrays */

  LWORK = LIWORK = -1 ;
    
  dsyevr_(&JOBZ, &RANGE, &UPLO, &dim, A, &LDA, &VL, &VU,
	  &IL, &IU, &ABSTOL, &M, evals, evecs,  &LDZ, ISUPPZ,
	  &tmp_double, &LWORK, &tmp_int, &LIWORK, &INFO);

  if(INFO != 0) {
    fprintf(stderr,"eigen: dsyevr returned with error code %d when computing optimal size of work arrays.\n",INFO) ;
    exit(1);
  }
  
  LWORK = (int) tmp_double ;
  LIWORK = tmp_int ;
  //printf("LWORK = %d\nLIWORK = %d\n", LWORK, LIWORK);
 
  
  if((WORK = (double *) malloc(LWORK*sizeof(double)))==NULL)
    alloc_error("WORK");
  if((IWORK = (int *) malloc(LIWORK*sizeof(int)))==NULL)
    alloc_error("IWORK");
  if((ISUPPZ = (int *) malloc(2*nvecs*sizeof(int)))==NULL)
    alloc_error("ISUPPZ");

    /* Now do real thing */
    
    dsyevr_(&JOBZ, &RANGE, &UPLO, &dim, A, &LDA, &VL, &VU,
	    &IL, &IU, &ABSTOL, &M, evals, evecs,  &LDZ, ISUPPZ,
	    WORK, &LWORK, IWORK, &LIWORK, &INFO);

    if(INFO != 0) {
      fprintf(stderr,"eigen: dsyevr returned with error code %d when computing optimal size of work arrays.\n",INFO);
      exit(1);
    }
    free(WORK);
    free(IWORK);
    free(ISUPPZ);
}
#endif

void read_U(FILE *U_in,gsl_matrix **U,int *groupsizes,int *relatedness_in_groups){

  int i,g,j,r;
  double t;
  char tag[TAG_SIZE],target[TAG_SIZE];

  g=0;  
  while(!feof(U_in) && g<ngroups){

    if(fscanf(U_in,TAG_S,tag)!=1){
      if(feof(U_in)) break; 
      fprintf(stderr,"ERROR with eigenvector file\n");
      exit(2);
    }
      
    sprintf(target,"GROUP%d",g+1);
    if(strcmp(tag,target)!=0){
      fprintf(stderr,"ERROR with eigenvector file: Expected:%s Got:%s\n",target,tag);
      exit(2);
    }

    if(fscanf(U_in,"%d %d",&i,&r)!=2)
      fprintf(stderr,"ERROR with eigenvector file: %s\n",tag);
    if(groupsizes[g]!=i){
      fprintf(stderr,"ERROR with eigenvector file: size of %s is %d != %d\n",tag,i,groupsizes[g]);
      exit(2);
    }
    if(r!=0 && r!=1){
      fprintf(stderr,"ERROR with eigenvector file: relatedness status of %s is %d\n",tag,r);
      exit(2);
    }
    relatedness_in_groups[g]=r;

    if(relatedness_in_groups[g] && groupsizes[g]){
      U[g]=gsl_matrix_alloc(groupsizes[g],groupsizes[g]);
      for(i=0;i<groupsizes[g];++i){
	for(j=0;j<groupsizes[g];++j){
	  if(fscanf(U_in,"%lf",&t)==1)
	    gsl_matrix_set(U[g],i,j,t);
	  else{
	    fprintf(stderr,"ERROR with eigenvector file:%s pair %d,%d\n",tag,i+1,j+1);
	    exit(2);
	  }
	}
      }
    }
    //else eigenvector matrix is I

#if ECHO_INPUT == 1
      printf("GROUP %d:\n",g+1);
      if(relatedness_in_groups[g]==1){
	for(i=0;i<groupsizes[g];++i){
	  for(j=0;j<groupsizes[g];++j){
	    printf("%lf ",gsl_matrix_get(U[g],i,j));
	  }
	  printf("\n");
	}
	printf("\n");
      }
      else
	printf("No relatedness\n");
#endif
      ++g;
  }

  if(g<ngroups){
    fprintf(stderr,"ERROR with eigenvector file: too few GROUPS (found %d expected %d) \n",g,ngroups);
    exit(2);
  }
}

void read_D(FILE *D_in,gsl_vector **d,int *groupsizes,int *relatedness_in_groups,double min_d){

  int i,g,r,zeroed;
  double t;
  char tag[TAG_SIZE],target[TAG_SIZE];

  zeroed=0;
  g=0;  
  while(!feof(D_in) && g<ngroups){

    if(fscanf(D_in,TAG_S,tag)!=1){
      if(feof(D_in)) break; 
      fprintf(stderr,"ERROR with eigenvalue file\n");
      exit(2);
    }
      
    sprintf(target,"GROUP%d",g+1);
    if(strcmp(tag,target)!=0){
      fprintf(stderr,"ERROR with eigenvalue file: Expected:%s Got:%s\n",target,tag);
      exit(2);
    }

    if(fscanf(D_in,"%d %d",&i,&r)!=2)
      fprintf(stderr,"ERROR with eigenvalue file: %s\n",tag);
    if(groupsizes[g]!=i){
      fprintf(stderr,"ERROR with eigenvalue file: size of %s is %d != %d\n",tag,i,groupsizes[g]);
      exit(2);
    }
    if(relatedness_in_groups[g]!=r){
      fprintf(stderr,"ERROR with eigenvalue file: relatedness status of %s is %d != %d\n",tag,r,relatedness_in_groups[g]);
      exit(2);
    }

    if(relatedness_in_groups[g] && groupsizes[g]){
      d[g]=gsl_vector_alloc(groupsizes[g]);
      for(i=0;i<groupsizes[g];++i){
	if(fscanf(D_in,"%lf",&t)==1){
	  if(t>=min_d)
	    gsl_vector_set(d[g],i,t);
	  else{
	    gsl_vector_set(d[g],i,min_d);
	    zeroed++;
	  }
	}
	else{
	  fprintf(stderr,"ERROR with eigenvalue file:%s ind %d\n",tag,i+1);
	  exit(2);
	}
      }
    }

#if ECHO_INPUT == 1
      printf("GROUP %d:\n",g+1);
      if(relatedness_in_groups[g]==1){
	for(i=0;i<groupsizes[g];++i){
	  printf("%lf\n",gsl_vector_get(d[g],i));
	}
	else{
	  printf("No relatedness\n");
	}
      }
#endif
      ++g;
  }

  if(g<ngroups){
    fprintf(stderr,"ERROR with eigenvalues file: too few GROUPS (found %d expected %d) \n",g,ngroups);
    exit(2);
  }

  if(zeroed>0)
    fprintf(stderr,"\nWARNING: %d eigenvalues were set to minimum value of %g.\n\n",zeroed,min_d);

}


int *read_attributes(FILE *in,gsl_vector **y,gsl_matrix **X,PAR *par,int *groupsizes,char **exclusionlist,int nexclusionlist){
  //returns the list of indexes that are NOT in exclusion list
  //prints out all the individuals and their exclusion status to file par->inds_out_file
  //checks that the file has as many groups as specified in the parameters file
  //if mean_center==1, then centers each covariate (except population mean)

  int i,j,g,outcome_col,*covar_col,nchars,found_cov,found_id,found_outcome,ncol,maxgroupsize,
    id_col=-1,total_ind,ngood_ind,ngood_ind_in_group,ind,*good_indexes,found;
  double **X_array,*y_array,*sums,*sums2,y_sum,y_sum2;
  char *tag=NULL,target[TAG_SIZE],*line;
  size_t line_size=100;
  FILE *line_str,*ind_out;

  ind_out=fopen(par->inds_out_file,"w");

  if((covar_col=(int*) malloc(par->n_covariates*sizeof(int)))==NULL)
    alloc_error("covar_col in read_attributes");
  for(i=0;i<par->n_covariates;++i) covar_col[i]=-1;

  if((line=(char*) malloc(line_size*sizeof(char)))==NULL)
    alloc_error("line in read_attributes");
  nchars=getline(&line,&line_size,in);
  line_str=fmemopen(line,nchars*sizeof(char),"r");
  if(nchars<1){
    fprintf(stderr,"ERROR reading attributes file. First line is empty.\n");
    exit(1);
  }

  ncol=0;
  found_id=0;
  found_outcome=0;
  //printf("Found columns for\n");
  while(fscanf(line_str,"%as",&tag)==1){
    found_cov=0;
    for(i=0;i<par->n_covariates;++i){
      if(strcmp(tag,par->covariates[i])==0){
	if(found_cov){fprintf(stderr,"ERROR several columns in Attributes file match %s.\n",tag);exit(1);}
	found_cov=1;
	covar_col[i]=ncol;
	if(strcmp(tag,par->outcome)==0){fprintf(stderr,"ERROR column %s in Attributes file is both outcome and a covariate.\n",tag);exit(1);}
	//printf("cov: '%s' at column %d\n",tag,ncol+1);
      }
    }
    if(par->id != NULL && strcmp(tag,par->id)==0){
      if(found_id){fprintf(stderr,"ERROR several columns in Attributes file match %s.\n",tag);exit(1);}
      found_id=1;
      id_col=ncol;
      printf("id: '%s' at column %d\n",tag,ncol+1);
    }
    if(strcmp(tag,par->outcome)==0){
      if(found_outcome){fprintf(stderr,"ERROR several columns in Attributes file match %s.\n",tag);exit(1);}
      found_outcome=1;
      outcome_col=ncol;
      //printf("outcome: '%s' at column %d\n",tag,ncol+1);
    }
    ++ncol;
    free(tag);
  }
  fclose(line_str);

  for(i=0;i<par->n_covariates;++i) 
    if(covar_col[i]==-1)
      {fprintf(stderr,"ERROR did not find covariate column %s from Attributes file.\n",par->covariates[i]);exit(1);}
  if(!found_outcome)
    {fprintf(stderr,"ERROR did not find outcome column %s from Attributes file.\n",par->outcome);exit(1);}
  if(par->read_E && !found_id)
    {fprintf(stderr,"ERROR did not find id column %s from Attributes file.\n",par->id);exit(1);}
 
  if((sums=(double*) calloc(par->n_covariates,sizeof(double)))==NULL)
    alloc_error("sums in read_attributes");
  if((sums2=(double*) calloc(par->n_covariates,sizeof(double)))==NULL)
    alloc_error("sums2 in read_attributes");
  y_sum=0.0;y_sum2=0.0;

  X_array=NULL;y_array=NULL;good_indexes=NULL;
  maxgroupsize=0;
  g=0;
  total_ind=0;
  ngood_ind=0;
  //printf("\nReads individuals:\n");
  fprintf(ind_out,"id group excluded\n");
  while(!feof(in) && g<ngroups){

    if(fscanf(in,"%as",&tag)!=1)
      {if(feof(in)) break; fprintf(stderr,"ERROR with Attributes file: no tag for GROUP%d\n",g+1);exit(2);}
      
    sprintf(target,"GROUP%d",g+1);
    if(strcmp(tag,target)!=0){
      fprintf(stderr,"ERROR with Attributes file: Expected:%s Got:%s\n",target,tag);
      exit(2);
    }
    free(tag);

    if(fscanf(in,"%d\n",groupsizes+g)!=1){
      fprintf(stderr,"ERROR with Attributes file: The size of %s\n",tag);
      exit(2);
    }
    if(groupsizes[g]<=0){
      fprintf(stderr,"ERROR with Attributes file: The size of %s is %d \n",tag,groupsizes[g]);
      exit(2);
    }

    if(groupsizes[g]>maxgroupsize){
      if((X_array=(double **) realloc(X_array,groupsizes[g]*sizeof(double*)))==NULL)
	alloc_error("X_array");
      for(i=maxgroupsize;i<groupsizes[g];++i)
	if((X_array[i]=(double *) malloc((1+par->n_covariates)*sizeof(double)))==NULL)
	  alloc_error("X_array");
      if((y_array=(double *) realloc(y_array,groupsizes[g]*sizeof(double)))==NULL)
	alloc_error("y_array");
      maxgroupsize=groupsizes[g];
    }
    if((good_indexes=(int *) realloc(good_indexes,(ngood_ind+groupsizes[g])*sizeof(int)))==NULL)
	alloc_error("good_indexes");

    ngood_ind_in_group=0;

    for(ind=0;ind<groupsizes[g];++ind){
      nchars=getline(&line,&line_size,in);
      
      if(id_col>=0){
	line_str=fmemopen(line,nchars*sizeof(char),"r");
	for(i=0;i<=id_col;++i)
	  if(fscanf(line_str,"%as",&tag)!=1){
	    fprintf(stderr,"ERROR with Attributes file: Line for individual %d of group %d does not have id value.\n",ind+1,g+1);
	    exit(2);
	  }else
	    if(i<id_col) free(tag);
	fclose(line_str);
      }
      else{//no id given in attributes file
	if((tag=(char*) malloc(TAG_SIZE*sizeof(char)))==NULL) alloc_error("tag in read_attributes");
	sprintf(tag,"Ind%d",total_ind+1);
      }

      if(!(par->read_E) || !nexclusionlist || !string_search(tag,exclusionlist,nexclusionlist)){
	//is not excluded
	fprintf(ind_out,"%s %d 0\n",tag,g+1);free(tag);
	++ngood_ind_in_group;
	X_array[ind][0]=1;
	line_str=fmemopen(line,nchars*sizeof(char),"r");
	good_indexes[(ngood_ind)++]=total_ind;
	for(i=0;i<ncol;++i){
	  found=0;
	  for(j=0;j<par->n_covariates;++j){
	    if(i==covar_col[j]){
	      if(fscanf(line_str,"%lf",X_array[ind]+j+1)!=1){
		fprintf(stderr,"ERROR with Attributes file: Individual %d of group %d does not have valid covariate value for %s.\n",
			ind+1,g+1,par->covariates[j]);
		exit(2);
	      }
	      sums[j]+=X_array[ind][j+1];
	      sums2[j]+=X_array[ind][j+1]*X_array[ind][j+1];
	      found=1;
	    }
	  }
	  if(i==outcome_col){
	    if(fscanf(line_str,"%lf",y_array+ind)!=1){
	      fprintf(stderr,"ERROR with Attributes file: Individual %d of group %d does not have valid value for %s.\n",
		      ind+1,g+1,par->outcome);
	      exit(2);
	    }
	    if(par->logOR>0 && *(y_array+ind)!=0 && *(y_array+ind)!=1){
	      fprintf(stderr,"ERROR with Attributes file: Binary outcome specified (by logOR>0) but individual %d of group %d has value %g (!=0,1) for outcome %s.\n",
		      ind+1,g+1,*(y_array+ind),par->outcome);
	      exit(2);
	    }
	    y_sum+=y_array[ind];
	    y_sum2+=y_array[ind]*y_array[ind];
	    found=1;
	  }
	  if(!found){
	    if(fscanf(line_str,"%as",&tag)!=1){
	      fprintf(stderr,"ERROR with Attributes file: Line for individual %d of group %d does not have value for column %d.\n",ind+1,g+1,i+1);
	      exit(2);
	    }
	    free(tag);
	  }
	}
	fclose(line_str);
      }else{
	//is excluded
	fprintf(ind_out,"%s %d 1\n",tag,g+1);
	X_array[ind][0]=0;
      }
      total_ind++;
      
    }
    fprintf(stdout,"Group%d: keeps %d, excludes %d individuals.\n",g+1,ngood_ind_in_group,groupsizes[g]-ngood_ind_in_group);
    
    if(ngood_ind_in_group){
      y[g]=gsl_vector_alloc(ngood_ind_in_group);
      X[g]=gsl_matrix_alloc(ngood_ind_in_group,2+par->n_covariates);
      ind=0;
      for(i=0;i<groupsizes[g];++i){
	if(X_array[i][0]){
	  gsl_vector_set(y[g],ind,y_array[i]);
	  for(j=0;j<1+par->n_covariates;++j)
	    gsl_matrix_set(X[g],ind,j,X_array[i][j]);//includes population mean
	  gsl_matrix_set(X[g],ind++,j,0.0);//will be snp
	}
      }
    }
    if(!par->read_R)
      groupsizes[g]=ngood_ind_in_group;
    //otherwise don't set this yet but first check that R file is consistent
	    
#if ECHO_INPUT == 1
      printf("GROUP %d:\n",g+1);
      for(i=0;i<groupsizes[g];++i)
	printf("%lf ",gsl_vector_get(y[g],i));
      printf("\n");
#endif
      ++g;
  }
  fclose(ind_out);
  printf("Wrote individuals to file %s.\n",par->inds_out_file);
  if(g<ngroups){
    fprintf(stderr,"ERROR with Attributes file: too few GROUPS (found %d expected %d) \n",g,ngroups);
    exit(2);
  }

  //Check that there are no more individuals or groups in the attributes file
  while(!feof(in)){
    nchars=getline(&line,&line_size,in);
    if(nchars>0){
      line_str=fmemopen(line,nchars*sizeof(char),"r");
      if(fscanf(line_str,"%as",&tag)==1){
	fprintf(stderr,"\nERROR with Attributes file: Found entry %s after %d groups were read in.\n",tag,g);
	fprintf(stderr," Check the number of groups (Parameters file) and the number of individuals in each group (Attributes file).\n");
	exit(2);
      }
      fclose(line_str);
    }
  }

  y_sum=y_sum/ngood_ind;y_sum2=y_sum2/ngood_ind;

  printf("outcome '%s' at column %d. mean=%g sd=%g\n",par->outcome,outcome_col+1,y_sum,sqrt(y_sum2-y_sum*y_sum));
  for(j=0;j<par->n_covariates;++j){
    sums[j]/=ngood_ind;
    sums2[j]/=ngood_ind;
    printf("covariate '%s' at column %d. mean=%g, sd=%g\n",par->covariates[j],covar_col[j]+1,sums[j],sqrt(sums2[j]-sums[j]*sums[j]));
  }
  
  if(par->mean_center && par->n_covariates){
    printf("Mean centering the covariates.\n");
    for(g=0;g<ngroups;++g)
      for(i=0;i<X[g]->size1;++i)
	for(j=0;j<par->n_covariates;++j)
	  gsl_matrix_set(X[g],i,1+j, gsl_matrix_get(X[g],i,1+j)-sums[j]);
  }

  free(sums);
  free(sums2);
  free(covar_col);
  free(y_array);
  for(i=0;i<maxgroupsize;++i)
    free(X_array[i]);
  free(X_array);
  free(line);

  par->n_good_ind=ngood_ind;
  par->n_total_ind=total_ind;

  return good_indexes;
}

int string_search(char *tag,char **list,int nlist){
  //binary search from ordered list of strings
  int i,left,right,cmp;

  if(nlist<0) return 0;
  left=0;right=nlist-1;
  if(strcmp(tag,list[left])==0 || strcmp(tag,list[right])==0)
    return 1;
  while(right-left>1){
    i=left+(right-left)/2;
    cmp=strcmp(tag,list[i]);
    if(cmp==0) return 1; //found
    if(cmp<0)
      right=i;
    else 
      left=i;
  }
  return 0; //not found
}

void print_available_parameters(){

  printf("\nParameter file:\n");
  printf("Generally format is a pair of 'tag' and 'value' per each line of parameter file, separated by white space.\n");
  printf("Exception is tag 'covariates'.\n\n");
  printf("Tags (strings) and their associated values:\n");
  printf("TAG\t\t\tVALUE\n");
  printf("ngroups\t\t\tnumber of groups (int > 0)\n");
  printf("covariates\t\tnumber of covariates (int >=0), followed by\n");
  printf("\t\t\tnames of each covariate as specified in ATTRIBUTES file (strings < 1000 chars)\n");
  printf("outcome\t\t\tname of outcome variables as specified in ATTRIBUTES file\n");
  printf("id\t\t\tnames of individual identifier as specified in ATTRIBUTES file\n");
  printf("max_iterations\t\tmax number of iterations in ML estimation (conditional maximisation iterations) (int>0)\n");
  printf("max_eta_iterations\tmax number of iterations for finding a maximum from a single eta interval (int>0)\n");
  printf("max_eta_intervals\tnumber of intervals to which [0,1] is divided to perform the maximisation w.r.t. eta (int >0)\n");
  printf("max_eta_divisions\tmax number of division of a single eta interval to find a maximum (int>0)\n");
  printf("n_integr_intervals\tnumber of intervals in which [0,1] is divided to perform the marginal likelihood calculation (int>0)\n");
  printf("mean_center\t\tshould z be mean centered? (0 or 1)\n");
  printf("check_geno\t\tshould value of z be checked to be in [0,2] or to match the missing value? (0 or 1)\n");
  printf("save_U_D\t\tshould U and D be written to file? (0 or 1)\n");
  printf("bayesian\t\tare bayesian marginal likelihood computations carried out? (0 or 1)\n");
  printf("logOR\t\t\twill the effects of z be transformed to log-odds scale? (0 or 1 or 2)\n");
  printf("tol\t\t\ttolerance for ML calculation (double >0 )\n");
  printf("fixed_eta\t\tvalue to which eta will be fixed [0,1]\n");
  printf("impute_missing\t\tshould missing values in z_file or in gen_file be set to population mean of non-missing values? (0 or 1)\n");
  printf("missing_z\t\tvalue of z that corresponds to missing entry (double)\n");
  printf("missing_gen\t\tif non-NULL class probabilities sum to < missing_gen then genotype is labelled missing [0,1]\n");
  printf("min_d\t\t\tif some eigenvalue < min_d then it is set to min_d (double >= 0)\n");
  printf("\nz_file, gen_file, A_file, R_file, E_file, Rmatrix_file, U_file, D_file, prior_file, results_file, inds_out_file are \n\t\t\ttags for input/output file names\n\n");

  printf("All file names can also be read from the command line by using switches\n");
  printf("-E_file, -z_file, -gen_file, -A_file, -R_file, -Rmatrix_file,\n-U_file, -D_file, -prior_file, -results_file, -inds_out_file\n");
  printf("followed by paths to the files.\n\n");

  printf("If any of these files are specified on the command line, then the command line must also\n");
  printf("specify a parameters file by using switch -P_file\n");
  printf("and the parameter file and the command line must not specify same files.\n\n");
    
  printf("At most one file can be read from stdin, by using - as the file name.\n");


}

void print_available_switches(){
  printf("\n\nAll file names (but no other parameters) can be read from the command line by using switches:\n");
  printf("-E_file, -z_file, -gen_file, -A_file, -R_file, -Rmatrix_file,\n-U_file, -D_file, -prior_file, -results_file, -inds_out_file\n");
  printf("followed by paths to the files.\n\n");
  printf("If any of these files are specified on the command line, then the command line must also\n");
  printf("specify a parameters file by using switch -P_file\n");
  printf("and the parameter file and the command line must not specify same files.\n\n");
    
  printf("At most one file can be read from stdin, by using - as the file name.\n");
}




void read_parameters(PAR *par,int argc, char *argv[])
{
  /*
    Reads in the parameter file and command line parameters.

    Generally format is pair of 'tag' and  'value' per each line of parameter file, separated by white space.
    Exception is for tag 'covariates'

    Exits if some tag is present several times.

    Tags (strings) and their associated values:
    TAG                VALUE
    ngroups            number of groups (int > 0)
    covariates         number of covariates (int >=0), followed by
                       names of each covariate as specified in ATTRIBUTES file (strings < 1000 chars)
    outcome	       name of outcome variables as specified in ATTRIBUTES file
    id                 names of individual identifier as specified in ATTRIBUTES file
    max_iterations     max number of iterations in ML estimation (conditional maximisation iterations) (int>0)
    max_eta_iterations max number of iterations for finding a maximum from a single eta interval (int>0)
    max_eta_intervals  number of intervals to which [0,1] is divided to perform the maximisation w.r.t. eta (int >0)
    max_eta_divisions  max number of division of a single eta interval to find a maximum (int>0)
    n_integr_intervals number of intervals to which [0,1] is divided into to perform the marginal likelihood calculation (int>0)
    mean_center        should z be mean centered? (0 or 1)
    check_geno         should value of z be checked to be in [0,2] or then to match the missing value? (0 or 1)
    save_U_D           should U and D be written to file? (0 or 1)
    bayesian           are bayesian marginal likelihood computations carried out? (0 or 1)
    logOR              will the effects of z be transformed to log-odds scale? 
                        (0 [no], 1 [first order approx] or 2 [better approx when z is an allele count at a SNP]) 
    tol                tolerance for ML calculation (double >0 )
    fixed_eta          value to which eta will be fixed [0,1]
    impute_missing     should missing values of z be set to population mean of non-missing z? (0 or 1)
    missing_z          value of z that corresponds to missing entry (double)
    missing_gen        if non-NULL class probabilities sum to < missing_gen then genotype is labelled missing [0,1]
    min_d              if some eigenvalue < min_d then it is set to min_d (double >= 0)

    E_file, z_file, gen_file, A_file, R_file, Rmatrix_file, U_file, D_file, prior_file, results_file, inds_out_file
    are tags for input/output file names

    All file names can also be read from the command line by using switches
    -E_file, -z_file, -gen_file, -A_file, -R_file, -Rmatrix_file, 
    -U_file, -D_file, -prior_file, -results_file, -inds_out_file
    followed by paths to the files.
    If any of the files are specified on the command line, then the command line must also
    specify parameters file by using switch -P_file and
    the parameter file and the command line must not specify same files.
    
    At most one file can be read from stdin, by using '-' as the file name.

   */
  int i,nchars,tag_ok,P_file_index,argi;
  char 
    tag[TAG_SIZE],target[TAG_SIZE],*line,
    n_groups_found=0,covariates_found=0,max_iterations_found=0,
    max_eta_iterations_found=0,max_eta_intervals_found=0,max_eta_divisions_found=0,
    n_integr_intervals_found=0,fixed_eta_found=0,tol_found=0,logOR_found=0,bayesian_found=0,
    mean_center_found=0,check_geno_found=0,A_file_found=0,E_file_found=0,
    save_U_D_found=0,z_file_found=0,gen_file_found=0,Rmatrix_file_found=0,R_file_found=0,U_file_found=0,D_file_found=0,
    missing_z_found=0,missing_gen_found=0,impute_missing_found=0,prior_file_found=0,outcome_found=0,
    id_found=0,min_d_found=0,results_file_found=0,P_file_found=0,inds_out_file_found=0,read_stdin=0,found;
  FILE *line_str,*in;
  size_t str_size,line_size=100;

  par->A_file=NULL;par->E_file=NULL;par->z_file=NULL;par->id=NULL;
  par->outcome=NULL;par->R_file=NULL;par->U_file=NULL;par->D_file=NULL;
  par->prior_file=NULL;par->results_file=NULL;par->inds_out_file=NULL;

  P_file_index=1;//default value if only one parameter given on the command line
  if(argc!=2){
    for(argi=1;argi<argc-1;++argi){
      found=0;
    
      sprintf(target,"-P_file");
      if(strcmp(argv[argi],target)==0){
	if(P_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-P_file' was found several times\n");exit(2);}
	P_file_index=++argi;
	P_file_found=1;found=1;
      }
      
      sprintf(target,"-gen_file");
      if(strcmp(argv[argi],target)==0){
	if(gen_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-gen_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->z_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->z_file");
	strcpy(par->z_file,argv[argi]);//NOTE gen_file is stored in variable par.z_file
	gen_file_found=1;found=1;par->read_gen=1;
      }

      sprintf(target,"-z_file");
      if(strcmp(argv[argi],target)==0){
	if(z_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-z_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->z_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->z_file");
	strcpy(par->z_file,argv[argi]);
	z_file_found=1;found=1;par->read_z=1;
      }

      sprintf(target,"-results_file");
      if(strcmp(argv[argi],target)==0){
	if(results_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-results_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->results_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->results_file");
	strcpy(par->results_file,argv[argi]);
	results_file_found=1;found=1;
      }

      sprintf(target,"-E_file");
      if(strcmp(argv[argi],target)==0){
	if(E_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-E_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->E_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->E_file");
	strcpy(par->E_file,argv[argi]);
	E_file_found=1;found=1;
      }

      sprintf(target,"-inds_out_file");
      if(strcmp(argv[argi],target)==0){
	if(inds_out_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-inds_out_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->inds_out_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->inds_out_file");
	strcpy(par->inds_out_file,argv[argi]);
	inds_out_file_found=1;found=1;
      }

      sprintf(target,"-A_file");
      if(strcmp(argv[argi],target)==0){
	if(A_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-A_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->A_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->A_file");
	strcpy(par->A_file,argv[argi]);
	A_file_found=1;found=1;
      }

      sprintf(target,"-R_file");
      if(strcmp(argv[argi],target)==0){
	if(R_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-R_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->R_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->R_file");
	strcpy(par->R_file,argv[argi]);
	R_file_found=1;found=1;
      }

      sprintf(target,"-Rmatrix_file");
      if(strcmp(argv[argi],target)==0){
	if(Rmatrix_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-Rmatrix_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->R_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->R_file");
	strcpy(par->R_file,argv[argi]);
	Rmatrix_file_found=1;found=1;
      }

      sprintf(target,"-U_file");
      if(strcmp(argv[argi],target)==0){
	if(U_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-U_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->U_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->U_file");
	strcpy(par->U_file,argv[argi]);
	U_file_found=1;found=1;
      }

      sprintf(target,"-D_file");
      if(strcmp(argv[argi],target)==0){
	if(D_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-D_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->D_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->D_file");
	strcpy(par->D_file,argv[argi]);
	D_file_found=1;found=1;
      }

      sprintf(target,"-prior_file");
      if(strcmp(argv[argi],target)==0){
	if(prior_file_found)
	  {fprintf(stderr,"ERROR with command line parameters: '-prior_file' was found several times\n");exit(2);}
	str_size=10+strlen(argv[++argi]);
	if((par->prior_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->prior_file");
	strcpy(par->prior_file,argv[argi]);
	prior_file_found=1;found=1;
      }

      if(!found)
	{fprintf(stderr,"ERROR with command line: invalid switch '%s'\n",argv[argi]);print_available_switches();exit(2);}

      //at this point index argi has already been read in

    }

    if(!P_file_found){//either no parameters (argc==1) or no -P_file switch found  
      fprintf(stderr,"\n\nDid not find a name for the parameters file.\nUse either:\n");
      fprintf(stderr,"./MMM 'parameters_file'\nor, if specifying other file names on the command line:\n");
      fprintf(stderr,"./MMM -P_file 'parameters_file' (and possibly other pairs of '-switch filename')\n");
      print_available_parameters();

      exit(1);
    }
  }

  //open parameters file for reading
  if(argv[P_file_index][0]=='-' && argv[P_file_index][1]=='\0'){
    in=stdin;
    printf("\nReading parameters file from stdin.\n");
    read_stdin=1;
  }else{
    if((in=fopen(argv[P_file_index],"r"))==NULL){
      fprintf(stderr,"Cannot open parameters file %s\n",argv[P_file_index]);
      exit(1);
    }  
    printf("\nReading parameters from file '%s'\n",argv[P_file_index]);
  }


  //read parameter file
  if((line=(char*) malloc(line_size*sizeof(char)))==NULL)
    alloc_error("line in read_parameters");
  nchars=0;

  while(!feof(in))
    {
      found=0;
      tag_ok=0;
      while(!tag_ok && !feof(in)){
	if((nchars=getline(&line,&line_size,in))>0){
	  line_str=fmemopen(line,nchars*sizeof(char),"r");
	  if(fscanf(line_str,TAG_S,tag)==1) //TAG_S is currently macro for "%99s"
	    tag_ok=1;
	  else
	    fclose(line_str);
	}
      }
      if(feof(in) && !tag_ok) break;
     
      sprintf(target,"n_groups");
      if(strcmp(tag,target)==0)
	{
	  if(n_groups_found)
	    {fprintf(stderr,"ERROR with parameters file: n_groups was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->n_groups)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for n_groups is invalid\n");exit(2);};
	  if(par->n_groups<1)
	    {fprintf(stderr,"ERROR with parameters file:value for n_groups is %d < 1\nExits!\n",par->n_groups);exit(2);}
	  n_groups_found=1;found=1;
	}

      sprintf(target,"covariates");
      if(strcmp(tag,target)==0)
	{
	  if(covariates_found)
	    {fprintf(stderr,"ERROR with parameters file: covariates was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->n_covariates)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for number of covariates is invalid\n");exit(2);};
	  if(par->n_covariates<0)
	    {fprintf(stderr,"ERROR with parameters file:value for number of covariates is %d < 0\nExits!\n",par->n_covariates);exit(2);}
	  if(par->n_covariates){
	    if((par->covariates=(char **) malloc(par->n_covariates * sizeof(char*)))==NULL)
	      alloc_error("covariates in read_parameters");
	    for(i=0;i<(par->n_covariates);++i){
	      if(fscanf(line_str,"%as",&(par->covariates[i]))!=1)
		{fprintf(stderr,"ERROR with parameters file:name for covariate %d is invalid\n",i+1);exit(2);}
	    }
	  }
	  covariates_found=1;found=1;
	}

      sprintf(target,"outcome");
      if(strcmp(tag,target)==0)
	{
	  if(outcome_found)
	    {fprintf(stderr,"ERROR with parameters file: outcome was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->outcome))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for outcome is invalid\n");exit(2);};
	  outcome_found=1;found=1;
	}

      sprintf(target,"id");
      if(strcmp(tag,target)==0)
	{
	  if(id_found)
	    {fprintf(stderr,"ERROR with parameters file: id was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->id))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for id is invalid\n");exit(2);};
	  id_found=1;found=1;
	}


      sprintf(target,"max_iterations");
      if(strcmp(tag,target)==0)
	{
	  if(max_iterations_found)
	    {fprintf(stderr,"ERROR with parameters file: max_iterations was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->max_iterations)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for max_iterations is invalid\n");exit(2);};
	  if(par->max_iterations<1)
	    {fprintf(stderr,"ERROR with parameters file:value for max_iterations is %d < 1\nExits!\n",par->max_iterations);exit(2);}
	  max_iterations_found=1;found=1;
	}

      sprintf(target,"max_eta_iterations");
      if(strcmp(tag,target)==0)
	{
	  if(max_eta_iterations_found)
	    {fprintf(stderr,"ERROR with parameters file: max_eta_iterations was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->max_eta_iterations)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for max_eta_iterations is invalid\n");exit(2);};
	  if(par->max_eta_iterations<1)
	    {fprintf(stderr,"ERROR with parameters file:value for max_eta_iterations is %d < 1\nExits!\n",par->max_eta_iterations);exit(2);}
	  max_eta_iterations_found=1;found=1;
	}

      sprintf(target,"max_eta_intervals");
      if(strcmp(tag,target)==0)
	{
	  if(max_eta_intervals_found)
	    {fprintf(stderr,"ERROR with parameters file: max_eta_intervals was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->max_eta_intervals)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for max_eta_intervals is invalid\n");exit(2);};
	  if(par->max_eta_intervals<1)
	    {fprintf(stderr,"ERROR with parameters file:value for max_eta_intervals is %d < 1\nExits!\n",par->max_eta_intervals);exit(2);}
	  max_eta_intervals_found=1;found=1;
	}

      sprintf(target,"max_eta_divisions");
      if(strcmp(tag,target)==0)
	{
	  if(max_eta_divisions_found)
	    {fprintf(stderr,"ERROR with parameters file: max_eta_divisions was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->max_eta_divisions)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for max_eta_divisions is invalid\n");exit(2);};
	  if(par->max_eta_divisions<1)
	    {fprintf(stderr,"ERROR with parameters file:value for max_eta_divisions is %d < 1\nExits!\n",par->max_eta_divisions);exit(2);}
	  max_eta_divisions_found=1;found=1;
	}     

      sprintf(target,"n_integr_intervals");
      if(strcmp(tag,target)==0)
	{
	  if(n_integr_intervals_found)
	    {fprintf(stderr,"ERROR with parameters file: n_integr_intervals was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->n_integr_intervals)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for n_integr_intervals is invalid\n");exit(2);};
	  if(par->n_integr_intervals<1)
	    {fprintf(stderr,"ERROR with parameters file:value for n_integr_intervals is %d < 1\nExits!\n",par->n_integr_intervals);exit(2);}
	  n_integr_intervals_found=1;found=1;
	}     

      sprintf(target,"mean_center");
      if(strcmp(tag,target)==0)
	{
	  if(mean_center_found)
	    {fprintf(stderr,"ERROR with parameters file: mean_center was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->mean_center)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for mean_center is invalid\n");exit(2);};
	  if(par->mean_center!=0 && par->mean_center!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for mean_center is %d !=0,1\nExits!\n",par->mean_center);exit(2);}
	  mean_center_found=1;found=1;
	}     

      sprintf(target,"check_geno");
      if(strcmp(tag,target)==0)
	{
	  if(check_geno_found)
	    {fprintf(stderr,"ERROR with parameters file: check_geno was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->check_geno)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for check_geno is invalid\n");exit(2);};
	  if(par->check_geno!=0 && par->check_geno!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for check_geno is %d !=0,1\nExits!\n",par->check_geno);exit(2);}
	  check_geno_found=1;found=1;
	}   

      sprintf(target,"save_U_D");
      if(strcmp(tag,target)==0)
	{
	  if(save_U_D_found)
	    {fprintf(stderr,"ERROR with parameters file: save_U_D was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->save_U_D)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for save_U_D is invalid\n");exit(2);};
	  if(par->save_U_D!=0 && par->save_U_D!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for save_U_D is %d !=0,1\nExits!\n",par->save_U_D);exit(2);}
	  save_U_D_found=1;found=1;
	}   
  
      sprintf(target,"bayesian");
      if(strcmp(tag,target)==0)
	{
	  if(bayesian_found)
	    {fprintf(stderr,"ERROR with parameters file: bayesian was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->bayesian)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for bayesian is invalid\n");exit(2);};
	  if(par->bayesian!=0 && par->bayesian!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for bayesian is %d !=0,1\nExits!\n",par->bayesian);exit(2);}
	  bayesian_found=1;found=1;
	}     

      sprintf(target,"logOR");
      if(strcmp(tag,target)==0)
	{
	  if(logOR_found)
	    {fprintf(stderr,"ERROR with parameters file: logOR was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->logOR)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for logOR is invalid\n");exit(2);};
	  if(par->logOR!=0 && par->logOR!=1 && par->logOR!=2)
	    {fprintf(stderr,"ERROR with parameters file:value for logOR is %d !=0,1,2\nExits!\n",par->logOR);exit(2);}
	  logOR_found=1;found=1;
	}     

      sprintf(target,"tol");
      if(strcmp(tag,target)==0)
	{
	  if(tol_found)
	    {fprintf(stderr,"ERROR with parameters file: tol was found several times\n");exit(2);}
	  if(fscanf(line_str," %lf ",&par->tol)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for tol is invalid\n");exit(2);};
	  if(par->tol<0.0)
	    {fprintf(stderr,"ERROR with parameters file:value for tol is %g < 0.0\nExits!\n",par->tol);exit(2);}
	  tol_found=1;found=1;
	}

      sprintf(target,"min_d");
      if(strcmp(tag,target)==0)
	{
	  if(min_d_found)
	    {fprintf(stderr,"ERROR with parameters file: min_d was found several times\n");exit(2);}
	  if(fscanf(line_str," %lf ",&par->min_d)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for min_d is invalid\n");exit(2);};
	  if(par->min_d<0.0)
	    {fprintf(stderr,"ERROR with parameters file:value for min_d is %g < 0.0\nExits!\n",par->min_d);exit(2);}
	  min_d_found=1;found=1;
	}

      sprintf(target,"fixed_eta");
      if(strcmp(tag,target)==0)
	{
	  if(fixed_eta_found)
	    {fprintf(stderr,"ERROR with parameters file: fixed_eta was found several times\n");exit(2);}
	  if(fscanf(line_str," %lf ",&par->fixed_eta)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for fixed_eta is invalid\n");exit(2);};
	  if(par->fixed_eta<0.0 || par->fixed_eta>1.0)
	    {fprintf(stderr,"ERROR with parameters file:value for fixed_eta is %g and not in [0,1]\nExits!\n",par->fixed_eta);exit(2);}
	  fixed_eta_found=1;found=1;par->fix_eta=1;
	}

      sprintf(target,"impute_missing");
      if(strcmp(tag,target)==0)
	{
	  if(impute_missing_found)
	    {fprintf(stderr,"ERROR with parameters file: impute_missing was found several times\n");exit(2);}
	  if(fscanf(line_str," %d ",&par->impute_missing)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for impute_missing is invalid\n");exit(2);};
	  if(par->impute_missing != 0 && par->impute_missing != 1)
	    {fprintf(stderr,"ERROR with parameters file:value for impute_missing is not 0 or 1.\n");exit(2);}
	  impute_missing_found=1;found=1;
	}
      
      sprintf(target,"missing_z");
      if(strcmp(tag,target)==0)
	{
	  if(missing_z_found)
	    {fprintf(stderr,"ERROR with parameters file: missing_z was found several times\n");exit(2);}
	  if(fscanf(line_str," %lf ",&par->missing_z)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for missing_z is invalid\n");exit(2);}
	  missing_z_found=1;found=1;
	}

      sprintf(target,"missing_gen");
      if(strcmp(tag,target)==0)
	{
	  if(missing_gen_found)
	    {fprintf(stderr,"ERROR with parameters file: missing_gen was found several times\n");exit(2);}
	  if(fscanf(line_str," %lf ",&par->missing_gen)!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for missing_gen is invalid\n");exit(2);}
	  if(par->missing_gen<0.0 || par->missing_gen>1.0)
	    {fprintf(stderr,"ERROR with parameters file:value for missing_gen is not within [0...1]\n");exit(2);}
	  missing_gen_found=1;found=1;
	}

      sprintf(target,"E_file");
      if(strcmp(tag,target)==0)
	{
	  if(E_file_found)
	    {fprintf(stderr,"ERROR with parameters file: E_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->E_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for E_file is invalid\n");exit(2);};
	  E_file_found=1;found=1;
	}

      sprintf(target,"inds_out_file");
      if(strcmp(tag,target)==0)
	{
	  if(inds_out_file_found)
	    {fprintf(stderr,"ERROR with parameters file: inds_out_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->inds_out_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for inds_out_file is invalid\n");exit(2);};
	  inds_out_file_found=1;found=1;
	}

      sprintf(target,"results_file");
      if(strcmp(tag,target)==0)
	{
	  if(results_file_found)
	    {fprintf(stderr,"ERROR with parameters file: results_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->results_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for results_file is invalid\n");exit(2);}
	  results_file_found=1;found=1;
	}

      sprintf(target,"z_file");
      if(strcmp(tag,target)==0)
	{
	  if(z_file_found)
	    {fprintf(stderr,"ERROR with parameters file: z_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->z_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for z_file is invalid\n");exit(2);}
	  z_file_found=1;found=1;par->read_z=1;
	}

      sprintf(target,"gen_file");
      if(strcmp(tag,target)==0)
	{
	  if(gen_file_found)
	    {fprintf(stderr,"ERROR with parameters file: gen_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->z_file))!=1) //NOTE gen_file stored in par.z_file
	    {fprintf(stderr,"ERROR with parameters file:value for gen_file is invalid\n");exit(2);}
	  gen_file_found=1;found=1;par->read_gen=1;
	}

      sprintf(target,"A_file");
      if(strcmp(tag,target)==0)
	{
	  if(A_file_found)
	    {fprintf(stderr,"ERROR with parameters file: A_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->A_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for A_file is invalid\n");exit(2);};
	  A_file_found=1;found=1;
	}

      sprintf(target,"Rmatrix_file");
      if(strcmp(tag,target)==0)
	{
	  if(Rmatrix_file_found)
	    {fprintf(stderr,"ERROR with parameters file: Rmatrix_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->R_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for Rmatrix_file is invalid\n");exit(2);};
	  Rmatrix_file_found=1;found=1;
	}

      sprintf(target,"R_file");
      if(strcmp(tag,target)==0)
	{
	  if(R_file_found)
	    {fprintf(stderr,"ERROR with parameters file: R_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->R_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for R_file is invalid\n");exit(2);};
	  R_file_found=1;found=1;
	}

      sprintf(target,"U_file");
      if(strcmp(tag,target)==0)
	{
	  if(U_file_found)
	    {fprintf(stderr,"ERROR with parameters file: U_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->U_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for U_file is invalid\n");exit(2);};
	  U_file_found=1;found=1;
	}

      sprintf(target,"D_file");
      if(strcmp(tag,target)==0)
	{
	  if(D_file_found)
	    {fprintf(stderr,"ERROR with parameters file: D_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->D_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for D_file is invalid\n");exit(2);};
	  D_file_found=1;found=1;
	}

      sprintf(target,"prior_file");
      if(strcmp(tag,target)==0)
	{
	  if(prior_file_found)
	    {fprintf(stderr,"ERROR with parameters file: prior_file was found several times\n");exit(2);}
	  if(fscanf(line_str," %as ",&(par->prior_file))!=1)
	    {fprintf(stderr,"ERROR with parameters file:value for prior_file is invalid\n");exit(2);};
	  prior_file_found=1;found=1;
	}

      if(found==0)
	{fprintf(stderr,"\nERROR with parameters file: unexpected label '%s' found\n\n",tag);exit(2);}
      fclose(line_str);
    }

  if(!max_iterations_found){par->max_iterations=100;}
  if(!tol_found){par->tol=1e-9;}
  if(!min_d_found){par->min_d=0.0;}
  if(!max_eta_iterations_found){par->max_eta_iterations=100;}
  if(!max_eta_intervals_found){par->max_eta_intervals=10;}
  if(!max_eta_divisions_found){par->max_eta_divisions=10;}
  if(!n_integr_intervals_found){par->n_integr_intervals=100;}
  if(!fixed_eta_found){par->fix_eta=0;}
  if(!mean_center_found){par->mean_center=0;}
  if(!logOR_found){par->logOR=0;}
  if(!check_geno_found){par->check_geno=0;}
  if(!save_U_D_found){par->save_U_D=1;}
  if(!impute_missing_found){par->impute_missing=0;}
  if(gen_file_found && !missing_gen_found){par->missing_gen=0.1;}
  if(!id_found){par->id=NULL;}

  if(!A_file_found){fprintf(stderr,"ERROR with parameters : Did not find label 'A_file' / switch '-A_file'.\n");exit(2);} 
  if(!outcome_found){fprintf(stderr,"ERROR with parameters : Did not find label 'outcome'.\n");exit(2);} 
  if(!n_groups_found){fprintf(stderr,"ERROR with parameters : Did not find label 'n_groups'.\n");exit(2);}
  if(E_file_found && !id_found){
    fprintf(stderr,"ERROR with parameters file: Did not find label 'id' even though 'E_file' is specified.\n");exit(2);}
  if(!results_file_found){
    if(!z_file_found && !gen_file_found){
      str_size=strlen(par->outcome)+10;
      if((par->results_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->results_file");
      sprintf(par->results_file,"%s_res.out",par->outcome);
    }else{
      str_size=strlen(par->z_file)+10;
      if((par->results_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->results_file");
      sprintf(par->results_file,"%s_res.out",par->z_file);}
  }
  if(!covariates_found){ par->n_covariates=0;}
  if(!z_file_found){par->read_z=0;}
  if(!gen_file_found){par->read_gen=0;}
  if(!bayesian_found){par->bayesian=0;}
  if(z_file_found && gen_file_found){fprintf(stderr,"ERROR with parameters : Both gen_file and z_file specified.\n");exit(2);}
  par->model_missing=0;
  if((z_file_found && missing_z_found) || (gen_file_found)) par->model_missing=1;
  if(par->bayesian && !prior_file_found){
    fprintf(stderr,"ERROR with parameters : Did not find label 'prior_file' even though 'bayesian' is used.\n");exit(2);}
  if(par->bayesian && par->model_missing && !par->impute_missing){
    fprintf(stderr,"ERROR with parameters : Must impute 'missing' data when 'bayesian' is used.\n");exit(2);}
  if(!U_file_found && !Rmatrix_file_found && !R_file_found){
    fprintf(stderr,"ERROR with parameters : Did find neither 'R_file','Rmatrix_file' nor 'U_file'.\n");exit(2);}
  if(U_file_found && Rmatrix_file_found){
    fprintf(stderr,"ERROR with parameters : Both 'Rmatrix_file' and 'U_file' specified.\n");exit(2);}
  if(U_file_found && R_file_found){
    fprintf(stderr,"ERROR with parameters : Both 'R_file' and 'U_file' specified.\n");exit(2);}
  if(Rmatrix_file_found && R_file_found){
    fprintf(stderr,"ERROR with parameters : Both 'R_file' and 'Rmatrix_file' specified.\n");exit(2);}
  if(D_file_found && Rmatrix_file_found){
    fprintf(stderr,"ERROR with parameters : Both 'Rmatrix_file' and 'D_file' specified.\n");exit(2);}
  if(D_file_found && R_file_found){
    fprintf(stderr,"ERROR with parameters : Both 'R_file' and 'D_file' specified.\n");exit(2);}
  if(U_file_found && !D_file_found){
    fprintf(stderr,"ERROR with parameters : Did not find label 'D_file' even though 'U_file' is specified.\n");exit(2);}
  if(!U_file_found && D_file_found){
    fprintf(stderr,"ERROR with parameters : Did not find label 'U_file' even though 'D_file' is specified.\n");exit(2);}
  
  if(U_file_found) par->read_R=0;
  if(R_file_found) par->read_R=1;
  if(Rmatrix_file_found) par->read_R=2;
  if(E_file_found)
    par->read_E=1;
  else
    par->read_E=0;
  if(!inds_out_file_found) {
    if((par->inds_out_file=(char *) malloc(20*sizeof(char)))==NULL) alloc_error("par->inds_out_file");
    sprintf(par->inds_out_file,"MMM_inds.out");
  }
  if(par->read_R){
    str_size=strlen(par->R_file)+10;
    if((par->U_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->U_file");
    sprintf(par->U_file,"%s_U.out",par->R_file);
    if((par->D_file=(char *) malloc(str_size*sizeof(char)))==NULL) alloc_error("par->D_file");
    sprintf(par->D_file,"%s_D.out",par->R_file);
  }
  if(par->model_missing && !par->impute_missing && par->read_R==0){
    fprintf(stderr,"ERROR with parameters : Must impute missing data when 'U-file' is used.\n");exit(2);}
  
  //here read_stdin is 1 if parameters file was read from stdin, otherwise 0
  read_stdin+=(par->read_R && (par->R_file[0]=='-' && par->R_file[1]=='\0'));
  read_stdin+=(par->A_file[0]=='-' && par->A_file[1]=='\0');
  read_stdin+=(E_file_found && (par->E_file[0]=='-' && par->E_file[1]=='\0'));
  read_stdin+=(!(par->read_R) && (par->U_file[0]=='-' && par->U_file[1]=='\0'));
  read_stdin+=(!(par->read_R) && (par->D_file[0]=='-' && par->D_file[1]=='\0'));
  read_stdin+=(prior_file_found && (par->prior_file[0]=='-' && par->prior_file[1]=='\0'));
  read_stdin+=((par->read_z||par->read_gen) && (par->z_file[0]=='-' && par->z_file[1]=='\0'));


  printf("A_file='%s'\n",par->A_file);
  printf("outcome='%s'\n",par->outcome);
  printf("n_groups=%d\nn_covariates=%d\n",par->n_groups,par->n_covariates);
  if(par->n_covariates>0) {
    printf("covariates=");
    for(i=0;i<par->n_covariates;++i)
      printf(" '%s'",par->covariates[i]);
    printf("\n");
  }
  if(par->read_z) printf("z_file='%s'\n",par->z_file);
  if(par->read_gen) printf("gen_file='%s'\n",par->z_file);
  if(par->read_R==1) printf("R_file='%s', save_U_D=%d\n",par->R_file,par->save_U_D);
  if(par->read_R==2) printf("Rmatrix_file='%s', save_U_D=%d\n",par->R_file,par->save_U_D);
  if(par->read_R==0) printf("U_file='%s'\nD_file='%s'\n",par->U_file,par->D_file);
  if(par->read_E) printf("E_file='%s', id='%s'\n",par->E_file,par->id);
  printf("inds_out_file='%s'\n",par->inds_out_file);
  printf("results_file='%s'\n",par->results_file);

  if(read_stdin>1)
    {fprintf(stderr,"\n\nERROR: Attempting to read several input files from stdin.\nExits.\n\n");exit(2);}  

  if(par->read_z && par->model_missing) printf("Models missing data in z-file, missingness labelled by '%g'\n",par->missing_z);
  if(par->read_z && !par->model_missing) printf("Does not model missing data in z-file.\n");
  if(par->read_gen) 
    printf("Genotype declared missing if non NULL genotype probability<%g\n",par->missing_gen);
  if(par->model_missing) printf("impute_missing=%d\n",par->impute_missing);
  printf("bayesian=%d\n",par->bayesian);
  if(par->bayesian){printf("n_integr_intervals=%d\nprior_file='%s'\n",
			   par->n_integr_intervals,par->prior_file);}
  if(par->fix_eta) printf("fixed_eta=%g\n",par->fixed_eta);
  printf("logOR=%d\nmean_center=%d\n",par->logOR,par->mean_center);
  if(par->read_z) printf("check_geno=%d\n",par->check_geno);
  printf("max_iterations=%d\nmax_eta_iterations=%d\n",par->max_iterations,par->max_eta_iterations);
  printf("max_eta_intervals=%d\nmax_eta_divisions=%d\ntolerance=%g\nmin_d=%g\n",
	 par->max_eta_intervals,par->max_eta_divisions,par->tol,par->min_d);

  if(par->logOR>0 && par->mean_center==0){
    printf("\n\n\nWARNING: effects are on the logOR scale but mean_center==0. Estimates are inaccurate!\n");
    printf("Why are you not using mean_center==1? You probably should!\n\n\n"); 
  }
  free(line);
  fclose(in); //also close if this is stdin?
}


char **read_E(FILE *in,int *n){

  char **list;
  int i,list_size=100;

  *n=0;
  if((list=(char**) malloc((list_size)*sizeof(char*)))==NULL)
    alloc_error("list in read_E");
  
  while(!feof(in)){
    if(*n==list_size){
      if((list=(char**) realloc(list,2*(list_size)*sizeof(char*)))==NULL)
	alloc_error("list in read_E");
      list_size *= 2;
    }
    if(fscanf(in,"%as",&(list[*n]))!=1)
      break;
    else
      (*n)++;
  }

  if(*n>0){
    if((list=(char**) realloc(list,(*n)*sizeof(char*)))==NULL)
      alloc_error("list in read_E");
    qsort(list, *n, sizeof(char*),cmpstringp);
  }
  return list;

}

int cmpstringp(const void *p1, const void *p2){
  //taken from 'man qsort'
  /* The actual arguments to this function are "pointers to
     pointers to char", but strcmp(3) arguments are "pointers
     to char", hence the following cast plus dereference */
  return strcmp(* (char * const *) p1, * (char * const *) p2);
}


void read_priors(FILE *in,gsl_matrix *inv_prior_var,double *logdet_prior_ratio_1to0,int ncovariates,PAR *par){

  /* 
     Prior distribution for parameters (beta,sigma^2) is NIG(m,V,a,b).
     I.e. Normal-inverse-gamma distribution with 
     mean-vector m=prior_exp
     variance-matrix V=prior_var
     shape parameter a=prior_d/2
     scale parameter b=prior_a/2

     Prior for eta is Beta(beta_1,beta_0).


     Note that the actual computations in MMM are written in terms of 
     prior_a and prior_d that refer to the NIG parametrisation used in O'hagan&Forster (2004).
     However, these parameters are read in as parameters a and b from the 
     more common parametrisation of INV-GAMMA distribution (see wikipedia).

     To choose the values for the NIG prior using R you may use the following code.
     If quantitative phenotype is used, then look at the distributions of beta.
     If binary phenotype is used, then look at the distribution of logodds.
     
     For example: A Case-control study where 0.44 of individuals are cases.
     Goal: To find parameters such that prior on effect on log-odds scale is approx. N(0,0.2^2)

     library(mvtnorm)
     m=c(0.44,0)
     V=diag(c(0.007,0.012))
     b=0.8
     a=5

     #sigma^(-2) has gamma(shape=a,rate=b)
     #mode of sigma^2
     b/(a+1)
     #mean of sigma^2
     b/(a-1)
     #var of sigma^2
     b^2/(a-1)^2/(a-2)

     n=50000
     sigma2=1/rgamma(n,shape=a,rate=b)
     beta=matrix(0,ncol=2,nrow=n)
     for(i in 1:n){beta[i,]=rmvnorm(1,m,sigma2[i]*V)}#sample values for intercept and slope

     par(mfrow=c(2,2))
     hist(beta[,1],breaks=100,main="intercept",xlab="")
     hist(beta[,2],breaks=100,main="slope",xlab="")

     logodds=beta[,2]/beta[,1]/(1-beta[,1])
     hist(logodds,breaks=100,main=paste("log-odds (mu=",signif(mean(logodds),3),"sd=",signif(sd(logodds),3),")"))
     hist(sqrt(sigma2),breaks=100,main="sigma",xlab="")

  */
  int i,j,nchars,tag_ok,ncols;
  char tag[TAG_SIZE],target[TAG_SIZE],prior_expectation_found=0,prior_variance_found=0,prior_a_found=0,prior_b_found=0,prior_beta_found=0,found,*line;
  double t,a,b;
  FILE *line_str;
  size_t line_size=100;

  if((line=(char*) malloc(line_size*sizeof(char)))==NULL)
    alloc_error("line in read_priors");
  nchars=0;
  
  ncols=ncovariates+(par->read_z + par->read_gen);

  while(!feof(in))
    {
      tag_ok=0;
      while(!tag_ok && !feof(in)){
	if((nchars=getline(&line,&line_size,in))>0){
	  line_str=fmemopen(line,nchars*sizeof(char),"r");
	  if(fscanf(line_str,TAG_S,tag)==1)
	    tag_ok=1;
	  else
	    fclose(line_str);
	}
      }
      if(feof(in) && !tag_ok) break;
      found=0;
      
      sprintf(target,"prior_expectation");
      if(strcmp(tag,target)==0)
	{
	  if(prior_expectation_found)
	    {fprintf(stderr,"\nERROR with prior file: prior_expectation was found several times.\n");exit(2);}
	  for(i=0;i<ncols;++i){
	    if(fscanf(line_str,"%lf",&t)!=1)
	      {fprintf(stderr,"\nERROR with prior file: Can't read element %d of prior_expectation.\n",i+1);exit(2);};
	    gsl_vector_set(par->prior_exp,i,t);
	  }
	  found=1;prior_expectation_found=1;
	}
      
      sprintf(target,"prior_variance");
      if(strcmp(tag,target)==0)
	{
	  if(prior_variance_found)
	    {fprintf(stderr,"\nERROR with prior file: prior_variance or prior_variance_matrix was found several times.\n");exit(2);}
	  for(i=0;i<ncols;++i){
	    if(fscanf(line_str,"%lf",&t)!=1 || t<0.0)
	      {fprintf(stderr,"\nERROR with prior file: Element %d of prior_variance invalid.\n",i+1);exit(2);}
	    gsl_matrix_set(par->prior_var,i,i,t);
	  }
	  found=1;prior_variance_found=1;
	}
      
      sprintf(target,"prior_variance_matrix");
      if(strcmp(tag,target)==0){
	if(prior_variance_found)
	  {fprintf(stderr,"\nERROR with prior file: prior_variance or prior_variance_matrix was found several times.\n");exit(2);}
	fclose(line_str);
	for(i=0;i<ncols;++i){
	  if((nchars=getline(&line,&line_size,in))>0){
	    line_str=fmemopen(line,nchars*sizeof(char),"r");
	    for(j=0;j<ncols;++j){
	      if(fscanf(line_str,"%lf",&t)!=1){
		fprintf(stderr,"\nERROR with prior file: Can't read element (%d,%d) of prior_variance_matrix.\n",i+1,j+1);
		exit(2);
	      }
	      gsl_matrix_set(par->prior_var,i,j,t);
	    }
	    if(i<ncols-1) fclose(line_str);
	  }
	  else{fprintf(stderr,"\nERROR with prior_variance_matrix: Could not read row %d.\n",i+1);exit(2);}
	}
	found=1;prior_variance_found=1;
      }
      
      sprintf(target,"prior_a");
      if(strcmp(tag,target)==0){
	if(prior_a_found)
	  {fprintf(stderr,"\nERROR with prior file: prior_a was found several times.\n");exit(2);}
	if((fscanf(line_str,"%lf",&a)!=1) || a< 0.0)
	  {fprintf(stderr,"\nERROR with prior file: Value %g for prior_a is invalid.\n",a);exit(2);}
	found=1;prior_a_found=1;
	par->prior_d=2*a;//in O'hagan parametrisation
      }

      sprintf(target,"prior_b");
      if(strcmp(tag,target)==0){
	if(prior_b_found)
	  {fprintf(stderr,"\nERROR with prior file: prior_b was found several times.\n");exit(2);}
	if(fscanf(line_str," %lf ",&b)!=1 || b<0.0)
	  {fprintf(stderr,"\nERROR with prior file: Value %g for prior_b is invalid.\n",b);exit(2);}
	found=1;prior_b_found=1;
	par->prior_a=2*b;//in O'hagan parametrisation
      }

      sprintf(target,"prior_beta");
      if(strcmp(tag,target)==0){
	if(prior_beta_found)
	  {fprintf(stderr,"\nERROR with prior file: prior_beta was found several times.\n");exit(2);}
	if(fscanf(line_str," %lf %lf",&a,&b)!=2 || a<1.0 || b<1.0)
	  {fprintf(stderr,"\nERROR with prior file: Values for prior_beta should be >= 1.0. Now they are %g %g.\n",a,b);exit(2);}
	found=1;prior_beta_found=1;
	par->prior_beta_1=a;
	par->prior_beta_0=b;
      }

      if(found==0)
	{fprintf(stderr,"\nERROR with prior file: unexpected label '%s' found\n\n",tag);exit(2);}
      fclose(line_str);
    }

  if(!prior_expectation_found){fprintf(stderr,"\nERROR with prior file: Did not find label 'prior_expectation'.\n");exit(2);} 
  if(!prior_variance_found){fprintf(stderr,"\nERROR with prior file: Did not find either of labels 'prior_variance' or 'prior_variance_matrix'.\n");exit(2);} 
  if(!prior_a_found){fprintf(stderr,"\nERROR with prior file: Did not find label 'prior_a'.\n");exit(2);} 
  if(!prior_b_found){fprintf(stderr,"\nERROR with prior file: Did not find label 'prior_b'.\n");exit(2);} 
  if(!prior_beta_found){par->prior_beta_0=1.0;par->prior_beta_1=1.0;} 
 

  printf("\nUsing priors:\nprior expectation:\n");
  for(i=0;i<ncols;++i)
    printf("%g ",gsl_vector_get(par->prior_exp,i));
  printf("\n");
  printf("prior variance:\n");
  for(i=0;i<ncols;++i){
    for(j=0;j<ncols;++j)
      printf("%g ",gsl_matrix_get(par->prior_var,i,j));
    printf("\n");
  }
  printf("prior_a=%g\n",(par->prior_d)/2.0);
  printf("prior_b=%g\n",(par->prior_a)/2.0);
  printf("prior_beta_1=%g\nprior_beta_0=%g\n",par->prior_beta_1,par->prior_beta_0); 

  if(ncols<ncovariates+1) gsl_matrix_set(par->prior_var,ncovariates,ncovariates,1.0);//this element is independent of the other matrix elements,
  //so doesn't affect inversions of the other part of the matrix

  gsl_matrix_memcpy (inv_prior_var,par->prior_var);
  gsl_set_error_handler_off ();//Check return value of cholesky manually
  if(gsl_linalg_cholesky_decomp (inv_prior_var)==GSL_EDOM){// not positive semi-definite
    fprintf(stderr,"\n\nERROR: Prior variance matrix is not positive definite.\nExits!\n");
    exit(1);
  }
  gsl_set_error_handler (NULL);//Return the default error handler
  *logdet_prior_ratio_1to0=0.0;
  for(i=0;i<ncols;++i)
    (*logdet_prior_ratio_1to0)+=2.0*log(gsl_matrix_get(inv_prior_var,i,i));//log(det(prior_var))
  
  gsl_linalg_cholesky_invert (inv_prior_var);
  
  free(line);

}


FILE* open_file_or_stdin(char *name,int *reading_stdin){

  FILE *in;
  
  *reading_stdin=1;
  if(name[0]=='-' && name[1]=='\0') in=stdin;
  else{
    if((in=fopen(name,"r"))==NULL){
      fprintf(stderr,"Cannot open file %s\nExits.\n",name);
      exit(1);
    }
    *reading_stdin=0;
  }
  return in;
}


void alloc_error(char *str)
{
  fprintf(stderr,"Can't allocate memory: %s\nExits!\n",str);
  exit(2);
}



#if SCORE_TEST_ETA==1

///
/// To compute p-value for score test of eta==0
/// Using Davies' method from CompQuadPack by Pierre Lafaye De Micheaux 
/// His code is in the file qfc.c
///  

double davies_cdf(double x,gsl_vector *d){

  double *nc,*d1,res,acc=1e-14,sigma=0.0;
  double trace[7]={};
  int i,*n,r,ifault,lim=100000;

  r=d->size;
  if((nc=(double*)calloc(r,sizeof(double)))==NULL)
    alloc_error("nc in davies_cdf");
  if((n=(int*)malloc(r*sizeof(int)))==NULL)
    alloc_error("nc in davies_cdf");
  if((d1=(double*)malloc(r*sizeof(double)))==NULL)
    alloc_error("d1 in davies_cdf");
  for(i=0;i<r;++i) {
    n[i]=1;
    d1[i]=gsl_vector_get(d,i)-1;
  }

  qfc(d1,nc,n,&r,&sigma,&x,&lim,&acc,trace,&ifault,&res);
  free(nc);free(n);free(d1);

  if(ifault==0) return (1-res);
  else return -1.0;
      

}

#endif
